  /******************************************************************************
  
   ARXIU    : 1_2_CreacioTaules.sql
   AUTOR    : Jess Hernndez Garcia - TFC - Aula 1
   
   PROPSIT : Script de creaci de les taules de la base de dades
                                
   ******************************************************************************/

CREATE TABLE TFC_BD.PAIS
(
  paiId       NUMBER            NOT NULL,
  Codi        VARCHAR2(3)       NOT NULL,
  Nom         VARCHAR2(100),
 
  CONSTRAINT PK_PAIS PRIMARY KEY (paiId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT UK_PAIS UNIQUE (Codi) USING INDEX TABLESPACE TS_TFCBD_IDX
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.ESPECIALITAT
(
  Codi        VARCHAR2(5)       NOT NULL,
  Descripcio  VARCHAR2(100),
 
  CONSTRAINT PK_ESPECIALITAT PRIMARY KEY (Codi) USING INDEX TABLESPACE TS_TFCBD_IDX
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.CIUTADA
(
  ciuId              NUMBER                         NOT NULL,
  Nom                VARCHAR2(50)                   NOT NULL,
  Cognoms            VARCHAR2(100)                  NOT NULL,
  DocumentIdent      VARCHAR2(15)                   NOT NULL,
  PaisId             NUMBER                         NOT NULL, 
  DataNaixement      DATE                           NOT NULL,
  CorreuElectronic   VARCHAR2(200),
  Sexe               VARCHAR2(10)                   NOT NULL,
  Borrat             VARCHAR2(1)     DEFAULT 'N'    NOT NULL,
 
  CONSTRAINT PK_CIUTADA PRIMARY KEY (ciuId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT UK_CIUTADA UNIQUE (DocumentIdent) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT FK_CIUTADA_PAIS FOREIGN KEY (PaisId) REFERENCES TFC_BD.PAIS(paiId),
  CONSTRAINT CK_CIUTADA_SEXE CHECK (Sexe IN('MASCULI', 'FEMENI')),
  CONSTRAINT CK_CIUTADA_BORRAT CHECK (Borrat IN('N', 'S'))
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.USUARI
(
  usuId                    NUMBER                  NOT NULL,
  NumAfiliacioSanitaria    VARCHAR2(15)            NOT NULL,
  UltimAnyActivitat        NUMBER(4)    DEFAULT 0  NOT NULL,
  
  CONSTRAINT PK_USUARI PRIMARY KEY (usuId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT UK_USUARI UNIQUE (NumAfiliacioSanitaria) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT FK_USUARI_CIUTADA FOREIGN KEY (usuId) REFERENCES TFC_BD.CIUTADA(ciuId)
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.METGE
(
  metId             NUMBER         NOT NULL,
  EspecialitatCodi  VARCHAR2(5)    NOT NULL,
  NumColegiat       VARCHAR2(15)   NOT NULL,
  
  CONSTRAINT PK_METGE PRIMARY KEY (metId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT UK_METGE UNIQUE (NumColegiat) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT FK_METGE_CIUTADA FOREIGN KEY (metId) REFERENCES TFC_BD.CIUTADA(ciuId),
  CONSTRAINT FK_METGE_ESPECIALITAT FOREIGN KEY (EspecialitatCodi) REFERENCES TFC_BD.ESPECIALITAT(Codi)
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.CENTRE_SANITARI
(
  cesId              NUMBER         NOT NULL,
  Nom                VARCHAR2(100)  NOT NULL,
  Adreca             VARCHAR2(100),
  Poblacio           VARCHAR2(100),
  PaisId             NUMBER         NOT NULL, 
  
  CONSTRAINT PK_CENTRE_SANITARI PRIMARY KEY (cesId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT FK_CENTRE_SANITARI_PAIS FOREIGN KEY (PaisId) REFERENCES TFC_BD.PAIS(paiId)
  
)
TABLESPACE TS_TFCBD_TAU;

-- Index del centre per nom per accelerar les cerques
CREATE INDEX TFC_BD.IX_CENTRE_SANITARI_NOM ON TFC_BD.CENTRE_SANITARI (Nom ASC) 
  TABLESPACE TS_TFCBD_IDX;


CREATE TABLE TFC_BD.VISITA
(
  visId              NUMBER         NOT NULL,
  CentreSanitariId   NUMBER         NOT NULL, 
  MetgeId            NUMBER         NOT NULL, 
  UsuariId           NUMBER         NOT NULL, 
  DataHora           TIMESTAMP      NOT NULL,
  NumConsulta        VARCHAR2(6)    NOT NULL,
  Comentari          VARCHAR2(500),
  
  CONSTRAINT PK_VISITA PRIMARY KEY (visId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT FK_VISITA_CENTRE_SANITARI FOREIGN KEY (CentreSanitariId) REFERENCES TFC_BD.CENTRE_SANITARI(cesId),
  CONSTRAINT FK_VISITA_METGE FOREIGN KEY (MetgeId) REFERENCES TFC_BD.METGE(metId),
  CONSTRAINT FK_VISITA_USUARI FOREIGN KEY (UsuariId) REFERENCES TFC_BD.USUARI(usuId)
  
)
TABLESPACE TS_TFCBD_TAU;

-- Index de clau nica per evitar repetici de visites
CREATE UNIQUE INDEX TFC_BD.IX_VISITA_UNICA ON TFC_BD.VISITA (CentreSanitariId, MetgeId, UsuariId, DataHora) 
  TABLESPACE TS_TFCBD_IDX;

CREATE TABLE TFC_BD.MALALTIA
(
  malId              NUMBER         NOT NULL,
  EspecialitatCodi   VARCHAR2(5)    NOT NULL,
  Descripcio         VARCHAR2(100)  NOT NULL,
  NivellGravetat     VARCHAR2(20)   NOT NULL, 
  
  CONSTRAINT PK_MALALTIA PRIMARY KEY (malId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT FK_MALALTIA_ESPECIALITAT FOREIGN KEY (EspecialitatCodi) REFERENCES TFC_BD.ESPECIALITAT(Codi),
  CONSTRAINT CK_MALALTIA_NIVELLGRAVETAT CHECK (NivellGravetat IN('Intranscendent', 'Lleu', 'Mitj', 'Greu', 'Mortal'))
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.DIAGNOSTIC
(
  diaId              NUMBER         NOT NULL,
  VisitaId           NUMBER         NOT NULL,
  MalaltiaId         NUMBER         NOT NULL,
  Comentari          VARCHAR2(500),
  
  CONSTRAINT PK_DIAGNOSTIC PRIMARY KEY (diaId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT FK_DIAGNOSTIC_VISITA FOREIGN KEY (VisitaId) REFERENCES TFC_BD.VISITA(visId),
  CONSTRAINT FK_DIAGNOSTIC_MALALTIA FOREIGN KEY (MalaltiaId) REFERENCES TFC_BD.MALALTIA(malId)
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.ACTIVITAT
(
  actId                NUMBER                 NOT NULL,
  UsuariId             NUMBER                 NOT NULL, 
  DataAct              DATE                   NOT NULL,
  NombrePasses         NUMBER(6)  DEFAULT  0  NOT NULL,
  NombreCalories       NUMBER(5)  DEFAULT  0  NOT NULL,
  NombreHoresInactives NUMBER(2)  DEFAULT 24  NOT NULL,
  
  CONSTRAINT PK_ACTIVITAT PRIMARY KEY (actId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT FK_ACTIVITAT_USUARI FOREIGN KEY (UsuariId) REFERENCES TFC_BD.USUARI(usuId),
  CONSTRAINT CK_ACTIVITAT_HINACT CHECK (NombreHoresInactives <= 24)
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.LOG_S
(
  logId            NUMBER         NOT NULL,
  NomUsuari        VARCHAR2(12)   NOT NULL,
  NomProcediment   VARCHAR2(60)   NOT NULL,
  ParEntrada       VARCHAR2(400)  NOT NULL,
  ParSortida       VARCHAR2(400)  NOT NULL,
  Resultat         VARCHAR2(300)  NOT NULL,
  DataHora         TIMESTAMP      NOT NULL,
  
  CONSTRAINT PK_LOG_S PRIMARY KEY (logId) USING INDEX TABLESPACE TS_TFCBD_IDX
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E1_USUSISTEMA
(
  estId       NUMBER         NOT NULL,
  PaisId      NUMBER         NOT NULL, 
  NumAny      NUMBER(4)      NOT NULL,
  Quantitat   NUMBER         NOT NULL,
  
  CONSTRAINT PK_E1_USUSISTEMA PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT FK_E1_USUSISTEMA_PAIS FOREIGN KEY (PaisId) REFERENCES TFC_BD.PAIS(paiId)
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E2_CALORIESUSU
(
  estId       NUMBER         NOT NULL,
  PaisId      NUMBER         NOT NULL, 
  NumAny      NUMBER(4)      NOT NULL,
  Quantitat   NUMBER         NOT NULL,
  
  CONSTRAINT PK_E2_CALORIESUSU PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT FK_E2_CALORIESUSU_PAIS FOREIGN KEY (PaisId) REFERENCES TFC_BD.PAIS(paiId)
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E3_PASSESUSU
(
  estId       NUMBER         NOT NULL,
  PaisId      NUMBER         NOT NULL, 
  NumAny      NUMBER(4)      NOT NULL,
  Quantitat   NUMBER         NOT NULL,
  
  CONSTRAINT PK_E3_PASSESUSU PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT FK_E3_PASSESUSU_PAIS FOREIGN KEY (PaisId) REFERENCES TFC_BD.PAIS(paiId)
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E4_HORESPOCAACTUSU
(
  estId       NUMBER         NOT NULL,
  PaisId      NUMBER         NOT NULL, 
  NumAny      NUMBER(4)      NOT NULL,
  Quantitat   NUMBER         NOT NULL,
  
  CONSTRAINT PK_E4_HORESPOCAACTUSU PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT FK_E4_HORESPOCAACTUSU_PAIS FOREIGN KEY (PaisId) REFERENCES TFC_BD.PAIS(paiId)
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E5_USUCARDIACA
(
  estId       NUMBER         NOT NULL,
  PaisId      NUMBER         NOT NULL, 
  NumAny      NUMBER(4)      NOT NULL,
  Quantitat   NUMBER         NOT NULL,
  
  CONSTRAINT PK_E5_USUCARDIACA PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT FK_E5_USUCARDIACA_PAIS FOREIGN KEY (PaisId) REFERENCES TFC_BD.PAIS(paiId)
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E6_USUCANCERIGENA
(
  estId       NUMBER         NOT NULL,
  PaisId      NUMBER         NOT NULL, 
  NumAny      NUMBER(4)      NOT NULL,
  Quantitat   NUMBER         NOT NULL,
  
  CONSTRAINT PK_E6_USUCANCERIGENA PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT FK_E6_USUCANCERIGENA_PAIS FOREIGN KEY (PaisId) REFERENCES TFC_BD.PAIS(paiId)
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E7_USUDEGENERATIVA
(
  estId       NUMBER         NOT NULL,
  PaisId      NUMBER         NOT NULL, 
  NumAny      NUMBER(4)      NOT NULL,
  Quantitat   NUMBER         NOT NULL,
  
  CONSTRAINT PK_E7_USUDEGENERATIVA PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT FK_E7_USUDEGENERATIVA_PAIS FOREIGN KEY (PaisId) REFERENCES TFC_BD.PAIS(paiId)
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E8_MALALTIESLLEUS
(
  estId       NUMBER         NOT NULL,
  PaisId      NUMBER         NOT NULL, 
  NumAny      NUMBER(4)      NOT NULL,
  Quantitat   NUMBER         NOT NULL,
  
  CONSTRAINT PK_E8_MALALTIESLLEUS PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT FK_E8_MALALTIESLLEUS_PAIS FOREIGN KEY (PaisId) REFERENCES TFC_BD.PAIS(paiId)
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E9_USUMESCALORIES
(
  estId       NUMBER         NOT NULL,
  PaisId      NUMBER         NOT NULL, 
  UsuariId    NUMBER         NOT NULL, 
  NumAny      NUMBER(4)      NOT NULL,
  
  CONSTRAINT PK_E8_USUMESCALORIES PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX,
  CONSTRAINT FK_E8_USUMESCALORIES_PAIS FOREIGN KEY (PaisId) REFERENCES TFC_BD.PAIS(paiId)
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E1C_USUSISTEMA
(
  estId       NUMBER         NOT NULL,
  NumAny      NUMBER(4)      NOT NULL,
  Quantitat   NUMBER         NOT NULL,
  
  CONSTRAINT PK_E1C_USUSISTEMA PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E2C_CALORIESUSU
(
  estId       NUMBER         NOT NULL,
  NumAny      NUMBER(4)      NOT NULL,
  Quantitat   NUMBER         NOT NULL,
  
  CONSTRAINT PK_E2C_CALORIESUSU PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E3C_PASSESUSU
(
  estId       NUMBER         NOT NULL,
  NumAny      NUMBER(4)      NOT NULL,
  Quantitat   NUMBER         NOT NULL,
  
  CONSTRAINT PK_E3C_PASSESUSU PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E4C_HORESPOCAACTUSU
(
  estId       NUMBER         NOT NULL,
  NumAny      NUMBER(4)      NOT NULL,
  Quantitat   NUMBER         NOT NULL,
  
  CONSTRAINT PK_E4C_HORESPOCAACTUSU PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E5C_USUCARDIACA
(
  estId       NUMBER         NOT NULL,
  NumAny      NUMBER(4)      NOT NULL,
  Quantitat   NUMBER         NOT NULL,
  
  CONSTRAINT PK_E5C_USUCARDIACA PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E6C_USUCANCERIGENA
(
  estId       NUMBER         NOT NULL,
  NumAny      NUMBER(4)      NOT NULL,
  Quantitat   NUMBER         NOT NULL,
  
  CONSTRAINT PK_E6C_USUCANCERIGENA PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E7C_USUDEGENERATIVA
(
  estId       NUMBER         NOT NULL,
  NumAny      NUMBER(4)      NOT NULL,
  Quantitat   NUMBER         NOT NULL,
  
  CONSTRAINT PK_E7C_USUDEGENERATIVA PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E8C_MALALTIESLLEUS
(
  estId       NUMBER         NOT NULL,
  NumAny      NUMBER(4)      NOT NULL,
  Quantitat   NUMBER         NOT NULL,
  
  CONSTRAINT PK_E8C_MALALTIESLLEUS PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX
  
)
TABLESPACE TS_TFCBD_TAU;


CREATE TABLE TFC_BD.E9C_USUMESCALORIES
(
  estId       NUMBER         NOT NULL,
  UsuariId    NUMBER         NOT NULL, 
  NumAny      NUMBER(4)      NOT NULL,
  
  CONSTRAINT PK_E8C_USUMESCALORIES PRIMARY KEY (estId) USING INDEX TABLESPACE TS_TFCBD_IDX
  
)
TABLESPACE TS_TFCBD_TAU;

COMMIT;
