  /******************************************************************************
  
   ARXIU    : 2_01_Utilitats.sql
   AUTOR    : Jess Hernndez Garcia - TFC - Aula 1
   
   PROPSIT : Script de creaci del paquet PQ_UTILITATS : 
              Procediments i funcions comunes de l'aplicatiu.
                                
   ******************************************************************************/

CREATE OR REPLACE PACKAGE TFC_BD.PQ_UTILITATS AS

  /*  Alta al registre d'activitat de l'aplicatiu */
  PROCEDURE PRO_ALTALOG (
    pNomUsuari      IN LOG_S.NomUsuari%TYPE,
    pNomProcediment IN LOG_S.NomProcediment%TYPE,
    pParEntrada     IN LOG_S.ParEntrada%TYPE,
    pParSortida     IN LOG_S.ParSortida%TYPE,
    pResultat       IN LOG_S.Resultat%TYPE,
    pDataHora       IN LOG_S.DataHora%TYPE
	);

  /* Funci que donat un codi de Pas en retorna el seu Identificador */
  FUNCTION DONA_PAIS_ID (
    pPais IN PAIS.Codi%TYPE
  ) RETURN NUMBER;

  /* Funci que donat un nmero d'afiliaci Sanitria d'un Usuari en retorna el seu Identificador */
  FUNCTION DONA_USUARI_ID (
    pNumAfiliacioSanitaria IN USUARI.NumAfiliacioSanitaria%TYPE
  ) RETURN NUMBER;

  /* Funci que donat un Nmero de Collegiat d'un Metge en retorna el seu Identificador */
  FUNCTION DONA_METGE_ID (
    pNumColegiat IN METGE.NumColegiat%TYPE
  ) RETURN NUMBER;
  
  /* Mostrar per consola la capalera d'una consulta */
  PROCEDURE CAPCALERA_CONSULTA(
    pTitol      IN VARCHAR2,
    pParametres IN VARCHAR2
  );
  

END PQ_UTILITATS;
/

CREATE OR REPLACE PACKAGE BODY TFC_BD.PQ_UTILITATS IS

    -- Variables de treball per a guardar moviments al registre d'activitat LOG_S
    l_NomUsuari        VARCHAR2(12);
    l_NomProcediment   VARCHAR2(60);
    l_ParEntrada       VARCHAR2(400);
    l_ParSortida       VARCHAR2(400);
    l_Resultat         VARCHAR2(300);
    l_DataHora         DATE;
 
  /******************************************************************************
   NOM         : PRO_ALTALOG
   
   PROPSIT    : Fa una alta al registre d'activitat de l'aplicatiu.
                 s capa d'autoverificar-se.
                                
   PARMETRES  :
      ENTRADA  :  pNomUsuari      : Usuari que ha executat el procediment
                  pNomProcediment : Nom del procediment
                  pParEntrada     : Parmetres d'entrada del procediment
                  pParSortida     : Parmetres de sortida del procediment
                  pResultat       : Resultat de l'execuci del procediment
                  pDataHora       : Data i hora d'execuci del procediment
      SORTIDA  :  ---                    
		
   PRECONDICI : --
   
   POSTCONDICI: S'ha creat una linia del registre d'activitat
   
   RETORNA     : ------ 
   
   ******************************************************************************/
	
  PROCEDURE PRO_ALTALOG (

    pNomUsuari      IN LOG_S.NomUsuari%TYPE,
    pNomProcediment IN LOG_S.NomProcediment%TYPE,
    pParEntrada     IN LOG_S.ParEntrada%TYPE,
    pParSortida     IN LOG_S.ParSortida%TYPE,
    pResultat       IN LOG_S.Resultat%TYPE,
    pDataHora       IN LOG_S.DataHora%TYPE
	
  ) IS
  
    -- Marcar el procediment com autnom a nivell de transaccions
    PRAGMA AUTONOMOUS_TRANSACTION;

    -- variables pel tractament d'excepcions
    eSenseNomUsuari     EXCEPTION;
  	eSenseProcediment   EXCEPTION;
    eSenseResultat      EXCEPTION;
    eSenseDataHora      EXCEPTION;

  BEGIN
    -- Inicialitzacions
    l_NomUsuari      := NVL(pNomUsuari,USER);
    l_NomProcediment := NVL(pNomProcediment,'SENSE NOM DEL PROCEDIMENT');
    l_ParEntrada     := NVL(pParEntrada,'SENSE PARMETRES D''ENTRADA');
    l_ParSortida     := NVL(pParSortida,'SENSE PARMETRES DE SORTIDA');
    l_DataHora       := NVL(pDataHora,SYSDATE);
  
    -- Mirar validesa dels parmetres d'entrada
    IF (pNomUsuari IS NULL OR pNomUsuari = '') THEN 
      RAISE eSenseNomUsuari; 
    END IF;
    
    IF (pNomProcediment IS NULL OR pNomProcediment = '') THEN 
      RAISE eSenseProcediment; 
    END IF;
    
    IF (pResultat IS NULL OR pResultat = '') THEN 
      RAISE eSenseResultat; 
    END IF;
    
    IF (pDataHora IS NULL OR pDataHora = '') THEN 
      RAISE eSenseDataHora; 
    END IF;
  
    INSERT INTO TFC_BD.LOG_S 
        (NomUsuari, NomProcediment, ParEntrada, ParSortida, Resultat, DataHora)
        VALUES (pNomUsuari, pNomProcediment, pParEntrada, pParSortida, pResultat, pDataHora);
      
    COMMIT;
    
  -- Tractament d'Excepcions
  EXCEPTION
    
    WHEN eSenseNomUsuari THEN
      l_Resultat := 'ERROR: NO S''HA ESPECIFICAT QUIN USUARI HA CRIDAT EL PROCEDIMENT';
      PRO_ALTALOG(l_NomUsuari, l_NomProcediment, l_ParEntrada, l_ParSortida, l_Resultat, l_DataHora);
      DBMS_OUTPUT.PUT_LINE(l_NomProcediment || ': ' || l_Resultat);
  
    WHEN eSenseProcediment THEN
      l_Resultat := 'ERROR: NO S''HA ESPECIFICAT EL NOM DEL PROCEDIMENT';
      PRO_ALTALOG(l_NomUsuari, l_NomProcediment, l_ParEntrada, l_ParSortida, l_Resultat, l_DataHora);
      DBMS_OUTPUT.PUT_LINE(l_NomProcediment || ': ' || l_Resultat);
  
    WHEN eSenseResultat THEN
      l_Resultat := 'ERROR: NO S''HA ESPECIFICAT QUIN HA ESTAT EL RESULTAT DEL PROCEDIMENT';
      PRO_ALTALOG(l_NomUsuari, l_NomProcediment, l_ParEntrada, l_ParSortida, l_Resultat, l_DataHora);
      DBMS_OUTPUT.PUT_LINE(l_NomProcediment || ': ' || l_Resultat);
  
    WHEN eSenseDataHora THEN
      l_Resultat := 'ERROR: NO S''HA ESPECIFICAT LA DATA I HORA EN QUE S''HA CRIDAT EL PROCEDIMENT';
      PRO_ALTALOG(l_NomUsuari, l_NomProcediment, l_ParEntrada, l_ParSortida, l_Resultat, l_DataHora);
      DBMS_OUTPUT.PUT_LINE(l_NomProcediment || ': ' || l_Resultat);
  
    WHEN OTHERS THEN
      l_Resultat := 'ERROR LOG: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      PRO_ALTALOG(l_NomUsuari, l_NomProcediment, l_ParEntrada, l_ParSortida, l_Resultat, l_DataHora);
      DBMS_OUTPUT.PUT_LINE(l_NomProcediment || ': ' || l_Resultat);

  END PRO_ALTALOG;
  
  
  /******************************************************************************
   NOM         : DONA_PAIS_ID
   
   PROPSIT    : Funci que donat un codi de Pas en retorna el seu Identificador.
                                
   PARMETRES  :
      ENTRADA  :  pPais      : Codi del pas a cercar
      SORTIDA  :  ---                    
		
   RETORNA     : SI es troba el pas, el seu identificador nic.
                 SI NO es troba, el valor -1. 
   
   ******************************************************************************/
  FUNCTION DONA_PAIS_ID (
    pPais IN PAIS.Codi%TYPE
  ) RETURN NUMBER AS
    
    vPaiId NUMBER;
    
  BEGIN
    
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'DONA_PAIS_ID';
    l_ParEntrada     := 'pPais: '||pPais;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA PAIS EFECTUADA';
    l_DataHora       := SYSDATE;
    
    SELECT paiId 
      INTO vPaiId 
      FROM TFC_BD.PAIS
     WHERE Codi = pPais; 
    
    -- Execuci correcta del procediment. Gravar registre d'activitat
    l_ParSortida := 'vPaiId: '||vPaiId;
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
      
    RETURN vPaiId;  
          
  EXCEPTION
    WHEN others THEN
      l_ParSortida := 'vPaiId: -1';
      l_Resultat   := 'ERROR: No s''ha trobat un Pas amb aquest Codi';
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
      RETURN -1;
    
  END DONA_PAIS_ID;
  
  
  /******************************************************************************
   NOM         : DONA_USUARI_ID
   
   PROPSIT    : Funci que donat un nmero d'afiliaci Sanitria d'un Usuari en 
                 retorna el seu Identificador
                                
   PARMETRES  :
      ENTRADA  :  pNumAfiliacioSanitaria : Nmero Afiliaci Sanitria de l'Usuari a 
                                           cercar 
      SORTIDA  :  ---                    
		
   RETORNA     : SI es troba l'Usuari, el seu identificador nic.
                 SI NO es troba, el valor -1. 
   
   ******************************************************************************/
  FUNCTION DONA_USUARI_ID (
    pNumAfiliacioSanitaria IN USUARI.NumAfiliacioSanitaria%TYPE
  ) RETURN NUMBER AS
    
    vUsuId NUMBER;
    
  BEGIN
    
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'DONA_USUARI_ID';
    l_ParEntrada     := 'pNumAfiliacioSanitaria: '||pNumAfiliacioSanitaria;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA USUARI EFECTUADA';
    l_DataHora       := SYSDATE;
    
    SELECT UsuId 
      INTO vUsuId 
      FROM TFC_BD.USUARI
     WHERE NumAfiliacioSanitaria = pNumAfiliacioSanitaria; 
    
    -- Execuci correcta del procediment. Gravar registre d'activitat
    l_ParSortida := 'vUsuId: '||vUsuId;
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
      
    RETURN vUsuId;  
          
  EXCEPTION
    WHEN others THEN
      l_ParSortida := 'vUsuId: -1';
      l_Resultat   := 'ERROR: No s''ha trobat un Usuari amb aquest Nmero d''Afiliaci';
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
      RETURN -1;
    
  END DONA_USUARI_ID;
  
  
  /******************************************************************************
   NOM         : DONA_METGE_ID
   
   PROPSIT    : Funci que donat un Nmero de Collegiat d'un Metge en retorna 
                 el seu Identificador
                                
   PARMETRES  :
      ENTRADA  :  pMETGE      : Codi del pas a cercar
      SORTIDA  :  ---                    
		
   RETORNA     : SI es troba el metge, el seu identificador nic.
                 SI NO es troba, el valor -1. 
   
   ******************************************************************************/
  FUNCTION DONA_METGE_ID (
    pNumColegiat IN METGE.NumColegiat%TYPE
  ) RETURN NUMBER AS
    
    vMetId NUMBER;
    
  BEGIN
    
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'DONA_METGE_ID';
    l_ParEntrada     := 'pNumColegiat: '||pNumColegiat;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA METGE EFECTUADA';
    l_DataHora       := SYSDATE;
    
    SELECT metId 
      INTO vMetId 
      FROM TFC_BD.METGE
     WHERE NumColegiat = pNumColegiat; 
    
    -- Execuci correcta del procediment. Gravar registre d'activitat
    l_ParSortida := 'vmetId: '||vMetId;
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
      
    RETURN vmetId;  
          
  EXCEPTION
    WHEN others THEN
      l_ParSortida := 'vMetId: -1';
      l_Resultat   := 'ERROR: No s''ha trobat un Metge amb aquest Nmero de Collegiat';
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
      RETURN -1;
    
  END DONA_METGE_ID;


  /******************************************************************************
   NOM         : CAPCALERA_CONSULTA
   
   PROPSIT    : Mostrar per consola la capalera d'una consulta
                                
   PARMETRES  :
      ENTRADA  :  pTitol      : Ttol de la Consulta
                  pParametres : Parmetres de la Consulta
                 
      SORTIDA  :  ---                    
    
   ******************************************************************************/
	
  PROCEDURE CAPCALERA_CONSULTA(
    pTitol      IN VARCHAR2,
    pParametres IN VARCHAR2
  ) IS

  BEGIN
  
    DBMS_OUTPUT.PUT_LINE('');
    DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
    DBMS_OUTPUT.PUT_LINE(RPAD(pTitol, 70) || 
                         LPAD(to_char(sysdate,'dd/mm/yy'), 10));
    DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
    DBMS_OUTPUT.PUT_LINE(pParametres);
    DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));         
    DBMS_OUTPUT.PUT_LINE('');
  
  END CAPCALERA_CONSULTA;

 
END;
/

COMMIT;  
