  /******************************************************************************
  
   ARXIU    : 2_02_UsuariABM.sql
   AUTOR    : Jess Hernndez Garcia - TFC - Aula 1
   
   PROPSIT : Script de creaci del paquet PQ_USUARI_ABM : 
              Procediments i funcions d'Altes, baixes i modificacions de la
			  taula USUARI.
                                
   ******************************************************************************/

CREATE OR REPLACE PACKAGE TFC_BD.PQ_USUARI_ABM AS

  /*  Alta d'un Usuari del Sistema de Recollida de Dades */
  PROCEDURE USUARI_A (
    pNom                    IN CIUTADA.Nom%TYPE,
    pCognoms                IN CIUTADA.Cognoms%TYPE,
    pDocumentIdent          IN CIUTADA.DocumentIdent%TYPE,
    pPais                   IN VARCHAR2,
    pDataNaixement          IN VARCHAR2,
    pCorreuElectronic       IN CIUTADA.CorreuElectronic%TYPE,
    pSexe                   IN CIUTADA.Sexe%TYPE,
    pNumAfiliacioSanitaria  IN USUARI.NumAfiliacioSanitaria%TYPE,
    oRSP                   OUT VARCHAR2
	);
	
  /*  Modificaci d'un Usuari del Sistema de Recollida de Dades */

  PROCEDURE USUARI_M (
    pNumAfiliacioSanitaria  IN USUARI.NumAfiliacioSanitaria%TYPE,
    pNom                    IN CIUTADA.Nom%TYPE,
    pCognoms                IN CIUTADA.Cognoms%TYPE,
    pDocumentIdent          IN CIUTADA.DocumentIdent%TYPE,
    pDataNaixement          IN VARCHAR2,
    pCorreuElectronic       IN CIUTADA.CorreuElectronic%TYPE,
    pSexe                   IN CIUTADA.Sexe%TYPE,
    oRSP                   OUT VARCHAR2
	);
	
  /*  Baixa d'un Usuari del Sistema de Recollida de Dades */

  PROCEDURE USUARI_B (
    pNumAfiliacioSanitaria  IN USUARI.NumAfiliacioSanitaria%TYPE,
    oRSP                   OUT VARCHAR2
	);


END PQ_USUARI_ABM;
/

CREATE OR REPLACE PACKAGE BODY TFC_BD.PQ_USUARI_ABM IS

    -- Variables de treball per a guardar moviments al registre d'activitat LOG_S
    l_NomUsuari        VARCHAR2(12);
    l_NomProcediment   VARCHAR2(60);
    l_ParEntrada       VARCHAR2(400);
    l_ParSortida       VARCHAR2(400);
    l_Resultat         VARCHAR2(300);
    l_DataHora         DATE;
    
  /******************************************************************************
   NOM         : USUARI_A
   
   PROPSIT    : Fa una alta d'un Usuari del Sistema de Recollida de Dades
                                
   PARMETRES  :
      ENTRADA  :  pNom                  : Nom de l'Usuari
                  pCognoms              : Cognoms de l'Usuari
                  pDocumentIdent        : Document d'identificaci de l'Usuari
                  pPais                 : Codi de Pas on resideix l'Usuari
                  pDataNaixement        : Data de Naixement de l'Usuari
                  pCorreuElectronic     : Correu Electrnic de contacte de l'Usuari
                  pSexe                 : Sexe de l'Usuari
                  pNumAfiliacioSanitaria: Nmero Afiliaci Sanitria de l'Usuari

      SORTIDA  :  oRSP        : Resposta del procediment            
		
   PRECONDICI : No existeix l'Usuari a la Base de Dades
   
   POSTCONDICI: S'ha donat d'alta l'Usuari
   
   RETORNA     : 'OK' si ha tingut lloc l'alta
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO D''AFILIACI DE L''USUARI (CAMP CLAU)' 
                 'ERROR: No s'ha trobat el Pas de l''Usuari' 
                 'ERROR: L'Usuari es vol donar d'alta amb el Nmero d'Identifiaci d'un altre'
                 'ERROR: L''Usuari ja havia estat donat d''alta' 
                 'ERROR: No es pot crear un Usuari amb les dades obligatries sense contingut'
                 'ERROR: El Sexe de l''Usuari noms pot ser ''MASCULI'' o ''FEMENI'''
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE USUARI_A (
    pNom                    IN CIUTADA.Nom%TYPE,
    pCognoms                IN CIUTADA.Cognoms%TYPE,
    pDocumentIdent          IN CIUTADA.DocumentIdent%TYPE,
    pPais                   IN VARCHAR2,
    pDataNaixement          IN VARCHAR2,
    pCorreuElectronic       IN CIUTADA.CorreuElectronic%TYPE,
    pSexe                   IN CIUTADA.Sexe%TYPE,
    pNumAfiliacioSanitaria  IN USUARI.NumAfiliacioSanitaria%TYPE,
    oRSP                   OUT VARCHAR2
  ) IS
  
    -- variables pel tractament d'excepcions
    eSenseNumAfiliacio  EXCEPTION; 
    ePaisNoExist        EXCEPTION;
    eUsuariExist        EXCEPTION;
    eCiutadaExist       EXCEPTION;

    vCiutadaId             NUMBER;      -- Identificador de la fitxa del ciutad
    vNom                   VARCHAR2(50);
    vCognoms               VARCHAR2(100);
    vDocumentIdent         VARCHAR2(15);
    vPaisId                NUMBER;      -- Identificador del Pas on es dona d'alta l'usuari
    vSexe                  VARCHAR2(10);
    vNumAfiliacioSanitaria VARCHAR2(15);
    vComptador             NUMBER(3);   -- Comptador dels usuaris amb el mateix nmero d'afiliaci que hi ha a la base
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'USUARI_A';
    l_ParEntrada     := 'pNom: '||pNom||', pCognoms:'||pCognoms||', pDocumentIdent:'||pDocumentIdent||
                        ', pPais:'||pPais||', pDataNaixement:'||pDataNaixement||
                        ', pCorreuElectronic:'||pCorreuElectronic||', pSexe:'||pSexe||
                        ', pNumAfiliacioSanitaria:'||pNumAfiliacioSanitaria;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. ALTA USUARI EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    -- Mirar validesa dels parmetres d'entrada (noms dels camps clau)
    IF (pNumAfiliacioSanitaria IS NULL OR pNumAfiliacioSanitaria = '') THEN 
      RAISE eSenseNumAfiliacio; 
    END IF;
  
    vNom           := UPPER(pNom);
    vCognoms       := UPPER(pCognoms);
    vDocumentIdent := UPPER(pDocumentIdent);
  
    -- Revisem que el Pas existeixi a la base, i a ms saber-ne el seu identificador
    vPaisId := PQ_UTILITATS.DONA_PAIS_ID(UPPER(pPais));
    IF (vPaisId <= 0) THEN
      RAISE ePaisNoExist;
    END IF;
  
    vSexe                  := UPPER(pSexe);
    vNumAfiliacioSanitaria := UPPER(pNumAfiliacioSanitaria);
	
    -- Verifica que el registre no s''hagi creat ja amb un altre ID
    SELECT COUNT(*) 
      INTO vComptador 
      FROM USUARI 
     WHERE NumAfiliacioSanitaria = vNumAfiliacioSanitaria;
    IF (vComptador > 0) THEN 
      RAISE eUsuariExist; 
    END IF;
 
    -- Si ja est com a metge, tamb estar com a ciutad i no cal fer l'alta.
    -- NOTA : Obviar les modificacions.
    BEGIN
      SELECT M.metId
        INTO vCiutadaId 
        FROM TFC_BD.METGE M, TFC_BD.CIUTADA C 
       WHERE C.DocumentIdent = vDocumentIdent
  	     AND C.ciuId         = M.metId;
    EXCEPTION
      WHEN no_data_found THEN
        vCiutadaID := 0;
    END;

    IF vCiutadaID = 0 THEN
    
      -- Revisem que no hi hagi un altre ciutad que ja tingui el mateix nmero d'identificaci   
      SELECT COUNT(*) 
        INTO vComptador 
        FROM TFC_BD.CIUTADA 
       WHERE DocumentIdent = vDocumentIdent;
      IF (vComptador > 0) THEN 
        RAISE eCiutadaExist; 
      END IF;
 
      -- Execuci: Alta de l'Usuari a les taules CIUTADA i USUARI
      INSERT INTO TFC_BD.CIUTADA (Nom, Cognoms, DocumentIdent, PaisId, DataNaixement,
	                       CorreuElectronic, Sexe, Borrat)
           VALUES (vNom, vCognoms, vDocumentIdent, vPaisId, TO_DATE(pDataNaixement),
                   pCorreuElectronic, vSexe, 'N');
      vCiutadaID := SQ_CIUTADA.currval;
    END IF;
	
    INSERT INTO TFC_BD.USUARI (usuId, NumAfiliacioSanitaria, UltimAnyActivitat)
        VALUES (vCiutadaId, vNumAfiliacioSanitaria, 0);

    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

  EXCEPTION
    WHEN eSenseNumAfiliacio THEN
      l_Resultat := 'ERROR: SENSE NMERO D''AFILIACI DE L''USUARI (CAMP CLAU)'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN ePaisNoExist THEN
      l_Resultat := 'ERROR : No s''ha trobat el Pas de l''Usuari'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eCiutadaExist THEN
      l_Resultat := 'ERROR : L''Usuari es vol donar d''alta amb el Nmero d''Identifiaci d''un altre';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eUsuariExist THEN
      l_Resultat := 'ERROR : L''Usuari ja havia estat donat d''alta';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      ROLLBACK;
      IF (SQLCODE = -01400) THEN 
        -- S'ha incomplert una restricci de camp no nul
        l_resultat := 'ERROR: No es pot crear un Usuari amb les dades obligatries sense contingut';
      ELSIF (SQLCODE = -00001) THEN 
        -- S'ha incomplert la restricci de N identificador duplicat
        l_resultat := 'ERROR : L''Usuari es vol donar d''alta amb el Nmero d''Identifiaci d''un altre';
      ELSIF (SQLCODE = -02290) THEN 
        -- S'ha incomplert la restricci del camp SEXE
        l_resultat := 'ERROR: El Sexe de l''Usuari noms pot ser ''MASCULI'' o ''FEMENI''';
      ELSE
        l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      END IF;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END USUARI_A;

  /******************************************************************************
   NOM         : USUARI_M
   
   PROPSIT    : Fa una modificaci de la fitxa d'un Usuari del Sistema
                                
   PARMETRES  :
      ENTRADA  :  pNumAfiliacioSanitaria: Nmero Afiliaci Sanitria de l'Usuari
                  pNom                  : Nom de l'Usuari
                  pCognoms              : Cognoms de l'Usuari
                  pDocumentIdent        : Document d'identificaci de l'Usuari
                  pDataNaixement        : Data de Naixement de l'Usuari
                  pCorreuElectronic     : Correu Electrnic de contacte de l'Usuari
                  pSexe                 : Sexe de l'Usuari
                  
      SORTIDA  :  oRSP        : Resposta del procediment            
		
   PRECONDICI : Existeix l'Usuari a la Base de Dades
   
   POSTCONDICI: S'ha fet la modificaci de la fitxa de l'Usuari
   
   RETORNA     : 'OK' si ha tingut lloc la modificaci
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO D''AFILIACI DE L''USUARI' 
                 'ERROR: NO HI HA CANVIS A FER EN LA FITXA DE L''USUARI'
                 'ERROR: No es pot canviar Document Identificaci Usuari degut a que un altre ja l''Utilitza'
                 'ERROR: L''usuari no s'ha trobat a la base de dades'
                 'ERROR: El Sexe de l''Usuari noms pot ser ''MASCULI'' o ''FEMENI'''
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE USUARI_M (
    pNumAfiliacioSanitaria  IN USUARI.NumAfiliacioSanitaria%TYPE,
    pNom                    IN CIUTADA.Nom%TYPE,
    pCognoms                IN CIUTADA.Cognoms%TYPE,
    pDocumentIdent          IN CIUTADA.DocumentIdent%TYPE,
    pDataNaixement          IN VARCHAR2,
    pCorreuElectronic       IN CIUTADA.CorreuElectronic%TYPE,
    pSexe                   IN CIUTADA.Sexe%TYPE,
    oRSP                   OUT VARCHAR2
  ) IS
  
    -- variables pel tractament d'excepcions
    eSenseNumAfiliacio  EXCEPTION; 
    eSenseModificacions EXCEPTION;
    eNoEsPotCanviarDoc  EXCEPTION;

    aUsuId              NUMBER;
    aNom                VARCHAR2(50);
    aCognoms            VARCHAR2(100);
    aDocumentIdent      VARCHAR2(15);
    aDataNaixement      DATE;
    aCorreuElectronic   VARCHAR2(200);
    aSexe               VARCHAR2(10);
  
    vNumAfiliacioSanitaria VARCHAR2(15);
    vNom                   VARCHAR2(50);
    vCognoms               VARCHAR2(100);
    vDocumentIdent         VARCHAR2(15);
    vDataNaixement         DATE;
    vCorreuElectronic      VARCHAR2(200);
    vSexe                  VARCHAR2(10);
    vComptador             NUMBER(3);   -- Comptador dels usuaris amb el mateix nmero document d'identificaci que hi ha a la base
    
  BEGIN
 
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'USUARI_M';
    l_ParEntrada     := 'pNumAfiliacioSanitaria:'||pNumAfiliacioSanitaria||', pNom: '||pNom||
                        ', pCognoms:'||pCognoms||', pDocumentIdent:'||pDocumentIdent||
                        ', pDataNaixement:'||pDataNaixement||
                        ', pCorreuElectronic:'||pCorreuElectronic||', pSexe:'||pSexe;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. MODIFICACI USUARI EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    -- Mirar validesa dels parmetres d'entrada (noms dels camps clau)
    IF (pNumAfiliacioSanitaria IS NULL OR pNumAfiliacioSanitaria = '') THEN 
      RAISE eSenseNumAfiliacio; 
    END IF;
  
    IF (pNom IS NULL OR pNom = '') AND 
       (pCognoms IS NULL OR pCognoms = '') AND 
       (pDocumentIdent IS NULL OR pDocumentIdent = '') AND 
       (pDataNaixement IS NULL OR pDataNaixement = '') AND 
       (pCorreuElectronic IS NULL OR pCorreuElectronic = '') AND 
       (pSexe IS NULL OR pSexe = '') THEN 
      RAISE eSenseModificacions; 
    END IF;
  
    vNumAfiliacioSanitaria := UPPER(pNumAfiliacioSanitaria);
    -- Agafa dades actuals de l'Usuari per contrastar-les amb les noves.
    SELECT U.usuId, C.Nom, C.Cognoms, C.DocumentIdent, C.DataNaixement, 
           C.CorreuElectronic, C.Sexe
      INTO aUsuId, aNom, aCognoms, aDocumentIdent, aDataNaixement, 
           aCorreuElectronic, aSexe
      FROM TFC_BD.USUARI U, TFC_BD.CIUTADA C
     WHERE U.NumAfiliacioSanitaria = vNumAfiliacioSanitaria
       AND U.usuId                 = C.ciuId;

    vNom              := UPPER(pNom);
    vCognoms          := UPPER(pCognoms);
    vDocumentIdent    := UPPER(pDocumentIdent);
    vCorreuElectronic := pCorreuElectronic;
    vSexe             := UPPER(pSexe);
	
     -- Si no hi ha modificacions, hi assignem les dades actuals
    IF (vNom IS NULL OR vNom = '') THEN
	    vNom := aNom;
    END IF;

    IF (vCognoms IS NULL OR vCognoms = '') THEN
	    vCognoms := aCognoms;
    END IF;

    IF (vDocumentIdent IS NULL OR vDocumentIdent = '') THEN
	    vDocumentIdent := aDocumentIdent;
    END IF;

    IF (pDataNaixement IS NULL OR pDataNaixement = '') THEN
      vDataNaixement := aDataNaixement;
    ELSE
      vDataNaixement := TO_DATE(pDataNaixement);
    END IF;
    
    IF (vCorreuElectronic IS NULL OR vCorreuElectronic = '') THEN
	    vCorreuElectronic := aCorreuElectronic;
    END IF;

    IF (vSexe IS NULL OR vSexe = '') THEN
	    vSexe := aSexe;
    END IF;

    -- Verifica que si canviem el Nmero d'identificador no hi hagi un altre ciutad que el tingui.
    IF (vDocumentIdent <> aDocumentIdent) THEN
      SELECT COUNT(*) 
        INTO vComptador 
        FROM TFC_BD.CIUTADA 
       WHERE DocumentIdent = vDocumentIdent
         AND ciuId <> aUsuId;
      IF (vComptador = 0) THEN 
        RAISE eNoEsPotCanviarDoc; 
      END IF;
    END IF;
 
    -- Execuci: Modificaci de la fitxa del ciutad
    -- NOTA : No es modifica taula USUARI per no poder-se canviar N Afiliaci (es considera nic)
    UPDATE TFC_BD.CIUTADA
	     SET Nom              = vNom,
           Cognoms          = vCognoms,
           DocumentIdent    = vDocumentIdent,
           DataNaixement    = vDataNaixement,
           CorreuElectronic = vCorreuElectronic,
           Sexe             = vSexe
     WHERE ciuId = aUsuId;

    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

  EXCEPTION
  
    WHEN eSenseNumAfiliacio THEN
      l_Resultat := 'ERROR: SENSE NMERO D''AFILIACI DE L''USUARI'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseModificacions THEN
      l_Resultat := 'ERROR: NO HI HA CANVIS A FER EN LA FITXA DE L''USUARI'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
		
    WHEN eNoEsPotCanviarDoc THEN
      l_Resultat := 'ERROR : No es pot canviar Document Identificaci Usuari degut a que un altre ja l''Utilitza'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: L''Usuari no s''ha trobat a la base de dades';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      ROLLBACK;
      IF (SQLCODE = -02290) THEN 
        -- S'ha incomplert la restricci del camp SEXE
        l_resultat := 'ERROR: El Sexe de l''Usuari noms pot ser ''MASCULI'' o ''FEMENI''';
      ELSIF (SQLCODE = -00001) THEN 
        -- S'ha incomplert la restricci de N identificador duplicat
        l_resultat := 'ERROR : No es pot canviar Document Identificaci Usuari degut a que un altre ja l''Utilitza';
      ELSE
        l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      END IF;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END USUARI_M;

  /******************************************************************************
   NOM         : USUARI_B
   
   PROPSIT    : Fa una Baixa d'un Usuari del Sistema de Recollida de Dades
                                
   PARMETRES  :
      ENTRADA  :  pNumAfiliacioSanitaria : Nmero Afiliaci Sanitria de l'Usuari

      SORTIDA  :  oRSP                   : Resposta del procediment            
		
   PRECONDICI : L'Usuari existeix a la Base de Dades
   
   POSTCONDICI: S'ha donat de baixa l'Usuari (marcat com a baixa) o
                 b s'ha tornat a donar d'alta si estava marcat com a baixa.
   
   RETORNA     : 'OK' si ha tingut lloc la baixa/recuperaci
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR : SENSE NMERO D''AFILIACI DE L''USUARI' 
                 'ERROR : ERROR: L''Usuari no s''ha trobat a la base de dades'
                 'ERROR : <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE USUARI_B (
    pNumAfiliacioSanitaria  IN USUARI.NumAfiliacioSanitaria%TYPE,
    oRSP                   OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseNumAfiliacio EXCEPTION; 

    vCiutadaId       NUMBER;      -- Identificador nic de la taula de Ciutadans
    vBorrat          VARCHAR2(1); -- Indicador de si ciutad est borrat o no
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'USUARI_B';
    l_ParEntrada     := 'pNumAfiliacioSanitaria: '||pNumAfiliacioSanitaria;
    l_ParSortida     := ' ';               
    l_DataHora       := SYSDATE;
    
    -- Mirar validesa dels parmetres d'entrada (noms dels camps clau)
    IF (pNumAfiliacioSanitaria IS NULL OR pNumAfiliacioSanitaria = '') THEN 
      RAISE eSenseNumAfiliacio; 
    END IF;

    -- Verifica que l'Usuari existeixi
    SELECT U.usuId, C.Borrat
      INTO vCiutadaId, vBorrat 
      FROM TFC_BD.USUARI U, TFC_BD.CIUTADA C 
     WHERE U.NumAfiliacioSanitaria = pNumAfiliacioSanitaria
	     AND U.usuId                 = C.ciuId;
     
    -- Execuci: Baixa o Alta de l'Usuari
    if vBorrat = 'S' then
      vBorrat := 'N';
      l_Resultat := 'OK. RECUPERACI FITXA USUARI EFECTUADA';
    ELSE
      vBorrat := 'S';
      l_Resultat := 'OK. BORRAT FITXA USUARI EFECTUADA';
    END IF;	
	
    oRSP := l_Resultat;
	
    UPDATE TFC_BD.CIUTADA 
	     SET Borrat = vBorrat 
     WHERE ciuId = vCiutadaId;
    
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;
    
  EXCEPTION
  
    WHEN eSenseNumAfiliacio THEN
      l_Resultat := 'ERROR: SENSE NMERO D''AFILIACI DE L''USUARI'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: L''Usuari no s''ha trobat a la base de dades';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      ROLLBACK;
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END USUARI_B;
   
END;
/

COMMIT;  
