  /******************************************************************************
  
   ARXIU    : 2_03_MetgeABM.sql
   AUTOR    : Jess Hernndez Garcia - TFC - Aula 1
   
   PROPSIT : Script de creaci del paquet PQ_METGE_ABM : 
              Procediments i funcions d'Altes, baixes i modificacions de la
			  taula METGE.
                                
   ******************************************************************************/

CREATE OR REPLACE PACKAGE TFC_BD.PQ_METGE_ABM AS

  /*  Alta d'un Metge del Sistema de Recollida de Dades */
  PROCEDURE METGE_A (
    pNom                IN CIUTADA.Nom%TYPE,
    pCognoms            IN CIUTADA.Cognoms%TYPE,
    pDocumentIdent      IN CIUTADA.DocumentIdent%TYPE,
    pPais               IN VARCHAR2,
    pDataNaixement      IN VARCHAR2,
    pCorreuElectronic   IN CIUTADA.CorreuElectronic%TYPE,
    pSexe               IN CIUTADA.Sexe%TYPE,
    pEspecialitatCodi   IN METGE.EspecialitatCodi%TYPE,
    pNumColegiat        IN METGE.NumColegiat%TYPE,
    oRSP               OUT VARCHAR2
	);
	
  /*  Modificaci d'un Metge del Sistema de Recollida de Dades */
  PROCEDURE METGE_M (
    pNumColegiat       IN METGE.NumColegiat%TYPE,
    pEspecialitatCodi  IN METGE.EspecialitatCodi%TYPE,
    pNom               IN CIUTADA.Nom%TYPE,
    pCognoms           IN CIUTADA.Cognoms%TYPE,
    pDocumentIdent     IN CIUTADA.DocumentIdent%TYPE,
    pDataNaixement     IN VARCHAR2,
    pCorreuElectronic  IN CIUTADA.CorreuElectronic%TYPE,
    pSexe              IN CIUTADA.Sexe%TYPE,
    oRSP              OUT VARCHAR2
	);
	
  /*  Baixa d'un Metge del Sistema de Recollida de Dades */
  PROCEDURE METGE_B (
    pNumColegiat  IN METGE.NumColegiat%TYPE,
    oRSP         OUT VARCHAR2
	);


END;
/

CREATE OR REPLACE PACKAGE BODY TFC_BD.PQ_METGE_ABM IS

    -- Variables de treball per a guardar moviments al registre d'activitat LOG_S
    l_NomUsuari        VARCHAR2(12);
    l_NomProcediment   VARCHAR2(60);
    l_ParEntrada       VARCHAR2(400);
    l_ParSortida       VARCHAR2(400);
    l_Resultat         VARCHAR2(300);
    l_DataHora         DATE;
    
  /******************************************************************************
   NOM         : METGE_A
   
   PROPSIT    : Fa una alta d'un Metge del Sistema de Recollida de Dades
                                
   PARMETRES  :
      ENTRADA  :  pNom              : Nom del Metge
                  pCognoms          : Cognoms del Metge
                  pDocumentIdent    : Document d'identificaci del Metge
                  pPais             : Codi de Pas on resideix el Metge
                  pDataNaixement    : Data de Naixement del Metge
                  pCorreuElectronic : Correu Electrnic de contacte del Metge
                  pSexe             : Sexe del Metge
                  pEspecialitatCodi : Codi de l'especialitat del Metge
                  pNumColegiat      : Nmero de Collegiat del Metge

      SORTIDA  :  oRSP              : Resposta del procediment            
		
   PRECONDICI : No existeix el Metge a la Base de Dades
   
   POSTCONDICI: S'ha donat d'alta el Metge
   
   RETORNA     : 'OK' si ha tingut lloc l'alta
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO DE COLLEGIAT DEL METGE (CAMP CLAU)' 
                 'ERROR : No s'ha trobat la Especialitat del Metge' 
                 'ERROR : No s'ha trobat el Pas del Metge' 
                 'ERROR : El Metge ja havia estat donat d''alta' 
                 'ERROR: No es pot crear un Metge amb les dades obligatries sense contingut';
                 'ERROR: El Sexe del Metge noms pot ser ''MASCULI'' o ''FEMENI'''
                 'ERROR: <NumError><DescError>' per les altres causes
  
   ******************************************************************************/

  PROCEDURE METGE_A (
    pNom               IN CIUTADA.Nom%TYPE,
    pCognoms           IN CIUTADA.Cognoms%TYPE,
    pDocumentIdent     IN CIUTADA.DocumentIdent%TYPE,
    pPais              IN VARCHAR2,
    pDataNaixement     IN VARCHAR2,
    pCorreuElectronic  IN CIUTADA.CorreuElectronic%TYPE,
    pSexe              IN CIUTADA.Sexe%TYPE,
    pEspecialitatCodi  IN METGE.EspecialitatCodi%TYPE,
    pNumColegiat       IN METGE.NumColegiat%TYPE,
    oRSP              OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseNumColegiat    EXCEPTION; 
    eEspecialitatNoExist EXCEPTION;
    ePaisNoExist         EXCEPTION;
    eMetgeExist          EXCEPTION;

    vCiutadaId             NUMBER;      -- Identificador de la fitxa del ciutad
    vNom                   VARCHAR2(50);
    vCognoms               VARCHAR2(100);
    vDocumentIdent         VARCHAR2(15);
    vPaisId                NUMBER;      -- Identificador del Pas on es dona d'alta el Metge
    vSexe                  VARCHAR2(10);
    vNumColegiat           VARCHAR2(15);
    vEspecialitatCodi      VARCHAR2(5);
    vComptador             NUMBER(3);   -- Comptador de registres que hi ha en una taula que 
                                        -- compleixen les condicions.
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'METGE_A';
    l_ParEntrada     := 'pNom: '||pNom||', pCognoms:'||pCognoms||', pDocumentIdent:'||pDocumentIdent||
                        ', pPais:'||pPais||', pDataNaixement:'||pDataNaixement||
                        ', pCorreuElectronic:'||pCorreuElectronic||', pSexe:'||pSexe||
                        ', pEspecialitatCodi:'||pEspecialitatCodi||', pNumColegiat:'||pNumColegiat;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. ALTA METGE EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    -- Mirar validesa dels parmetres d'entrada (noms dels camps clau)
    IF (pNumColegiat IS NULL OR pNumColegiat = '') THEN 
      RAISE eSenseNumColegiat; 
    END IF;
    
    vEspecialitatCodi := UPPER(pEspecialitatCodi);
    
    -- Revisem que la especialitat del Metge existeixi a la base
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.ESPECIALITAT
     WHERE Codi = vEspecialitatCodi;
    IF (vComptador <= 0) THEN 
      RAISE eEspecialitatNoExist; 
    END IF;
  
    -- Revisem que el Pas existeixi a la base, i a ms saber-ne el seu identificador
    vPaisId := PQ_UTILITATS.DONA_PAIS_ID(UPPER(pPais));
    IF (vPaisId <= 0) THEN
      RAISE ePaisNoExist;
    END IF;
  
    vNom              := UPPER(pNom);
    vCognoms          := UPPER(pCognoms);
    vDocumentIdent    := UPPER(pDocumentIdent);
    vSexe             := UPPER(pSexe);
    vNumColegiat      := UPPER(pNumColegiat);
	
    -- Verifica que el registre no s''hagi creat ja amb un altre ID
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.METGE 
     WHERE NumColegiat = vNumColegiat;
    IF (vComptador > 0) THEN 
      RAISE eMetgeExist; 
    END IF;
 
    -- Si ja est com a usuari, tamb estar com a ciutad i no cal fer l'alta.
    -- NOTA : Obviar les modificacions.
    BEGIN
      SELECT U.usuId
        INTO vCiutadaId 
        FROM TFC_BD.USUARI U, TFC_BD.CIUTADA C 
       WHERE C.DocumentIdent = vDocumentIdent
	       AND C.ciuId         = U.usuId;
    EXCEPTION
      WHEN no_data_found THEN
        vCiutadaID := 0;
    END;

    -- Execuci: Alta del Metge a les taules CIUTADA i METGE
    IF vCiutadaID = 0 THEN
      INSERT INTO TFC_BD.CIUTADA (Nom, Cognoms, DocumentIdent, PaisId, DataNaixement,
                                  CorreuElectronic, Sexe, Borrat)
           VALUES (vNom, vCognoms, vDocumentIdent, vPaisId, TO_DATE(pDataNaixement),
                   pCorreuElectronic, vSexe, 'N');
      vCiutadaID := SQ_CIUTADA.currval;
    END IF;
	
    INSERT INTO TFC_BD.METGE (metId, EspecialitatCodi, NumColegiat)
        VALUES (vCiutadaId, vEspecialitatCodi, vNumColegiat);

    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

  EXCEPTION
    WHEN eSenseNumColegiat THEN
      l_Resultat := 'ERROR: SENSE NMERO DE COLLEGIAT DEL METGE (CAMP CLAU)'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eEspecialitatNoExist THEN
      l_Resultat := 'ERROR : No s''ha trobat la Especialitat del Metge'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN ePaisNoExist THEN
      l_Resultat := 'ERROR : No s''ha trobat el Pas del Metge'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eMetgeExist THEN
      l_Resultat := 'ERROR : El Metge ja havia estat donat d''alta';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      ROLLBACK;
      IF (SQLCODE = -01400) THEN 
        -- S'ha incomplert una restricci de camp no nul
        l_resultat := 'ERROR: No es pot crear un Metge amb les dades obligatries sense contingut';
      ELSIF (SQLCODE = -02290) THEN 
        -- S'ha incomplert la restricci del camp SEXE
        l_resultat := 'ERROR: El Sexe del Metge noms pot ser ''MASCULI'' o ''FEMENI''';
      ELSE
        l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      END IF;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END METGE_A;

  /******************************************************************************
   NOM         : METGE_M
   
   PROPSIT    : Fa una modificaci de la fitxa d'un Metge del Sistema
                                
   PARMETRES  :
      ENTRADA  :  pNumColegiat      : Nmero de Collegiat del Metge
                  pEspecialitatCodi : Codi de l'especialitat del Metge
                  pNom              : Nom del Metge
                  pCognoms          : Cognoms del Metge
                  pDocumentIdent    : Document d'identificaci del Metge
                  pDataNaixement    : Data de Naixement del Metge
                  pCorreuElectronic : Correu Electrnic de contacte del Metge
                  pSexe             : Sexe del Metge
                  
      SORTIDA  :  oRSP        : Resposta del procediment            
		
   PRECONDICI : Existeix el Metge a la Base de Dades
   
   POSTCONDICI: S'ha fet la modificaci de la fitxa del Metge
   
   RETORNA     : 'OK' si ha tingut lloc la modificaci
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO DE COLLEGIAT DEL METGE'
                 'ERROR: NO HI HA CANVIS A FER EN LA FITXA DEL METGE'
                 'ERROR : No s'ha trobat la Especialitat del Metge' 
                 'ERROR : No es pot canviar Document Identificaci Metge degut a que un altre ja l''Utilitza'
                 'ERROR : El Metge no s'ha trobat a la base de dades'
                 'ERROR: El Sexe del Metge noms pot ser ''MASCULI'' o ''FEMENI'''
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE METGE_M (
    pNumColegiat       IN METGE.NumColegiat%TYPE,
    pEspecialitatCodi  IN METGE.EspecialitatCodi%TYPE,
    pNom               IN CIUTADA.Nom%TYPE,
    pCognoms           IN CIUTADA.Cognoms%TYPE,
    pDocumentIdent     IN CIUTADA.DocumentIdent%TYPE,
    pDataNaixement     IN VARCHAR2,
    pCorreuElectronic  IN CIUTADA.CorreuElectronic%TYPE,
    pSexe              IN CIUTADA.Sexe%TYPE,
    oRSP              OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseNumColegiat    EXCEPTION; 
    eSenseModificacions  EXCEPTION;
    eEspecialitatNoExist EXCEPTION;
    eNoEsPotCanviarDoc   EXCEPTION;

    aMetId              NUMBER;
    aEspecialitatCodi   VARCHAR2(5);
    aNom                VARCHAR2(50);
    aCognoms            VARCHAR2(100);
    aDocumentIdent      VARCHAR2(15);
    aDataNaixement      DATE;
    aCorreuElectronic   VARCHAR2(200);
    aSexe               VARCHAR2(10);
  
    vNumColegiat        VARCHAR2(15);
    vEspecialitatCodi   VARCHAR2(5);
    vNom                VARCHAR2(50);
    vCognoms            VARCHAR2(100);
    vDocumentIdent      VARCHAR2(15);
    vDataNaixement      DATE;
    vCorreuElectronic   VARCHAR2(200);
    vSexe               VARCHAR2(10);
    vComptador          NUMBER(3);  -- Comptador de registres que hi ha en una taula que 
                                    -- compleixen les condicions.
  BEGIN
 
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'METGE_M';
    l_ParEntrada     := 'pNumColegiat:'||pNumColegiat||', pEspecialitatCodi:'||pEspecialitatCodi||
                        ', pNom: '||pNom||', pCognoms:'||pCognoms||', pDocumentIdent:'||pDocumentIdent||
                        ', pDataNaixement:'||pDataNaixement||
                        ', pCorreuElectronic:'||pCorreuElectronic||', pSexe:'||pSexe;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. MODIFICACI METGE EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    -- Mirar validesa dels parmetres d'entrada (noms dels camps clau)
    IF (pNumColegiat IS NULL OR pNumColegiat = '') THEN 
      RAISE eSenseNumColegiat; 
    END IF;
  
    IF (pEspecialitatCodi IS NULL OR pEspecialitatCodi = '') AND 
       (pNom IS NULL OR pNom = '') AND 
       (pCognoms IS NULL OR pCognoms = '') AND 
       (pDocumentIdent IS NULL OR pDocumentIdent = '') AND 
       (pDataNaixement IS NULL OR pDataNaixement = '') AND 
       (pCorreuElectronic IS NULL OR pCorreuElectronic = '') AND 
       (pSexe IS NULL OR pSexe = '') THEN 
      RAISE eSenseModificacions; 
    END IF;
  
    vNumColegiat := UPPER(pNumColegiat);
    -- Agafa dades actuals del Metge per contrastar-les amb les noves.
    SELECT M.metId, M.EspecialitatCodi, C.Nom, C.Cognoms, C.DocumentIdent, 
           C.DataNaixement, C.CorreuElectronic, C.Sexe
      INTO aMetId, aEspecialitatCodi, aNom, aCognoms, aDocumentIdent, 
           aDataNaixement, aCorreuElectronic, aSexe
      FROM TFC_BD.METGE M, TFC_BD.CIUTADA C
     WHERE M.NumColegiat = vNumColegiat
       AND M.metId       = C.ciuId;

    vEspecialitatCodi := UPPER(pEspecialitatCodi);
    vNom              := UPPER(pNom);
    vCognoms          := UPPER(pCognoms);
    vDocumentIdent    := UPPER(pDocumentIdent);
    vCorreuElectronic := pCorreuElectronic;
    vSexe             := UPPER(pSexe);
	
     -- Si no hi ha modificacions, hi assignem les dades actuals
    IF (vEspecialitatCodi IS NULL OR vEspecialitatCodi = '') THEN
	    vEspecialitatCodi := aEspecialitatCodi;
    END IF;

    IF (vNom IS NULL OR vNom = '') THEN
	    vNom := aNom;
    END IF;

    IF (vCognoms IS NULL OR vCognoms = '') THEN
	    vCognoms := aCognoms;
    END IF;

    IF (vDocumentIdent IS NULL OR vDocumentIdent = '') THEN
	    vDocumentIdent := aDocumentIdent;
    END IF;

    IF (pDataNaixement IS NULL OR pDataNaixement = '') THEN
      vDataNaixement := aDataNaixement;
    ELSE
      vDataNaixement := TO_DATE(pDataNaixement);
    END IF;
    
    IF (vCorreuElectronic IS NULL OR vCorreuElectronic = '') THEN
	    vCorreuElectronic := aCorreuElectronic;
    END IF;

    IF (vSexe IS NULL OR vSexe = '') THEN
	    vSexe := aSexe;
    END IF;

    -- Revisem que la especialitat del Metge existeixi a la base
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.ESPECIALITAT
     WHERE Codi = vEspecialitatCodi;
    IF (vComptador <= 0) THEN 
      RAISE eEspecialitatNoExist; 
    END IF;
  
    -- Verifica que si canviem el Nmero d'identificador no hi hagi un altre ciutad que el tingui.
    IF (vDocumentIdent <> aDocumentIdent) THEN
      SELECT COUNT(*) 
        INTO vComptador 
        FROM TFC_BD.CIUTADA 
       WHERE DocumentIdent = vDocumentIdent
         AND ciuId <> aMetId;
      IF (vComptador = 0) THEN 
        RAISE eNoEsPotCanviarDoc; 
      END IF;
    END IF;
 
    -- Execuci: Modificaci de la fitxa del ciutad
    UPDATE TFC_BD.CIUTADA
	     SET Nom              = vNom,
           Cognoms          = vCognoms,
           DocumentIdent    = vDocumentIdent,
           DataNaixement    = vDataNaixement,
           CorreuElectronic = vCorreuElectronic,
           Sexe             = vSexe
     WHERE ciuId = aMetId;
     
    -- NOTA : Es modifica de la taula METGE noms la Especialitat, ja no es pot
    -- canviar N Collegiat (es considera nic).
    UPDATE TFC_BD.METGE
	     SET EspecialitatCodi = vEspecialitatCodi
     WHERE metId = aMetId;
 
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

  EXCEPTION
  
    WHEN eSenseNumColegiat THEN
      l_Resultat := 'ERROR: SENSE NMERO DE COLLEGIAT DEL METGE'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseModificacions THEN
      l_Resultat := 'ERROR: NO HI HA CANVIS A FER EN LA FITXA DEL METGE'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
		
    WHEN eEspecialitatNoExist THEN
      l_Resultat := 'ERROR : No s''ha trobat la Especialitat del Metge'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eNoEsPotCanviarDoc THEN
      l_Resultat := 'ERROR : No es pot canviar Document Identificaci Metge degut a que un altre ja l''Utilitza'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: El Metge no s''ha trobat a la base de dades';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      ROLLBACK;
      IF (SQLCODE = -02290) THEN 
        -- S'ha incomplert la restricci del camp SEXE
        l_resultat := 'ERROR: El Sexe del Metge noms pot ser ''MASCULI'' o ''FEMENI''';
      ELSIF (SQLCODE = -00001) THEN 
        -- S'ha incomplert la restricci de N identificador duplicat
        l_resultat := 'ERROR : No es pot canviar Document Identificaci Metge degut a que un altre ja l''Utilitza';
      ELSE
        l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      END IF;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END METGE_M;

  /******************************************************************************
   NOM         : METGE_B
   
   PROPSIT    : Fa una Baixa d'un Metge del Sistema de Recollida de Dades
                                
   PARMETRES  :
      ENTRADA  :  pNumColegiat      : Nmero de Collegiat del Metge
                  
      SORTIDA  :  oRSP              : Resposta del procediment            
		
   PRECONDICI : El Metge existeix a la Base de Dades
   
   POSTCONDICI: S'ha donat de baixa el Metge (marcat com a baixa) o
                 b s'ha tornat a donar d'alta si estava marcat com a baixa.
   
   RETORNA     : 'OK' si ha tingut lloc la baixa/recuperaci
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR : SENSE NMERO DE COLLEGIAT DEL METGE' 
                 'ERROR : ERROR: El Metge no s''ha trobat a la base de dades'
                 'ERROR : <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE METGE_B (
    pNumColegiat IN METGE.NumColegiat%TYPE,
    oRSP        OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseNumColegiat EXCEPTION; 

    vCiutadaId NUMBER;      -- Identificador nic de la taula de Ciutadans
    vBorrat    VARCHAR2(1); -- Indicador de si ciutad est borrat o no
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'METGE_B';
    l_ParEntrada     := 'pNumColegiat: '||pNumColegiat;
    l_ParSortida     := ' ';               
    l_DataHora       := SYSDATE;
    
    -- Mirar validesa dels parmetres d'entrada (noms dels camps clau)
    IF (pNumColegiat IS NULL OR pNumColegiat = '') THEN 
      RAISE eSenseNumColegiat; 
    END IF;

    -- Verifica que el Metge existeixi
    SELECT M.metId, C.Borrat
      INTO vCiutadaId, vBorrat 
      FROM TFC_BD.METGE M, TFC_BD.CIUTADA C 
     WHERE M.NumColegiat = pNumColegiat
	     AND M.metId       = C.ciuId;
     
    -- Execuci: Baixa o Alta del Metge
    if vBorrat = 'S' then
      vBorrat := 'N';
      l_Resultat := 'OK. RECUPERACI FITXA METGE EFECTUADA';
    ELSE
      vBorrat := 'S';
      l_Resultat := 'OK. BORRAT FITXA METGE EFECTUADA';
    END IF;	
	
    oRSP := l_Resultat;
	
    UPDATE TFC_BD.CIUTADA 
	     SET Borrat = vBorrat 
     WHERE ciuId = vCiutadaId;
    
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;
    
  EXCEPTION
  
    WHEN eSenseNumColegiat THEN
      l_Resultat := 'ERROR: SENSE NMERO DE COLLEGIAT DEL METGE'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: El Metge no s''ha trobat a la base de dades';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      ROLLBACK;
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END METGE_B;

END;
/

COMMIT;  

