  /******************************************************************************
  
   ARXIU    : 2_04_CentreSanitariABM.sql
   AUTOR    : Jess Hernndez Garcia - TFC - Aula 1
   
   PROPSIT : Script de creaci del paquet PQ_CENTRE_SANITARI_ABM : 
              Procediments i funcions d'Altes, baixes i modificacions de la
			  taula CENTRE_SANITARI.
                                
   ******************************************************************************/

CREATE OR REPLACE PACKAGE TFC_BD.PQ_CENTRE_SANITARI_ABM AS

  /*  Alta d'un Centre Sanitari on s'efectuen visites */
  PROCEDURE CENTRE_SANITARI_A (
    pNom       IN CENTRE_SANITARI.Nom%TYPE,
    pAdreca    IN CENTRE_SANITARI.Adreca%TYPE,
    pPoblacio  IN CENTRE_SANITARI.Poblacio%TYPE,
    pPais      IN VARCHAR2,
    oRSP      OUT VARCHAR2
	);
	
  /*  Modificaci d'un Centre Sanitari on s'efectuen visites */
  PROCEDURE CENTRE_SANITARI_M (
    pCentreId  IN CENTRE_SANITARI.cesId%TYPE,
    pNom       IN CENTRE_SANITARI.Nom%TYPE,
    pAdreca    IN CENTRE_SANITARI.Adreca%TYPE,
    pPoblacio  IN CENTRE_SANITARI.Poblacio%TYPE,
    pPais      IN VARCHAR2,
    oRSP      OUT VARCHAR2
	);
	
  /*  Baixa d'un Centre Sanitari on s'efectuen visites */
  PROCEDURE CENTRE_SANITARI_B (
    pCentreId IN CENTRE_SANITARI.cesId%TYPE,
    oRSP      OUT VARCHAR2
	);

END PQ_CENTRE_SANITARI_ABM;
/

CREATE OR REPLACE PACKAGE BODY TFC_BD.PQ_CENTRE_SANITARI_ABM IS

    -- Variables de treball per a guardar moviments al registre d'activitat LOG_S
    l_NomUsuari        VARCHAR2(12);
    l_NomProcediment   VARCHAR2(60);
    l_ParEntrada       VARCHAR2(400);
    l_ParSortida       VARCHAR2(400);
    l_Resultat         VARCHAR2(300);
    l_DataHora         DATE;
    
  /******************************************************************************
   NOM         : CENTRE_SANITARI_A
   
   PROPSIT    : Fa una alta d'un Centre Sanitari d'un pas on s'efectuen visites 
                                
   PARMETRES  :
      ENTRADA  :  pNom        : Nom del Centre Sanitari
                  pAdreca     : Adrea del Centre Sanitari
                  pPoblacio   : Poblaci del Centre Sanitari
                  pPais       : Codi de Pas del Centre Sanitari

      SORTIDA  :  oRSP        : Resposta del procediment            
		
   PRECONDICI : No existeix el Centre Sanitari a la Base de Dades
   
   POSTCONDICI: S'ha donat d'alta el Centre Sanitari
   
   RETORNA     : 'OK' si ha tingut lloc l'alta
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR : No s'ha trobat el Pas del Centre Sanitari' 
                 'ERROR : El Centre Sanitari ja havia estat donat d'alta' 
                 'ERROR : SENSE NOM DEL CENTRE SANITARI (CAMP CLAU)' 
                 'ERROR : <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CENTRE_SANITARI_A (
    pNom       IN CENTRE_SANITARI.Nom%TYPE,
    pAdreca    IN CENTRE_SANITARI.Adreca%TYPE,
    pPoblacio  IN CENTRE_SANITARI.Poblacio%TYPE,
    pPais      IN VARCHAR2,
    oRSP      OUT VARCHAR2
  ) IS
  
    -- variables pel tractament d'excepcions
    ePaisNoExist     EXCEPTION;
    eCentreExist     EXCEPTION;

    vNom             VARCHAR2(100);
    vAdreca          VARCHAR2(100);
    vPoblacio        VARCHAR2(100);
    vPaisId          NUMBER;      -- Identificador del Pas on es dona d''alta el centre
    vComptador       NUMBER(3);   -- Comptador dels centres amb el nom que hi ha a la base
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CENTRE_SANITARI_A';
    l_ParEntrada     := 'pNom: '||pNom||
                        ', pAdreca:'||pAdreca||
                        ', pPoblacio:'||pPoblacio||
                        ', pPais:'||pPais;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. ALTA CENTRE SANITARI EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    vNom      := UPPER(pNom);
    vAdreca   := UPPER(pAdreca);
    vPoblacio := UPPER(pPoblacio);
  
    -- Revisem que el Pas existeixi a la base, i a ms saber-ne el seu identificador
    vPaisId := PQ_UTILITATS.DONA_PAIS_ID(UPPER(pPais));
    IF (vPaisId <= 0) THEN
      RAISE ePaisNoExist;
    END IF;
  
    -- Verifica que el registre no s''hagi creat ja amb un altre ID
    -- (Es permeten diferents centres amb el mateix nom per en diferent pas)
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.CENTRE_SANITARI 
     WHERE Nom    = vNom 
       AND PaisId = vPaisId;
    IF (vComptador > 0) THEN 
      RAISE eCentreExist; 
    END IF;
 
    -- Execuci: Alta del Centre Sanitari
    INSERT INTO TFC_BD.CENTRE_SANITARI (Nom, Adreca, Poblacio, PaisId)
        VALUES (vNom, vAdreca, vPoblacio, vPaisId);

    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

  EXCEPTION
    WHEN ePaisNoExist THEN
      l_Resultat := 'ERROR : No s''ha trobat el Pas del Centre Sanitari'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eCentreExist THEN
      l_Resultat := 'ERROR : El Centre Sanitari ja havia estat donat d''alta';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      ROLLBACK;
      IF (SQLCODE = -01400) THEN 
        l_Resultat := 'ERROR : SENSE NOM DEL CENTRE SANITARI (CAMP CLAU)'; 
      ELSE
        l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      END IF;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CENTRE_SANITARI_A;

  /******************************************************************************
   NOM         : CENTRE_SANITARI_M
   
   PROPSIT    : Fa una modificaci de la fitxa d'un Centre Sanitari d'un pas 
                                
   PARMETRES  :
      ENTRADA  :  pCentreId   : Identificador del Centre Sanitari
                  pNom        : Nom del Centre Sanitari
                  pAdreca     : Adrea del Centre Sanitari
                  pPoblacio   : Poblaci del Centre Sanitari
                  pPais       : Codi de Pas del Centre Sanitari

      SORTIDA  :  oRSP        : Resposta del procediment            
		
   PRECONDICI : Existeix el Centre Sanitari a la Base de Dades
   
   POSTCONDICI: S'ha fet la modificaci de la fitxa del Centre Sanitari
   
   RETORNA     : 'OK' si ha tingut lloc la modificaci
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE IDENTIFICADOR DEL CENTRE SANITARI' 
                 'ERROR: NO HI HA CANVIS A FER EN EL CENTRE SANITARI'
                 'ERROR : No s''ha trobat el Pas al que es vol assignar el Centre Sanitari' 
                 'ERROR : El Centre Sanitari no s'ha trobat a la base de dades'
                 'ERROR : <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CENTRE_SANITARI_M (
    pCentreId  IN CENTRE_SANITARI.cesId%TYPE,
    pNom       IN CENTRE_SANITARI.Nom%TYPE,
    pAdreca    IN CENTRE_SANITARI.Adreca%TYPE,
    pPoblacio  IN CENTRE_SANITARI.Poblacio%TYPE,
    pPais      IN VARCHAR2,
    oRSP      OUT VARCHAR2
  ) IS
  
    -- variables pel tractament d'excepcions
    eSenseIdentificador EXCEPTION;
    eSenseModificacions EXCEPTION;
  	ePaisNoExist        EXCEPTION;
	
    -- Joc de variables per guardar valors actuals
    aNom             VARCHAR2(100);
    aAdreca          VARCHAR2(100);
    aPoblacio        VARCHAR2(100);
    aPais            VARCHAR2(3);
	
    -- Joc de variables per a gestionar valors nous
    vNom             VARCHAR2(100);
    vAdreca          VARCHAR2(100);
    vPoblacio        VARCHAR2(100);
  	vPais            VARCHAR2(3);
    vPaisId          NUMBER;      -- Identificador del Pas on est o estar el centre
    vComptador       NUMBER(3);   -- Comptador dels centres amb el nom que hi ha a la base
    
  BEGIN
 
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CENTRE_SANITARI_M';
    l_ParEntrada     := 'pCentreId: '||pCentreId||
                        ', pNom: '||pNom||
                        ', pAdreca:'||pAdreca||
                        ', pPoblacio:'||pPoblacio||
                        ', pPais:'||pPais;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. MODIFICACIO CENTRE SANITARI EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    -- Mirar validesa dels parmetres d'entrada
    IF (pCentreId IS NULL OR pCentreId = 0) THEN 
      RAISE eSenseIdentificador; 
    END IF;

    IF (pNom      IS NULL OR pNom      = '') AND 
       (pAdreca   IS NULL OR pAdreca   = '') AND
       (pPoblacio IS NULL OR pPoblacio = '') AND
       (pPais     IS NULL OR pPais     = '') THEN 
      RAISE eSenseModificacions; 
    END IF;
  
    -- Agafa dades actuals del centre per contrastar-les amb les noves.
    SELECT C.Nom, C.Adreca, C.Poblacio, P.Codi
      INTO aNom, aAdreca, aPoblacio, aPais
      FROM TFC_BD.CENTRE_SANITARI C, TFC_BD.PAIS P
     WHERE C.cesId  = pCentreId
       AND C.PaisId = P.paiId;
 

    vNom      := UPPER(pNom);
    vAdreca   := UPPER(pAdreca);
    vPoblacio := UPPER(pPoblacio);
    vPais     := UPPER(pPais);
	
     -- Si no hi ha modificacions, hi assignem les dades actuals
    IF (vNom IS NULL OR vNom = '') THEN
	    vNom := aNom;
    END IF;

    IF (vAdreca IS NULL OR vAdreca = '') THEN
	    vAdreca := aAdreca;
    END IF;

    IF (vPoblacio IS NULL OR vPoblacio = '') THEN
	    vPoblacio := aPoblacio;
    END IF;

    IF (vPais IS NULL OR vPais = '') THEN
	    vPais := aPais;
    END IF;

    -- Revisem que el Pas a modificar existeixi a la base, i a ms saber-ne el seu identificador
    vPaisId := PQ_UTILITATS.DONA_PAIS_ID(vPais);
    IF (vPaisId <= 0) THEN
      RAISE ePaisNoExist;
    END IF;
  
    -- Execuci: Modificaci del Centre Sanitari
    UPDATE TFC_BD.CENTRE_SANITARI 
	   SET Nom      = vNom, 
         Adreca   = vAdreca,
         Poblacio = vPoblacio,
         PaisId   = vPaisId
     WHERE cesId = pCentreId;

    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
      
    COMMIT;

  EXCEPTION
  
    WHEN eSenseIdentificador THEN
      l_Resultat := 'ERROR: SENSE IDENTIFICADOR DEL CENTRE SANITARI'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseModificacions THEN
      l_Resultat := 'ERROR: NO HI HA CANVIS A FER EN EL CENTRE SANITARI'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
		
    WHEN ePaisNoExist THEN
      l_Resultat := 'ERROR : No s''ha trobat el Pas al que es vol assignar el Centre Sanitari'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: El Centre Sanitari no s''ha trobat a la base de dades';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      ROLLBACK;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CENTRE_SANITARI_M;

  /******************************************************************************
   NOM         : CENTRE_SANITARI_B
   
   PROPSIT    : Fa una Baixa d'un Centre Sanitari d'un pas on s'efectuen visites 
                                
   PARMETRES  :
      ENTRADA  :  pCentreId   : Identificador del Centre Sanitari

      SORTIDA  :  oRSP        : Resposta del procediment            
		
   PRECONDICI : El Centre Sanitari existeix a la Base de Dades
   
   POSTCONDICI: S'ha donat de baixa el Centre Sanitari
   
   RETORNA     : 'OK' si ha tingut lloc la baixa
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR : SENSE IDENTIFICADOR DEL CENTRE SANITARI' 
                 'ERROR : El Centre Sanitari no s'ha trobat a la base de dades'
                 'ERROR : No es pot esborrar el Centre Sanitari per estar relacionat
                          amb altres dades'
                 'ERROR : <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CENTRE_SANITARI_B (
    pCentreId IN CENTRE_SANITARI.cesId%TYPE,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseIdentificador EXCEPTION;
    eCentreNoExist      EXCEPTION;

    vComptador NUMBER(3);  -- Comptador de tuples de Centres amb mateix ID (en principi noms 1)
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CENTRE_SANITARI_B';
    l_ParEntrada     := 'pCentreId: '||pCentreId;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. BAIXA CENTRE SANITARI EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    -- Mirar validesa dels parmetres d'entrada (camps clau)
    IF (pCentreId IS NULL OR pCentreId = 0) THEN 
      RAISE eSenseIdentificador; 
    END IF;

    -- Verifica que el Centre existeixi
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.CENTRE_SANITARI 
     WHERE cesId = pCentreId;
    IF (vComptador = 0) THEN 
      RAISE eCentreNoExist; 
    END IF;
 
    -- Execuci: Baixa del Centre Sanitari
    DELETE FROM TFC_BD.CENTRE_SANITARI WHERE cesId = pCentreId;

    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;
    
  EXCEPTION
  
    WHEN eSenseIdentificador THEN
      l_Resultat := 'ERROR: SENSE IDENTIFICADOR DEL CENTRE SANITARI'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eCentreNoExist THEN
      l_Resultat := 'ERROR: El Centre Sanitari no s''ha trobat a la base de dades';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      ROLLBACK;
      IF (SQLCODE = -02292) THEN 
        l_resultat := 'ERROR: No es pot esborrar el Centre Sanitari per estar relacionat amb altres dades';
      ELSE
        l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      END IF;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CENTRE_SANITARI_B;

END;
/

COMMIT;  
