  /******************************************************************************
  
   ARXIU    : 2_05_VisitaABM.sql
   AUTOR    : Jess Hernndez Garcia - TFC - Aula 1
   
   PROPSIT : Script de creaci del paquet PQ_VISITA_ABM : 
              Procediments i funcions d'Altes, baixes i modificacions de la
			  taula VISITA.
                                
   ******************************************************************************/

CREATE OR REPLACE PACKAGE TFC_BD.PQ_VISITA_ABM AS

  /*  Alta d'una visita concertada en un Centre Sanitari */
  PROCEDURE VISITA_A (
    pCentreSanitariId   IN VISITA.CentreSanitariId%TYPE,
    pNumColegiatMetge   IN VARCHAR2,
    pNumAfiliacioUsuari IN VARCHAR2,
    pDataHora           IN VARCHAR2,
    pNumConsulta        IN VISITA.NumConsulta%TYPE,
    pComentari          IN VISITA.Comentari%TYPE,
    oRSP               OUT VARCHAR2
	);

  /*  Modificaci d'una visita concertada en un Centre Sanitari */
  PROCEDURE VISITA_M (
    pVisitaId           IN VISITA.visId%TYPE,
    pCentreSanitariId   IN VISITA.CentreSanitariId%TYPE,
    pNumColegiatMetge   IN VARCHAR2,
    pNumAfiliacioUsuari IN VARCHAR2,
    pDataHora           IN VARCHAR2,
    pNumConsulta        IN VISITA.NumConsulta%TYPE,
    pComentari          IN VISITA.Comentari%TYPE,
    oRSP               OUT VARCHAR2
	);
  
  /*  Baixa d'una visita concertada en un Centre Sanitari */
  PROCEDURE VISITA_B (
    pVisitaId  IN VISITA.visId%TYPE,
    oRSP      OUT VARCHAR2
	);
  
END PQ_VISITA_ABM;
/

CREATE OR REPLACE PACKAGE BODY TFC_BD.PQ_VISITA_ABM IS

    -- Variables de treball per a guardar moviments al registre d'activitat LOG_S
    l_NomUsuari        VARCHAR2(12);
    l_NomProcediment   VARCHAR2(60);
    l_ParEntrada       VARCHAR2(400);
    l_ParSortida       VARCHAR2(400);
    l_Resultat         VARCHAR2(300);
    l_DataHora         DATE;
    
  /******************************************************************************
   NOM         : VISITA_A
   
   PROPSIT    : Fa una alta d'una visita concertada en un Centre Sanitari
                                
   PARMETRES  :
      ENTRADA  :  pCentreSanitariId   : Identificador nic del Centre Sanitari
                  pNumColegiatMetge   : Nmero de Collegiat del Metge que fa la Visita
                  pNumAfiliacioUsuari : Nmero Afiliaci Sanitria de l'Usuari que es visita
                  pDataHora           : Data i Hora en que es fa la Visita
                                        (Format DD/MM/YY HH:MM')
                  pNumConsulta        : Nmero de consulta de referncia on es fa la Visita
                  pComentari          : Comentaris de la Visita

      SORTIDA  :  oRSP                : Resposta del procediment            
		
   PRECONDICI : No existeix la Visita mdica a la Base de Dades
   
   POSTCONDICI: S'ha donat d'alta la Visita
   
   RETORNA     : 'OK' si ha tingut lloc l'alta
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO D''IDENTIFICADOR DEL CENTRE SANITARI' 
                 'ERROR: ES VOL FER ALTA VISITA SENSE ESPECIFICAR DATA I HORA'
                 'ERROR: No s'ha trobat el Centre Sanitari de la Visita' 
                 'ERROR: No s'ha trobat el Metge de la Visita' 
                 'ERROR: No s'ha trobat l'Usuari de la Visita' 
                 'ERROR: No es pot crear una Visita amb les dades obligatries sense contingut'
                 'ERROR: La Visita ja havia estat donada d''alta' 
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/


  PROCEDURE VISITA_A (
    pCentreSanitariId   IN VISITA.CentreSanitariId%TYPE,
    pNumColegiatMetge   IN VARCHAR2,
    pNumAfiliacioUsuari IN VARCHAR2,
    pDataHora           IN VARCHAR2,
    pNumConsulta        IN VISITA.NumConsulta%TYPE,
    pComentari          IN VISITA.Comentari%TYPE,
    oRSP               OUT VARCHAR2
	) IS
	
    -- variables pel tractament d'excepcions
    eSenseCentreSanitari EXCEPTION;
    eSenseDataHora       EXCEPTION;
    eCentreNoExist       EXCEPTION;
    eMetgeNoExist        EXCEPTION;
    eUsuariNoExist       EXCEPTION;
    eVisitaExist         EXCEPTION;

    vMetgeId          NUMBER;
    vUsuariId         NUMBER;
    vComptador        NUMBER(3);   -- Comptador dels registres que hi ha a la base amb
                                   -- les condicions de cerca  
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'VISITA_A';
    l_ParEntrada     := 'pCentreSanitariId: '||pCentreSanitariId||
                        ', pNumColegiatMetge:'||pNumColegiatMetge||
                        ', pNumAfiliacioUsuari:'||pNumAfiliacioUsuari||
                        ', pDataHora:'||pDataHora||
                        ', pNumConsulta:'||pNumConsulta||
                        ', pComentari:'||pComentari;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. ALTA VISITA CONCERTADA EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    /* Mirar validesa dels parmetres d'entrada abans de fer la cerca a les taules
       associades */
    IF (pCentreSanitariId IS NULL OR pCentreSanitariId = '') THEN 
      RAISE eSenseCentreSanitari; 
    END IF;
    
    IF (pDataHora IS NULL OR pDataHora = '') THEN 
      RAISE eSenseDataHora; 
    END IF;
    
    -- Revisem que el Centre Sanitari existeixi a la base
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.CENTRE_SANITARI
     WHERE cesId = pCentreSanitariId;
    IF (vComptador <= 0) THEN 
      RAISE eCentreNoExist; 
    END IF;

    -- Revisem que el Metge existeixi a la base, i a ms saber-ne el seu identificador
    vMetgeId := PQ_UTILITATS.DONA_METGE_ID(UPPER(pNumColegiatMetge));
    IF (vMetgeId <= 0) THEN
      RAISE eMetgeNoExist;
    END IF;
  
    -- Revisem que l'Usuari existeixi a la base, i a ms saber-ne el seu identificador
    vUsuariId := PQ_UTILITATS.DONA_USUARI_ID(UPPER(pNumAfiliacioUsuari));
    IF (vUsuariId <= 0) THEN
      RAISE eUsuariNoExist;
    END IF;

    /* Revisar que la visita no estigui ja creada
       NOTA : 
       Noms es considera repetida si s en el mateix centre, feta pel mateix
       metge, al mateix usuari i en la mateixa data i hora.                   */
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.VISITA
     WHERE CentreSanitariId = pCentreSanitariId
       AND MetgeId          = vMetgeId
       AND UsuariId         = vUsuariId
       AND DataHora         = TO_TIMESTAMP(pDataHora);
    IF (vComptador > 0) THEN 
      RAISE eVisitaExist; 
    END IF;

    -- Execuci: Alta de la Visita concertada
    INSERT INTO TFC_BD.VISITA (CentreSanitariId, MetgeId, UsuariId, DataHora, NumConsulta, Comentari)
        VALUES (pCentreSanitariId, vMetgeId, vUsuariId, TO_TIMESTAMP(pDataHora), pNumConsulta, pComentari);

    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

  EXCEPTION
  
    WHEN eSenseCentreSanitari THEN
      l_Resultat := 'ERROR: SENSE NMERO D''IDENTIFICADOR DEL CENTRE SANITARI'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseDataHora THEN
      l_Resultat := 'ERROR: ES VOL FER ALTA VISITA SENSE ESPECIFICAR DATA I HORA'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eCentreNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat el Centre Sanitari de la Visita Concertada'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eMetgeNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat el Metge de la Visita Concertada'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eUsuariNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat l''Usuari de la Visita Concertada'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eVisitaExist THEN
      l_Resultat := 'ERROR: La Visita ja havia estat donada d''alta'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
                 
    WHEN OTHERS THEN
      ROLLBACK;
      IF (SQLCODE = -01400) THEN 
        -- S'ha incomplert una restricci de camp no nul
        l_resultat := 'ERROR: No es pot crear una Visita amb les dades obligatries sense contingut';
      ELSE
        l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      END IF;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END VISITA_A;

  /******************************************************************************
   NOM         : VISITA_M
   
   PROPSIT    : Fa una modificaci d'una Visita mdica concerta en el Sistema
                                
   PARMETRES  :
      ENTRADA  :  pVisitaId           : Identificador nic de la Visita Concertada
                  pCentreSanitariId   : Identificador nic del Centre Sanitari
                  pNumColegiatMetge   : Nmero de Collegiat del Metge que fa la Visita
                  pNumAfiliacioUsuari : Nmero Afiliaci Sanitria de l'Usuari que es visita
                  pDataHora           : Data i Hora en que es fa la Visita
                                        (Format DD/MM/YY HH:MM')
                  pNumConsulta        : Nmero de consulta de referncia on es fa la Visita
                  pComentari          : Comentaris de la Visita

      SORTIDA  :  oRSP                : Resposta del procediment            
		
		
   PRECONDICI : Existeix la Visita Concertada a la Base de Dades
   
   POSTCONDICI: S'ha fet la modificaci de la Visita Concertada
   
   RETORNA     : 'OK' si ha tingut lloc la modificaci
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE IDENTIFICADOR DE LA VISITA CONCERTADA'
                 'ERROR: NO HI HA CANVIS A FER EN LA VISITA CONCERTADA'
                 'ERROR: No s''ha trobat el Centre Sanitari per a la Visita Concertada'
                 'ERROR: No s'ha trobat el Metge assignat a la Visita Concertada'
                 'ERROR: No s'ha trobat l'Usuari citat per a la Visita Concertada'
                 'ERROR: Consulta del Centre Sanitari ocupada per una altra Visita'
                 'ERROR: Aquest Metge ja est assignat a una altra Visita Concertada'
                 'ERROR: Aquest Usuari ja est citat per una altra Visita Concertada'
                 'ERROR: La Visita Concertada no s'ha trobat a la base de dades'
                 'ERROR: <NumError><DescError>' per les altres causes
                 
   ******************************************************************************/

  PROCEDURE VISITA_M (
    pVisitaId           IN VISITA.visId%TYPE,
    pCentreSanitariId   IN VISITA.CentreSanitariId%TYPE,
    pNumColegiatMetge   IN VARCHAR2,
    pNumAfiliacioUsuari IN VARCHAR2,
    pDataHora           IN VARCHAR2,
    pNumConsulta        IN VISITA.NumConsulta%TYPE,
    pComentari          IN VISITA.Comentari%TYPE,
    oRSP               OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseIdentificador EXCEPTION;
    eSenseModificacions EXCEPTION;
    eCentreNoExist      EXCEPTION;
    eMetgeNoExist       EXCEPTION;
    eUsuariNoExist      EXCEPTION;
    eConsultaOcupada    EXCEPTION;
    eMetgeAssignat      EXCEPTION;
    eUsuariCitat        EXCEPTION;
    
    aCentreSanitariId   NUMBER;
    aNumColegiatMetge   VARCHAR2(15);
    aNumAfiliacioUsuari VARCHAR2(15);
    aDataHora           TIMESTAMP; 
    aNumConsulta        VARCHAR2(6);
    aComentari          VARCHAR2(500);
  
    vCentreSanitariId   NUMBER;
    vNumColegiatMetge   VARCHAR2(15);
    vNumAfiliacioUsuari VARCHAR2(15);
    vDataHora           TIMESTAMP; 
    vNumConsulta        VARCHAR2(6);
    vComentari          VARCHAR2(500);
    
    vMetgeId            NUMBER;
    vUsuariId           NUMBER;
    vComptador          NUMBER(3);  -- Comptador de registres que hi ha en una taula que 
                                    -- compleixen les condicions.
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'VISITA_M';
    l_ParEntrada     := 'pVisitaId: '||pVisitaId||
                        ', pCentreSanitariId: '||pCentreSanitariId||
                        ', pNumColegiatMetge:'||pNumColegiatMetge||
                        ', pNumAfiliacioUsuari:'||pNumAfiliacioUsuari||
                        ', pDataHora:'||pDataHora||
                        ', pNumConsulta:'||pNumConsulta||
                        ', pComentari:'||pComentari;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. MODIFICACIO VISITA CONCERTADA EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    IF (pVisitaId IS NULL OR pVisitaId = 0) THEN 
      RAISE eSenseIdentificador; 
    END IF;

    IF (pCentreSanitariId IS NULL OR pCentreSanitariId = 0) AND 
       (pNumColegiatMetge IS NULL OR pNumColegiatMetge = '') AND 
       (pNumAfiliacioUsuari IS NULL OR pNumAfiliacioUsuari = '') AND 
       (pDataHora IS NULL OR pDataHora = '') AND 
       (pNumConsulta IS NULL OR pNumConsulta = '') AND 
       (pComentari IS NULL OR pComentari = '') THEN 
      RAISE eSenseModificacions; 
    END IF;
  
    -- Agafa dades actuals de la Visita per contrastar-les amb les noves.
    SELECT V.CentreSanitariId, M.NumColegiat, U.NumAfiliacioSanitaria, V.DataHora, 
           V.NumConsulta, V.Comentari
      INTO aCentreSanitariId, aNumColegiatMetge, aNumAfiliacioUsuari, aDataHora, 
           aNumConsulta, aComentari
      FROM TFC_BD.VISITA V, TFC_BD.METGE M, TFC_BD.USUARI U
     WHERE V.visId    = pVisitaId
       AND V.MetgeId  = M.metId
       AND V.UsuariId = U.usuId;

    vCentreSanitariId   := pCentreSanitariId;
    vNumColegiatMetge   := UPPER(pNumColegiatMetge);
    vNumAfiliacioUsuari := UPPER(pNumAfiliacioUsuari);
    vDataHora           := TO_TIMESTAMP(pDataHora);
    vNumConsulta        := pNumConsulta;
    vComentari          := pComentari;
	
       -- Si no hi ha modificacions, hi assignem les dades actuals
    IF (vCentreSanitariId IS NULL OR vCentreSanitariId = 0) THEN
	    vCentreSanitariId := aCentreSanitariId;
    END IF;

    IF (vNumColegiatMetge IS NULL OR vNumColegiatMetge = '') THEN
	    vNumColegiatMetge := aNumColegiatMetge;
    END IF;

    IF (vNumAfiliacioUsuari IS NULL OR vNumAfiliacioUsuari = '') THEN
	    vNumAfiliacioUsuari := aNumAfiliacioUsuari;
    END IF;

    IF (vDataHora IS NULL OR vDataHora = '') THEN
	    vDataHora := aDataHora;
    END IF;

    IF (vNumConsulta IS NULL OR vNumConsulta = '') THEN
	    vNumConsulta := aNumConsulta;
    END IF;

    IF (vComentari IS NULL OR vComentari = '') THEN
	    vComentari := aComentari;
    END IF;


    -- Revisem que el Centre Sanitari de la Visita Concertada existeixi a la base
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.CENTRE_SANITARI 
     WHERE cesId = vCentreSanitariId;
    IF (vComptador <= 0) THEN 
      RAISE eCentreNoExist; 
    END IF;
 
    -- Revisem que el Metge existeixi a la base, i a ms saber-ne el seu identificador
    vMetgeId := PQ_UTILITATS.DONA_METGE_ID(UPPER(vNumColegiatMetge));
    IF (vMetgeId <= 0) THEN
      RAISE eMetgeNoExist;
    END IF;
  
    -- Revisem que l'Usuari existeixi a la base, i a ms saber-ne el seu identificador
    vUsuariId := PQ_UTILITATS.DONA_USUARI_ID(UPPER(vNumAfiliacioUsuari));
    IF (vUsuariId <= 0) THEN
      RAISE eUsuariNoExist;
    END IF;

    -- Verificar que la consulta no estigui ocupada per una altra visita.
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.VISITA 
     WHERE CentreSanitariId = vCentreSanitariId
       AND DataHora         = vDataHora
       AND NumConsulta      = vNumConsulta
       AND visId <> pVisitaId;
    IF (vComptador > 0) THEN 
      RAISE eConsultaOcupada; 
    END IF;

    -- Verificar que el Metge no estigui assignat a una altra visita.
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.VISITA 
     WHERE MetgeId  = vMetgeId
       AND DataHora = vDataHora
       AND visId <> pVisitaId;
    IF (vComptador > 0) THEN 
      RAISE eMetgeAssignat; 
    END IF;

    -- Verificar que l'Usuari no estigui citat per a una altra visita.
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.VISITA 
     WHERE UsuariId = vUsuariId
       AND DataHora = vDataHora
       AND visId <> pVisitaId;
    IF (vComptador > 0) THEN 
      RAISE eUsuariCitat; 
    END IF;

    -- Execuci: Modificaci de la Visita Concertada
    UPDATE TFC_BD.VISITA
	     SET CentreSanitariId = vCentreSanitariId,
           MetgeId          = vMetgeId,
           UsuariId         = vUsuariId,
           DataHora         = vDataHora,
           NumConsulta      = vNumConsulta,
           Comentari        = vComentari
     WHERE visId = pVisitaId;
     
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

EXCEPTION
  
    WHEN eSenseIdentificador THEN
      l_Resultat := 'ERROR: SENSE IDENTIFICADOR DE LA VISITA CONCERTADA'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
    WHEN eSenseModificacions THEN
      l_Resultat := 'ERROR: NO HI HA CANVIS A FER EN LA VISITA CONCERTADA'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
		
    WHEN eCentreNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat el Centre Sanitari per a la Visita Concertada'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eMetgeNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat el Metge assignat a la Visita Concertada'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eUsuariNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat l''Usuari citat per a la Visita Concertada'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eConsultaOcupada THEN
      l_Resultat := 'ERROR: Consulta del Centre Sanitari ocupada per una altra Visita'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eMetgeAssignat THEN
      l_Resultat := 'ERROR: Aquest Metge ja est assignat a una altra Visita Concertada'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eUsuariCitat THEN
      l_Resultat := 'ERROR: Aquest Usuari ja est citat per una altra Visita Concertada'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: La Visita Concertada no s''ha trobat a la base de dades';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      ROLLBACK;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    
  END VISITA_M;
	
  /******************************************************************************
   NOM         : VISITA_B
   
   PROPSIT    : Fa una Baixa d'una visita concertada en un Centre Sanitari
                                
   PARMETRES  :
      ENTRADA  :  pVisitaId : Identificador de la Visita mdica

      SORTIDA  :  oRSP      : Resposta del procediment            
		
   PRECONDICI : La Visita Concertada existeix a la Base de Dades
   
   POSTCONDICI: S'ha donat de baixa la Visita Concertada
   
   RETORNA     : 'OK' si ha tingut lloc la baixa
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR : SENSE IDENTIFICADOR DE LA VISITA CONCERTADA' 
                 'ERROR : La Visita Concertada no s'ha trobat a la base de dades'
                 'ERROR : No es pot esborrar la Visita per estar relacionada amb altres dades'
                 'ERROR : <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE VISITA_B (
    pVisitaId  IN VISITA.visId%TYPE,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseIdentificador EXCEPTION;
    eVisitaNoExist      EXCEPTION;

    vComptador NUMBER(3);  -- Comptador de tuples de Visites amb mateix ID (en principi noms 1)
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'VISITA_B';
    l_ParEntrada     := 'pVisitaId: '||pVisitaId;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. BAIXA VISITA CONCERTADA EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    -- Mirar validesa dels parmetres d'entrada (camps clau)
    IF (pVisitaId IS NULL OR pVisitaId = 0) THEN 
      RAISE eSenseIdentificador; 
    END IF;

    -- Verifica que la Visita Concertada existeixi
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.VISITA 
     WHERE visId = pVisitaId;
    IF (vComptador = 0) THEN 
      RAISE eVisitaNoExist; 
    END IF;

    -- Execuci: Baixa de la Visita Concertada
    DELETE FROM TFC_BD.VISITA WHERE visId = pVisitaId;

    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;
    
  EXCEPTION
    WHEN eSenseIdentificador THEN
      l_Resultat := 'ERROR : SENSE IDENTIFICADOR DE LA VISITA CONCERTADA'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eVisitaNoExist THEN
      l_Resultat := 'ERROR : La Visita Concertada no s''ha trobat a la base de dades';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      ROLLBACK;
      IF (SQLCODE = -02292) THEN 
        l_resultat := 'ERROR : No es pot esborrar la Visita per estar relacionada amb altres dades';
      ELSE
        l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      END IF;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END VISITA_B;

  
END PQ_VISITA_ABM;
/

COMMIT;  
