  /******************************************************************************
  
   ARXIU    : 2_06_MalaltiaABM.sql
   AUTOR    : Jess Hernndez Garcia - TFC - Aula 1
   
   PROPSIT : Script de creaci del paquet PQ_MALALTIA_ABM : 
              Procediments i funcions d'Altes, baixes i modificacions de la
              taula MALALTIA.
                                
   ******************************************************************************/

CREATE OR REPLACE PACKAGE TFC_BD.PQ_MALALTIA_ABM AS

  /*  Alta d'una malaltia de les controlades pel sistema */
  PROCEDURE MALALTIA_A (
    pEspecialitatCodi   IN MALALTIA.EspecialitatCodi%TYPE,
    pDescripcio         IN MALALTIA.Descripcio%TYPE,
    pNivellGravetat     IN MALALTIA.NivellGravetat%TYPE,
    oRSP               OUT VARCHAR2
	);

  /*  Modificaci d'una Malaltia de les controlades pel sistema */
  PROCEDURE MALALTIA_M (
    pMalaltiaId       IN MALALTIA.malId%TYPE,
    pDescripcio       IN MALALTIA.Descripcio%TYPE,
    oRSP            OUT VARCHAR2
	);
	
  /*  Baixa d'una Malaltia de les controlades pel sistema */
  PROCEDURE MALALTIA_B (
    pMalaltiaId IN MALALTIA.malId%TYPE,
    oRSP      OUT VARCHAR2
	);

END PQ_MALALTIA_ABM;
/

CREATE OR REPLACE PACKAGE BODY TFC_BD.PQ_MALALTIA_ABM IS

    -- Variables de treball per a guardar moviments al registre d'activitat LOG_S
    l_NomUsuari        VARCHAR2(12);
    l_NomProcediment   VARCHAR2(60);
    l_ParEntrada       VARCHAR2(400);
    l_ParSortida       VARCHAR2(400);
    l_Resultat         VARCHAR2(300);
    l_DataHora         DATE;
    
  /******************************************************************************
   NOM         : MALALTIA_A
   
   PROPSIT    : Fa una alta d'una Malaltia controlada pel Sistema de Recollida de Dades
                                
   PARMETRES  :
      ENTRADA  :  pEspecialitatCodi : Codi de l'especialitat de la Malaltia
                  pDescripcio       : Descripci de la Malaltia
                  pNivellGravetat   : Avaluaci del Nivell de Gravetat de la Malaltia

      SORTIDA  :  oRSP              : Resposta del procediment            
		
   PRECONDICI : No existeix la MALALTIA a la Base de Dades
   
   POSTCONDICI: S'ha donat d'alta la MALALTIA
   
   RETORNA     : 'OK' si ha tingut lloc l'alta
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE ESPECIFICAR ESPECIALITAT DE LA MALALTIA'
                 'ERROR: No s''ha trobat la Especialitat de la Malaltia'
                 'ERROR: La Malaltia ja havia estat donada d''alta'
                 'ERROR: No es pot crear una MALALTIA amb les dades obligatries sense contingut'
                 'ERROR: El Nivell de gravetat de la Malaltia no est dins dels possibles'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/


  PROCEDURE MALALTIA_A (
    pEspecialitatCodi   IN MALALTIA.EspecialitatCodi%TYPE,
    pDescripcio         IN MALALTIA.Descripcio%TYPE,
    pNivellGravetat     IN MALALTIA.NivellGravetat%TYPE,
    oRSP               OUT VARCHAR2
	) IS
	
    -- variables pel tractament d'excepcions
    eSenseEspecialitat   EXCEPTION;
    eEspecialitatNoExist EXCEPTION;
    eMalaltiaExist       EXCEPTION;

    vEspecialitatCodi    VARCHAR2(5);
    vComptador           NUMBER(3);   -- Comptador de registres que hi ha en una taula que 
                                      -- compleixen les condicions.
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'MALALTIA_A';
    l_ParEntrada     := 'pEspecialitatCodi: '||pEspecialitatCodi||
                        ', pDescripcio:'||pDescripcio||
                        ', pNivellGravetat:'||pNivellGravetat;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. ALTA MALALTIA EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    /* Mirar validesa dels parmetres d'entrada abans de fer la cerca a les taules
      associades */
    IF (pEspecialitatCodi IS NULL OR pEspecialitatCodi = '') THEN 
      RAISE eSenseEspecialitat; 
    END IF;
    
    vEspecialitatCodi := UPPER(pEspecialitatCodi);
    
    -- Revisem que la especialitat de la Malaltia existeixi a la base
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.ESPECIALITAT
     WHERE Codi = vEspecialitatCodi;
    IF (vComptador <= 0) THEN 
      RAISE eEspecialitatNoExist; 
    END IF;
  
    /* Revisar que la MALALTIA no estigui ja creada
       NOTA : Es considera repetida si s de la mateixa especialitat i descripci.
       MILLORES : Especificar un camp ms de la taula que sigui la codificaci 
                  i clau nica 
    */
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.MALALTIA
     WHERE EspecialitatCodi = vEspecialitatCodi
       AND Descripcio       = pDescripcio;
    IF (vComptador > 0) THEN 
      RAISE eMalaltiaExist; 
    END IF;

    -- Execuci: Alta de la MALALTIA
    INSERT INTO TFC_BD.MALALTIA (EspecialitatCodi, Descripcio, NivellGravetat)
        VALUES (vEspecialitatCodi, pDescripcio, pNivellGravetat);

    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

  EXCEPTION
  
    WHEN eSenseEspecialitat THEN
      l_Resultat := 'ERROR: SENSE ESPECIFICAR ESPECIALITAT DE LA MALALTIA'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eEspecialitatNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat la Especialitat de la Malaltia'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eMalaltiaExist THEN
      l_Resultat := 'ERROR: La Malaltia ja havia estat donada d''alta';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      ROLLBACK;
      IF (SQLCODE = -01400) THEN 
        -- S'ha incomplert una restricci de camp no nul
        l_resultat := 'ERROR: No es pot crear una MALALTIA amb les dades obligatries sense contingut';
      ELSIF (SQLCODE = -02290) THEN 
        -- S'ha incomplert la restricci del camp NivellGravetat
        l_resultat := 'ERROR: El Nivell de gravetat de la Malaltia no est dins dels possibles';
      ELSE
        l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      END IF;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END MALALTIA_A;

  /******************************************************************************
   NOM         : MALALTIA_M
   
   PROPSIT    : Fa una modificaci de la fitxa d'una Malaltia de les controlades
                 pel Sistema de Recollida de Dades. nicament es permet modificar
                 la descripci.
                                
   PARMETRES  :
      ENTRADA  :  pMalaltiaId       : Identificador nic de la Malaltia
                  pDescripcio       : Descripci de la Malaltia

      SORTIDA  :  oRSP              : Resposta del procediment            
		
   PRECONDICI : Existeix la Malaltia a la Base de Dades
   
   POSTCONDICI: S'ha fet la modificaci de la fitxa de la Malaltia
   
   RETORNA     : 'OK' si ha tingut lloc la modificaci
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE IDENTIFICADOR DE LA MALALTIA' 
                 'ERROR: NO HI HA CANVIS A FER EN LA MALALTIA'
                 'ERROR: La Malaltia no s'ha trobat a la base de dades'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE MALALTIA_M (
    pMalaltiaId       IN MALALTIA.malId%TYPE,
    pDescripcio       IN MALALTIA.Descripcio%TYPE,
    oRSP             OUT VARCHAR2
  ) IS
  
    -- variables pel tractament d'excepcions
    eSenseIdentificador  EXCEPTION;
    eSenseModificacions  EXCEPTION;
	
	  vInstruccio      NUMBER(1);

    -- Joc de variables per guardar valors actuals
    aDescripcio       VARCHAR2(100);
	
    -- Joc de variables per a gestionar valors nous
    vDescripcio       VARCHAR2(100);
    vComptador        NUMBER(3);   -- Comptador de registres que hi ha en una taula que 
                                   -- compleixen les condicions.
    
  BEGIN
 
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'MALALTIA_M';
    l_ParEntrada     := 'pMalaltiaId: '||pMalaltiaId||
                        ', pDescripcio:'||pDescripcio;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. MODIFICACIO MALALTIA EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    -- Mirar validesa dels parmetres d'entrada
    IF (pMalaltiaId IS NULL OR pMalaltiaId = 0) THEN 
      RAISE eSenseIdentificador; 
    END IF;

    IF (pDescripcio IS NULL OR pDescripcio = '') THEN 
      RAISE eSenseModificacions; 
    END IF;
  
    -- Agafa dades actuals de la malaltia per contrastar-les amb les noves.
    SELECT Descripcio
      INTO aDescripcio
      FROM TFC_BD.MALALTIA
     WHERE malId = pMalaltiaId;

    vDescripcio := pDescripcio;
	
     -- Si no hi ha modificacions, hi assignem les dades actuals
    IF (vDescripcio IS NULL OR vDescripcio = '') THEN
	    vDescripcio := aDescripcio;
    END IF;

    -- Execuci: Modificaci dLa Malaltia
    UPDATE TFC_BD.MALALTIA 
	   SET Descripcio = vDescripcio
     WHERE malId = pMalaltiaId;

    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
      
    COMMIT;

  EXCEPTION
  
    WHEN eSenseIdentificador THEN
      l_Resultat := 'ERROR: SENSE IDENTIFICADOR DE LA MALALTIA'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseModificacions THEN
      l_Resultat := 'ERROR: NO HI HA CANVIS A FER EN LA MALALTIA'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
		
    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: La Malaltia no s''ha trobat a la base de dades';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      ROLLBACK;
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);


  END MALALTIA_M;

  /******************************************************************************
   NOM         : MALALTIA_B
   
   PROPSIT    : Fa una Baixa d'una Malaltia controlada pel Sistema de Recollida de Dades
                                
   PARMETRES  :
      ENTRADA  :  pMalaltiaId : Identificador de la MALALTIA

      SORTIDA  :  oRSP        : Resposta del procediment            
		
   PRECONDICI : La Malaltia existeix a la Base de Dades
   
   POSTCONDICI: S'ha donat de baixa la Malaltia
   
   RETORNA     : 'OK' si ha tingut lloc la baixa
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE IDENTIFICADOR DE LA MALALTIA' 
                 'ERROR: La Malaltia no s'ha trobat a la base de dades'
                 'ERROR: No es pot esborrar La Malaltia per estar relacionada
                         amb altres dades'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE MALALTIA_B (
    pMalaltiaId IN MALALTIA.malId%TYPE,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseIdentificador EXCEPTION;
    eMalaltiaNoExist    EXCEPTION;

    vComptador NUMBER(3);  -- Comptador de tuples de Malalties 
                           -- amb mateix ID (en principi noms 1)
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'MALALTIA_B';
    l_ParEntrada     := 'pMalaltiaId: '||pMalaltiaId;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. BAIXA MALALTIA EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    -- Mirar validesa dels parmetres d'entrada (camps clau)
    IF (pMalaltiaId IS NULL OR pMalaltiaId = 0) THEN 
      RAISE eSenseIdentificador; 
    END IF;

    -- Verifica que la Malaltia existeixi
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.MALALTIA 
     WHERE malId = pMalaltiaId;
    IF (vComptador = 0) THEN 
      RAISE eMalaltiaNoExist; 
    END IF;
 
    -- Execuci: Baixa de la Malaltia
    DELETE FROM TFC_BD.MALALTIA WHERE malId = pMalaltiaId;

    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;
    
  EXCEPTION
  
    WHEN eSenseIdentificador THEN
      l_Resultat := 'ERROR: SENSE IDENTIFICADOR DE LA MALALTIA'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eMalaltiaNoExist THEN
      l_Resultat := 'ERROR: La Malaltia no s''ha trobat a la base de dades';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      ROLLBACK;
      IF (SQLCODE = -02292) THEN 
        l_resultat := 'ERROR: No es pot esborrar la Malaltia per estar relacionada amb altres dades';
      ELSE
        l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      END IF;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END MALALTIA_B;

END PQ_MALALTIA_ABM;
/

COMMIT;  
