  /******************************************************************************
  
   ARXIU    : 2_07_DiagnosticABM.sql
   AUTOR    : Jess Hernndez Garcia - TFC - Aula 1
   
   PROPSIT : Script de creaci del paquet PQ_DIAGNOSTIC_ABM : 
              Procediments i funcions d'Altes, baixes i modificacions de la
			        taula DIAGNOSTIC.
                                
   ******************************************************************************/

CREATE OR REPLACE PACKAGE TFC_BD.PQ_DIAGNOSTIC_ABM AS

  /*  Alta d'un diagnstic que es comunica durant una visita */
  PROCEDURE DIAGNOSTIC_A (
    pVisitaId    IN DIAGNOSTIC.VisitaId%TYPE,
    pMalaltiaId  IN DIAGNOSTIC.MalaltiaId%TYPE,
    pComentari   IN DIAGNOSTIC.Comentari%TYPE,
    oRSP        OUT VARCHAR2
	);
  
  /*  Modificaci d'un Diagnstic (exclusivament el comentari) */
  PROCEDURE DIAGNOSTIC_M (
    pDiagnosticId  IN DIAGNOSTIC.diaId%TYPE,
    pComentari     IN DIAGNOSTIC.Comentari%TYPE,
    oRSP          OUT VARCHAR2
	);

  /*  Baixa d'un Diagnstic que es comunica durant una visita  */
  PROCEDURE DIAGNOSTIC_B (
    pDiagnosticId  IN DIAGNOSTIC.diaId%TYPE,
    oRSP          OUT VARCHAR2
	);

END PQ_DIAGNOSTIC_ABM;
/

CREATE OR REPLACE PACKAGE BODY TFC_BD.PQ_DIAGNOSTIC_ABM IS

    -- Variables de treball per a guardar moviments al registre d'activitat LOG_S
    l_NomUsuari        VARCHAR2(12);
    l_NomProcediment   VARCHAR2(60);
    l_ParEntrada       VARCHAR2(400);
    l_ParSortida       VARCHAR2(400);
    l_Resultat         VARCHAR2(300);
    l_DataHora         DATE;
    
  /******************************************************************************
   NOM         : DIAGNOSTIC_A
   
   PROPSIT    : Fa una alta d'un diagnstic que es comunica durant una visita
                                
   PARMETRES  :
      ENTRADA  :  pVisitaId   : Identificador nic de la visita on es comunica el diagnstic
                  pMalaltiaId : Identificador nic de la malaltia diagnosticada
                  pComentari  : Comentaris sobre el diagnstic comunicat

      SORTIDA  :  oRSP        : Resposta del procediment            
		
   PRECONDICI : No existeix el Diagnstic a la Base de Dades
   
   POSTCONDICI: S'ha donat d'alta el Diagnstic
   
   RETORNA     : 'OK' si ha tingut lloc l'alta
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE IDENTIFICADOR DE LA VISITA'
                 'ERROR: SENSE IDENTIFICADOR DE LA MALALTIA'
                 'ERROR : No s'ha trobat la Visita del Diagnstic'; 
                 'ERROR : No s'ha trobat la Malaltia del Diagnstic' 
                 'ERROR : El Diagnstic ja havia estat donat d'alta' 
                 'ERROR : <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE DIAGNOSTIC_A (
    pVisitaId    IN DIAGNOSTIC.VisitaId%TYPE,
    pMalaltiaId  IN DIAGNOSTIC.MalaltiaId%TYPE,
    pComentari   IN DIAGNOSTIC.Comentari%TYPE,
    oRSP        OUT VARCHAR2
  ) IS
  
    -- variables pel tractament d'excepcions
    eSenseVisita     EXCEPTION;
    eSenseMalaltia   EXCEPTION;
    eVisitaNoExist   EXCEPTION;
    eMalaltiaNoExist EXCEPTION;
    eDiagnosticExist EXCEPTION;

    vNom       VARCHAR2(100);
    vAdreca    VARCHAR2(100);
    vComentari VARCHAR2(100);
    vComptador NUMBER(3);   -- Comptador dels Diagnstics de la visita que hi ha a la base
    
    vRSP       VARCHAR2(300);  -- Resultat actualitzaci estadstiques                                
BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'DIAGNOSTIC_A';
    l_ParEntrada     := 'pVisitaId: '||pVisitaId||
                        ', pMalaltiaId:'||pMalaltiaId||
                        ', pComentari:'||pComentari;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. ALTA DIAGNOSTIC EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    -- Mirar validesa dels parmetres d'entrada
    IF (pVisitaId IS NULL OR pVisitaId = '') THEN 
      RAISE eSenseVisita; 
    END IF;
    
    IF (pMalaltiaId IS NULL OR pMalaltiaId = '') THEN 
      RAISE eSenseMalaltia; 
    END IF;
    
    -- Revisem que la Visita existeixi a la base
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.VISITA
     WHERE visId = pVisitaId;
    IF (vComptador <= 0) THEN
      RAISE eVisitaNoExist;
    END IF;
  
    -- Revisem que la Malaltia existeixi a la base
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.MALALTIA
     WHERE malId = pMalaltiaId;
    IF (vComptador <= 0) THEN
      RAISE eMalaltiaNoExist;
    END IF;
  
    -- Verifica que el registre no s'hagi creat ja amb un altre ID
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.DIAGNOSTIC 
     WHERE VisitaId   = pVisitaId
       AND MalaltiaId = pMalaltiaId;
    IF (vComptador > 0) THEN 
      RAISE eDiagnosticExist; 
    END IF;
 
    -- Execuci: Alta del Diagnstic
    INSERT INTO TFC_BD.DIAGNOSTIC (VisitaId, MalaltiaId, Comentari)
        VALUES (pVisitaId, pMalaltiaId, pComentari);

    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

    -- Inserci de les dades estadstiques corresponent a l'activitat creada
    PQ_ACT_ESTADISTIQUES.INS_EST_DIAGNOSTIC (pVisitaId, pMalaltiaId, vRSP);

  EXCEPTION
    WHEN eSenseVisita THEN
      l_Resultat := 'ERROR: SENSE IDENTIFICADOR DE LA VISITA'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseMalaltia THEN
      l_Resultat := 'ERROR: SENSE IDENTIFICADOR DE LA MALALTIA'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eVisitaNoExist THEN
      l_Resultat := 'ERROR : No s''ha trobat la Visita del Diagnstic'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eMalaltiaNoExist THEN
      l_Resultat := 'ERROR : No s''ha trobat la Malaltia del Diagnstic'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eDiagnosticExist THEN
      l_Resultat := 'ERROR : El Diagnstic ja havia estat donat d''alta';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      ROLLBACK;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END DIAGNOSTIC_A;


  /******************************************************************************
   NOM         : DIAGNOSTIC_M
   
   PROPSIT    : Fa una modificaci d'un Diagnstic. Noms es considera
                 modificable el comentari,
                                
   PARMETRES  :
      ENTRADA  :  pDiagnosticId : Identificador nic del Diagnstic
                  pComentari    : Comentaris sobre el diagnstic comunicat

      SORTIDA  :  oRSP          : Resposta del procediment            
		
   PRECONDICI : Existeix el Diagnstic a la Base de Dades
   
   POSTCONDICI: S'ha fet la modificaci del comentari del Diagnstic
   
   RETORNA     : 'OK' si ha tingut lloc la modificaci
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE IDENTIFICADOR DEL DIAGNOSTIC' 
                 'ERROR: NO HI HA CANVIS A FER EN EL DIAGNOSTIC'
                 'ERROR : El Diagnstic no s'ha trobat a la base de dades'
                 'ERROR : <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE DIAGNOSTIC_M (
    pDiagnosticId  IN DIAGNOSTIC.diaId%TYPE,
    pComentari     IN DIAGNOSTIC.Comentari%TYPE,
    oRSP          OUT VARCHAR2
  ) IS
  
    -- variables pel tractament d'excepcions
    eSenseIdentificador EXCEPTION;
    eSenseModificacions EXCEPTION;
    eNoHiHaDiagnostic   EXCEPTION;
    
    vComptador NUMBER(3);   -- Comptador del diag
    
  BEGIN
 
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'DIAGNOSTIC_M';
    l_ParEntrada     := 'pDiagnosticId: '||pDiagnosticId||
                        ', pComentari: '||pComentari;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. MODIFICACIO DIAGNOSTIC EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    -- Mirar validesa dels parmetres d'entrada
    IF (pDiagnosticId IS NULL OR pDiagnosticId = 0) THEN 
      RAISE eSenseIdentificador; 
    END IF;

    IF (pComentari IS NULL OR pComentari = '') THEN 
      RAISE eSenseModificacions; 
    END IF;
  
    -- Verifica que el diagnstic estigui a la base de dades
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.DIAGNOSTIC 
     WHERE diaId = pDiagnosticId;
    IF (vComptador <> 1) THEN 
      RAISE eNoHiHaDiagnostic; 
    END IF;

    -- Execuci: Modificaci del Diagnstic
    UPDATE TFC_BD.DIAGNOSTIC 
	     SET Comentari = pComentari
     WHERE diaId = pDiagnosticId;

    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
      
    COMMIT;

  EXCEPTION
  
    WHEN eSenseIdentificador THEN
      l_Resultat := 'ERROR: SENSE IDENTIFICADOR DEL DIAGNOSTIC'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseModificacions THEN
      l_Resultat := 'ERROR: NO HI HA CANVIS A FER EN EL DIAGNOSTIC'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
		
    WHEN eNoHiHaDiagnostic THEN
      l_Resultat := 'ERROR: El Diagnstic no s''ha trobat a la base de dades';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      ROLLBACK;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END DIAGNOSTIC_M;

  /******************************************************************************
   NOM         : DIAGNOSTIC_B
   
   PROPSIT    : Fa una Baixa d'un Diagnstic que es comunica durant una visita 
                                
   PARMETRES  :
      ENTRADA  :  pDiagnosticId : Identificador nic del Diagnstic

      SORTIDA  :  oRSP          : Resposta del procediment            
		
   PRECONDICI : El Diagnstic existeix a la Base de Dades
   
   POSTCONDICI: S'ha donat de baixa el Diagnstic
   
   RETORNA     : 'OK' si ha tingut lloc la baixa
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR : SENSE IDENTIFICADOR DEL DIAGNOSTIC' 
                 'ERROR : El Diagnstic no s'ha trobat a la base de dades'
                 'ERROR : <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE DIAGNOSTIC_B (
    pDiagnosticId  IN DIAGNOSTIC.diaId%TYPE,
    oRSP          OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseIdentificador EXCEPTION;
    eDiagnosticNoExist  EXCEPTION;

    vComptador  NUMBER(3);  -- Comptador de tuples de Diagnstics 
                            -- amb mateix ID (en principi noms 1)
    
    aVisitaId   NUMBER;
    aMalaltiaId NUMBER;
    
    vRSP        VARCHAR2(300);  -- Resultat actualitzaci estadstiques                                
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'DIAGNOSTIC_B';
    l_ParEntrada     := 'pDiagnosticId: '||pDiagnosticId;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. BAIXA DIAGNOSTIC EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    -- Mirar validesa dels parmetres d'entrada (camps clau)
    IF (pDiagnosticId IS NULL OR pDiagnosticId = 0) THEN 
      RAISE eSenseIdentificador; 
    END IF;

    -- Verifica que el Diagnstic existeixi
    BEGIN
       SELECT VisitaId, MalaltiaId
         INTO aVisitaId, aMalaltiaId
         FROM TFC_BD.DIAGNOSTIC 
        WHERE diaId = pDiagnosticId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RAISE eDiagnosticNoExist; 
    END;
	
    -- Execuci: Baixa del Diagnstic
    DELETE FROM TFC_BD.DIAGNOSTIC WHERE diaId = pDiagnosticId;

    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;
    
    -- Inserci de les dades estadstiques corresponent a l'activitat creada
    PQ_ACT_ESTADISTIQUES.DEL_EST_DIAGNOSTIC (aVisitaId, aMalaltiaId, vRSP);

  EXCEPTION
  
    WHEN eSenseIdentificador THEN
      l_Resultat := 'ERROR: SENSE IDENTIFICADOR DEL DIAGNOSTIC'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eDiagnosticNoExist THEN
      l_Resultat := 'ERROR: El Diagnstic no s''ha trobat a la base de dades';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      ROLLBACK;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END DIAGNOSTIC_B;

END PQ_DIAGNOSTIC_ABM;
/

COMMIT;  
