  /******************************************************************************
  
   ARXIU    : 2_08_ActivitatABM.sql
   AUTOR    : Jess Hernndez Garcia - TFC - Aula 1
   
   PROPSIT : Script de creaci del paquet PQ_ACTIVITAT_ABM : 
              Procediments i funcions d'Altes, baixes i modificacions de la
			        taula ACTIVITAT.
                                
   ******************************************************************************/

CREATE OR REPLACE PACKAGE TFC_BD.PQ_ACTIVITAT_ABM AS

  /*  Alta d'una Activitat fsica d'un Usuari seguit pel Sistema */
  PROCEDURE ACTIVITAT_A (
    pNumAfiliacioUsuari    IN VARCHAR2,
    pDataAct               IN VARCHAR2,
    pNombrePasses          IN ACTIVITAT.NombrePasses%TYPE,
    pNombreCalories        IN ACTIVITAT.NombreCalories%TYPE,
    pNombreHoresInactives  IN ACTIVITAT.NombreHoresInactives%TYPE,
    oRSP                  OUT VARCHAR2
	);

  /*  Modificaci d'una Activitat fsica d'un Usuari seguit pel Sistema */
  PROCEDURE ACTIVITAT_M (
    pActivitatId           IN ACTIVITAT.actId%TYPE,
    pNumAfiliacioUsuari    IN VARCHAR2,
    pDataAct               IN VARCHAR2,
    pNombrePasses          IN ACTIVITAT.NombrePasses%TYPE,
    pNombreCalories        IN ACTIVITAT.NombreCalories%TYPE,
    pNombreHoresInactives  IN ACTIVITAT.NombreHoresInactives%TYPE,
    oRSP                  OUT VARCHAR2
	);

  /*  Baixa d'una Activitat fsica d'un Usuari seguit pel Sistema */

  PROCEDURE ACTIVITAT_B (
    pActivitatId  IN ACTIVITAT.actId%TYPE,
    oRSP      OUT VARCHAR2
	);
  
  
END PQ_ACTIVITAT_ABM;
/

CREATE OR REPLACE PACKAGE BODY TFC_BD.PQ_ACTIVITAT_ABM IS

    -- Variables de treball per a guardar moviments al registre d'activitat LOG_S
    l_NomUsuari        VARCHAR2(12);
    l_NomProcediment   VARCHAR2(60);
    l_ParEntrada       VARCHAR2(400);
    l_ParSortida       VARCHAR2(400);
    l_Resultat         VARCHAR2(300);
    l_DataHora         DATE;
    
  /******************************************************************************
   NOM         : ACTIVITAT_A
   
   PROPSIT    : Alta d'una Activitat fsica d'un Usuari seguit pel Sistema
                                
   PARMETRES  :
      ENTRADA  :  pNumAfiliacioUsuari   : Nmero Afiliaci Sanitria de l'Usuari que
                                          ha fet l'activitat  
                  pDataAct              : Data de l'Activitat (Format DD/MM/YY)
                  pNombrePasses         : Nombre de passes del dia fetes
                  pNombreCalories       : Nombre de calories cremades del dia
                  pNombreHoresInactives : Nombre d'hores del dia que usuari est
                                          inactiu.

      SORTIDA  :  oRSP                  : Resposta del procediment            
		
   PRECONDICI : No existeix l'Activitat de l'Usuari a la Base de Dades
   
   POSTCONDICI: S'ha donat d'alta l'Activitat
   
   RETORNA     : 'OK' si ha tingut lloc l'alta
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO D''AFILIACI DE L''USUARI' 
                 'ERROR: No s'ha trobat l'Usuari de l''Activitat' 
                 'ERROR: No es pot crear una Activitat amb les dades obligatries sense contingut'
                 'ERROR: Les hores Inactives de l''Usuari no poden ser majors a les d''un dia'
                 'ERROR: L'Activitat ja havia estat donada d'alta' 
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/


  PROCEDURE ACTIVITAT_A (
    pNumAfiliacioUsuari    IN VARCHAR2,
    pDataAct               IN VARCHAR2,
    pNombrePasses          IN ACTIVITAT.NombrePasses%TYPE,
    pNombreCalories        IN ACTIVITAT.NombreCalories%TYPE,
    pNombreHoresInactives  IN ACTIVITAT.NombreHoresInactives%TYPE,
    oRSP                  OUT VARCHAR2
	) IS
	
    -- variables pel tractament d'excepcions
    eSenseUsuari    EXCEPTION;
    eUsuariNoExist  EXCEPTION;
    eActivitatExist EXCEPTION;

    vUsuariId  NUMBER;
    vComptador NUMBER(3);   -- Comptador dels registres que hi ha a la base amb
                            -- les condicions de cerca  
    vRSP       VARCHAR2(300);  -- Resultat actualitzaci estadstiques                                
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'ACTIVITAT_A';
    l_ParEntrada     := 'pNumAfiliacioUsuari: '||pNumAfiliacioUsuari||
                        ', pDataAct:'||pDataAct||
                        ', pNombrePasses:'||pNombrePasses||
                        ', pNombreCalories:'||pNombreCalories||
                        ', pNombreHoresInactives:'||pNombreHoresInactives;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. ALTA ACTIVITAT EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    /* Mirar validesa dels parmetres d'entrada abans de fer la cerca a les taules
       associades */
    IF (pNumAfiliacioUsuari IS NULL OR pNumAfiliacioUsuari = '') THEN 
      RAISE eSenseUsuari; 
    END IF;
    
    -- Revisem que l'Usuari existeixi a la base, i a ms saber-ne el seu identificador
    vUsuariId := PQ_UTILITATS.DONA_USUARI_ID(UPPER(pNumAfiliacioUsuari));
    IF (vUsuariId <= 0) THEN
      RAISE eUsuariNoExist;
    END IF;

    /* Revisar que l'activitat no estigui ja creada
       NOTA : Noms es considera repetida si s del mateix usuari i en la mateixa data  */
    SELECT COUNT(*) 
      INTO vComptador 
      FROM TFC_BD.ACTIVITAT
     WHERE UsuariId = vUsuariId
       AND DataAct  = TO_DATE(pDataAct);
    IF (vComptador > 0) THEN 
      RAISE eActivitatExist; 
    END IF;

    -- Execuci: Alta de l'Activitat
    INSERT INTO TFC_BD.ACTIVITAT (UsuariId, DataAct, NombrePasses, NombreCalories, NombreHoresInactives)
        VALUES (vUsuariId, TO_DATE(pDataAct), pNombrePasses, pNombreCalories, pNombreHoresInactives);

    -- Execuci correcta del procediment. Actualitzem any ltima activitat de l'usuari
    UPDATE TFC_BD.USUARI
	     SET UltimAnyActivitat = extract(year from TO_DATE(pDataAct))
     WHERE usuId = vUsuariId
       AND UltimAnyActivitat < extract(year from TO_DATE(pDataAct));
       
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

    -- Inserci de les dades estadstiques corresponent a l'activitat creada
    PQ_ACT_ESTADISTIQUES.INS_EST_ACTIVITAT
      (vUsuariId, TO_DATE(pDataAct), pNombrePasses, pNombreCalories, pNombreHoresInactives, vRSP);

  EXCEPTION
  
    WHEN eSenseUsuari THEN
      l_Resultat := 'ERROR: SENSE NMERO D''AFILIACI DE L''USUARI'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eUsuariNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat l''Usuari de l''Activitat'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eActivitatExist THEN
      l_Resultat := 'ERROR: L''Activitat ja havia estat donada d''alta'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
                 
    WHEN OTHERS THEN
      ROLLBACK;
      IF (SQLCODE = -01400) THEN 
        -- S'ha incomplert una restricci de camp no nul
        l_resultat := 'ERROR: No es pot crear una Activitat amb les dades obligatries sense contingut';
      ELSIF (SQLCODE = -02290) THEN 
        -- S'ha incomplert la restricci del camp NombreHoresInactives
        l_resultat := 'ERROR: Les hores Inactives de l''Usuari no poden ser majors a les d''un dia';
      ELSE
        l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      END IF;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END ACTIVITAT_A;

  /******************************************************************************
   NOM         : ACTIVITAT_M
   
   PROPSIT    : Fa una modificaci d'una Activitat fsica d'un Usuari seguit pel Sistema
                                
   PARMETRES  :
      ENTRADA  :  pActivitatId          : Identificador nic de l''Activitat
      ENTRADA  :  pNumAfiliacioUsuari   : Nmero Afiliaci Sanitria de l'Usuari que
                                          ha fet l'activitat  
                  pDataAct              : Data de l'Activitat (Format DD/MM/YY)
                  pNombrePasses         : Nombre de passes del dia fetes
                  pNombreCalories       : Nombre de calories cremades del dia
                  pNombreHoresInactives : Nombre d'hores del dia que usuari est
                                          inactiu.

      SORTIDA  :  oRSP                  : Resposta del procediment            

   PRECONDICI : Existeix l''Activitat a la Base de Dades
   
   POSTCONDICI: S'ha fet la modificaci de l'Activitat
   
   RETORNA     : 'OK' si ha tingut lloc la modificaci
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE IDENTIFICADOR DE L''ACTIVITAT'
                 'ERROR: NO HI HA CANVIS A FER EN L''ACTIVITAT'
                 'ERROR: No s'ha trobat l'Usuari que ha fet l''Activitat'
                 'ERROR: Aquest Usuari ja t una Activitat en aquest dia'
                 'ERROR: Les hores Inactives de l''Usuari no poden ser majors a les d''un dia'
                 'ERROR: L'Activitat no s'ha trobat a la base de dades'
                 'ERROR: <NumError><DescError>' per les altres causes
                 
   ******************************************************************************/

  PROCEDURE ACTIVITAT_M (
    pActivitatId           IN ACTIVITAT.actId%TYPE,
    pNumAfiliacioUsuari    IN VARCHAR2,
    pDataAct               IN VARCHAR2,
    pNombrePasses          IN ACTIVITAT.NombrePasses%TYPE,
    pNombreCalories        IN ACTIVITAT.NombreCalories%TYPE,
    pNombreHoresInactives  IN ACTIVITAT.NombreHoresInactives%TYPE,
    oRSP                  OUT VARCHAR2
	) IS

    -- variables pel tractament d'excepcions
    eSenseIdentificador EXCEPTION;
    eSenseModificacions EXCEPTION;
    eUsuariNoExist      EXCEPTION;
    eUsuariJaEntrat     EXCEPTION;

    aNumAfiliacioUsuari   VARCHAR2(15);
    aDataAct              DATE; 
    aNombrePasses         NUMBER(6);
    aNombreCalories       NUMBER(5);
    aNombreHoresInactives NUMBER(2);
  
    vNumAfiliacioUsuari   VARCHAR2(15);
    vDataAct              DATE; 
    vNombrePasses         NUMBER(6);
    vNombreCalories       NUMBER(5);
    vNombreHoresInactives NUMBER(2);
    vUsuariId             NUMBER;
    vComptador            NUMBER(3);  -- Comptador de registres que hi ha en una taula que 
                                      -- compleixen les condicions.
    vRSP                  VARCHAR2(300);  -- Resultat actualitzaci estadstiques                                
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'ACTIVITAT_M';
    l_ParEntrada     := 'pActivitatId: '||pActivitatId||
                        ', pNumAfiliacioUsuari:'||pNumAfiliacioUsuari||
                        ', pDataAct:'||pDataAct||
                        ', pNombrePasses:'||pNombrePasses||
                        ', pNombreCalories:'||pNombreCalories||
                        ', pNombreHoresInactives:'||pNombreHoresInactives;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. MODIFICACIO ACTIVITAT EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    IF (pActivitatId IS NULL OR pActivitatId = 0) THEN 
      RAISE eSenseIdentificador; 
    END IF;

    IF (pNumAfiliacioUsuari   IS NULL OR pNumAfiliacioUsuari   = '') AND 
       (pDataAct              IS NULL OR pDataAct              = '') AND 
       (pNombrePasses         IS NULL OR pNombrePasses         = '') AND 
       (pNombreCalories       IS NULL OR pNombreCalories       = '') AND 
       (pNombreHoresInactives IS NULL OR pNombreHoresInactives = '') THEN 
      RAISE eSenseModificacions; 
    END IF;
  
    -- Agafa dades actuals de l''Activitat per contrastar-les amb les noves.
    SELECT U.NumAfiliacioSanitaria, A.DataAct, A.NombrePasses, A.NombreCalories,
           A.NombreHoresInactives
      INTO aNumAfiliacioUsuari, aDataAct, aNombrePasses, aNombreCalories,
           aNombreHoresInactives
      FROM TFC_BD.ACTIVITAT A, TFC_BD.USUARI U
     WHERE A.actId    = pActivitatId
       AND A.UsuariId = U.usuId;

    vNumAfiliacioUsuari   := UPPER(pNumAfiliacioUsuari);
    vDataAct              := pDataAct;
    vNombrePasses         := pNombrePasses;
    vNombreCalories       := pNombreCalories;
    vNombreHoresInactives := pNombreHoresInactives;
    
       -- Si no hi ha modificacions, hi assignem les dades actuals
    IF (vNumAfiliacioUsuari IS NULL OR vNumAfiliacioUsuari = '') THEN
	    vNumAfiliacioUsuari := aNumAfiliacioUsuari;
    END IF;

    IF (vDataAct IS NULL OR vDataAct = '') THEN
	    vDataAct := aDataAct;
    END IF;
    
    IF (vNombrePasses IS NULL OR vNombrePasses = 0) THEN
	    vNombrePasses := aNombrePasses;
    END IF;

    IF (vNombreCalories IS NULL OR vNombreCalories = 0) THEN
	    vNombreCalories := aNombreCalories;
    END IF;

    IF (vNombreHoresInactives IS NULL OR vNombreHoresInactives = 0) THEN
	    vNombreHoresInactives := aNombreHoresInactives;
    END IF;

    -- Revisem que l'Usuari existeixi a la base, i a ms saber-ne el seu identificador
    vUsuariId := PQ_UTILITATS.DONA_USUARI_ID(UPPER(vNumAfiliacioUsuari));
    IF (vUsuariId <= 0) THEN
      RAISE eUsuariNoExist;
    END IF;

    -- Verificar que l'Usuari nou (diferent de l'actual) no tingui ja entrada l'activitat.
    -- Tambe verificar que si es canvia de data, l'usuari no hi tingui ja l'activitat
    -- del dia entrada
    IF (vNumAfiliacioUsuari <> aNumAfiliacioUsuari) OR
       (vDataAct <> aDataAct) THEN
      SELECT COUNT(*) 
        INTO vComptador 
        FROM TFC_BD.ACTIVITAT 
       WHERE UsuariId = vUsuariId
         AND DataAct  = vDataAct
         AND actId <> pActivitatId;
      IF (vComptador > 0) THEN 
        RAISE eUsuariJaEntrat; 
      END IF;
    END IF;

    -- Execuci: Modificaci de l''Activitat
    UPDATE TFC_BD.ACTIVITAT
	     SET UsuariId             = vUsuariId,
           DataAct              = vDataAct,
           NombrePasses         = vNombrePasses,
           NombreCalories       = vNombreCalories,
           NombreHoresInactives = vNombreHoresInactives
     WHERE actId = pActivitatId;
     
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

    -- Modificaci de les dades estadstiques corresponent a l'activitat modificada
    PQ_ACT_ESTADISTIQUES.UPD_EST_ACTIVITAT
      (PQ_UTILITATS.DONA_USUARI_ID(UPPER(aNumAfiliacioUsuari)), 
       aDataAct, aNombrePasses, aNombreCalories, aNombreHoresInactives, 
       vUsuariId, vDataAct, vNombrePasses, vNombreCalories, vNombreHoresInactives, vRSP);

  EXCEPTION
  
    WHEN eSenseIdentificador THEN
      l_Resultat := 'ERROR: SENSE IDENTIFICADOR DE L''ACTIVITAT'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
    WHEN eSenseModificacions THEN
      l_Resultat := 'ERROR: NO HI HA CANVIS A FER EN L''ACTIVITAT'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
		
    WHEN eUsuariNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat l''Usuari que ha fet l''Activitat'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eUsuariJaEntrat THEN
      l_Resultat := 'ERROR: Aquest Usuari ja t una Activitat en aquest dia'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: L''Activitat no s''ha trobat a la base de dades';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      ROLLBACK;
      IF (SQLCODE = -02290) THEN 
        -- S'ha incomplert la restricci del camp NombreHoresInactives
        l_resultat := 'ERROR: Les hores Inactives de l''Usuari no poden ser majors a les d''un dia';
      ELSE
        l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      END IF;
      ROLLBACK;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  END ACTIVITAT_M;

  /******************************************************************************
   NOM         : ACTIVITAT_B
   
   PROPSIT    : Fa una Baixa d'una Activitat fsica d'un Usuari seguit pel Sistema 
                                
   PARMETRES  :
      ENTRADA  : pActivitatId : Identificador de l'Activitat d'un usuari

      SORTIDA  : oRSP         : Resposta del procediment            
		
   PRECONDICI : L'Activitat existeix a la Base de Dades
   
   POSTCONDICI: S'ha donat de baixa l''Activitat
   
   RETORNA     : 'OK' si ha tingut lloc la baixa
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR : SENSE IDENTIFICADOR DE L'ACTIVITAT' 
                 'ERROR : La Activitat no s'ha trobat a la base de dades'
                 'ERROR : <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE ACTIVITAT_B (
    pActivitatId  IN ACTIVITAT.actId%TYPE,
    oRSP         OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseIdentificador EXCEPTION;
    eActivitatNoExist   EXCEPTION;

    vComptador            NUMBER(3);  -- Comptador de tuples d'Activitats amb mateix ID (en principi noms 1)
    
    aUsuariId             NUMBER;
    aDataAct              DATE; 
    aNombrePasses         NUMBER(6);
    aNombreCalories       NUMBER(5);
    aNombreHoresInactives NUMBER(2);
    
    vRSP                  VARCHAR2(300);  -- Resultat actualitzaci estadstiques                                
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'ACTIVITAT_B';
    l_ParEntrada     := 'pActivitatId: '||pActivitatId;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. BAIXA ACTIVITAT EFECTUADA';
    l_DataHora       := SYSDATE;
    oRSP             := l_Resultat;
    
    -- Mirar validesa dels parmetres d'entrada (camps clau)
    IF (pActivitatId IS NULL OR pActivitatId = 0) THEN 
      RAISE eSenseIdentificador; 
    END IF;

    -- Verifica que l'Activitat existeixi
    BEGIN
      SELECT UsuariId, DataAct, NombrePasses, NombreCalories, NombreHoresInactives
        INTO aUsuariId, aDataAct, aNombrePasses, aNombreCalories, aNombreHoresInactives 
        FROM TFC_BD.ACTIVITAT 
       WHERE actId = pActivitatId;     
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RAISE eActivitatNoExist; 
    END;

    -- Execuci: Baixa de l''Activitat
    DELETE FROM TFC_BD.ACTIVITAT WHERE actId = pActivitatId;

    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;
    
    -- Descomptar de les dades estadstiques l'activitat esborrada
    PQ_ACT_ESTADISTIQUES.DEL_EST_ACTIVITAT
      (aUsuariId, TO_DATE(aDataAct), aNombrePasses, aNombreCalories, aNombreHoresInactives, vRSP);

  EXCEPTION
    WHEN eSenseIdentificador THEN
      l_Resultat := 'ERROR : SENSE IDENTIFICADOR DE L''ACTIVITAT'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eActivitatNoExist THEN
      l_Resultat := 'ERROR : L''Activitat no s''ha trobat a la base de dades';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      ROLLBACK;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END ACTIVITAT_B;
  
END PQ_ACTIVITAT_ABM;
/

COMMIT;  
