  /******************************************************************************
  
   ARXIU    : 2_09_Consultes.sql
   AUTOR    : Jess Hernndez Garcia - TFC - Aula 1
   
   PROPSIT : Script de creaci del paquet PQ_CONSULTES : 
              Procediments i funcions de Consultes
    
   ******************************************************************************/

CREATE OR REPLACE PACKAGE TFC_BD.PQ_CONSULTES AS

  /*  Consulta Llistat de Ciutadans (Usuaris del sistema) d'un pas */
  PROCEDURE LLISTAT_USUARIS (
    pAny              IN NUMBER,
    pCodiPais         IN PAIS.Codi%TYPE,
    pEdat             IN NUMBER,
    pLlindarCalories  IN NUMBER,
    oRSP             OUT VARCHAR2
	);
  
  /* Consulta Dades Anuals Ciutad (Usuari del sistema) per mesos */
  PROCEDURE LLISTAT_MESOS_USUARI (
    pNumAfiliacioUsuari IN USUARI.NumAfiliacioSanitaria%TYPE,
    pAny                IN NUMBER,
    oRSP               OUT VARCHAR2
	);
  
  /*  Consulta Llistat de Pasos de la Uni Europea */
  PROCEDURE LLISTAT_PAISOS (
    pAny   IN NUMBER,
    pEdat  IN NUMBER,
    oRSP  OUT VARCHAR2
	);
  
END PQ_CONSULTES;
/

CREATE OR REPLACE PACKAGE BODY TFC_BD.PQ_CONSULTES IS

    -- Variables de treball per a guardar moviments al registre d'activitat LOG_S
    l_NomUsuari        VARCHAR2(12);
    l_NomProcediment   VARCHAR2(60);
    l_ParEntrada       VARCHAR2(400);
    l_ParSortida       VARCHAR2(400);
    l_Resultat         VARCHAR2(300);
    l_DataHora         DATE;
    
  /******************************************************************************
   NOM         : LLISTAT_USUARIS
   
   PROPSIT    : Mostrar llistat de tots els ciutadans d'un pas, que tinguin 
                 una determinada edat, on s'inclogui per a un any concret la 
                 segent informaci :
                 - El nombre de visites al metge realitzades pel ciutada
                 - El nombre mitj de calories gastades cada dia 
                   (sempre i quan aquest nombre mitj de calories estigui per sota 
                   del llindar de calories passat per parmetre)
                 - Any de diagnstic de malaltia cardaca
                 - Any de diagnstic de malaltia cancergena
                 - Any de diagnstic de malaltia neurolgica degenerativa
                 - Nombre de malalties lleus diagnosticades
                                
   PARMETRES  :
      ENTRADA  : pAny             : Any a consultar
                 pCodiPais        : Codi alfanumric del pas a consultar
                 pEdat            : Edat dels ciutadans a consultar
                 pLlindarCalories : Llindar de Calories a consultar
      
      SORTIDA  : oRSP             : Resposta del procediment            
		
   
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO D'ANY A CONSULTAR [R3] a' 
                 'ERROR: SENSE CODI DE PAIS A CONSULTAR [R3] a' 
                 'ERROR: SENSE EDAT A CONSULTAR [R3] a' 
                 'ERROR : No s'ha trobat el Pas per a fer la consulta [R3] a' 
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/
   
  PROCEDURE LLISTAT_USUARIS (
    pAny              IN NUMBER,
    pCodiPais         IN PAIS.Codi%TYPE,
    pEdat             IN NUMBER,
    pLlindarCalories  IN NUMBER,
    oRSP             OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseAny     EXCEPTION;
    eSensePais    EXCEPTION;
    eSenseEdat    EXCEPTION;
    eSenseLlindar EXCEPTION;
    ePaisNoExist  EXCEPTION;

    vPaisId NUMBER;      -- Identificador del Pas que es vol consultar
    
    vNomPais       VARCHAR2(100);
    vUsuariId      NUMBER;
    vNomUsuari     VARCHAR2(50);
    vCognomsUsuari VARCHAR2(100);
    vNumAfilUsuari VARCHAR2(15);
    
    vVisites       NUMBER; 
    vMitjCalDia    NUMBER;
    vAnyCAR        VARCHAR(4);
    vAnyCAN        VARCHAR(4);
    vAnyDEG        VARCHAR(4);
    vSumaMalLleus  NUMBER;
    
    vNumCiutadans  NUMBER;
    vTotalVisites  NUMBER;
    vTotalMitjCal  NUMBER;
    vTotalMLleus   NUMBER;
  
    -- Cursor per a retornar els usuaris (ciutadans) d'un pas fins a una edat.
    CURSOR C_USUARIS_PAIS(parPaisId NUMBER, parEdat NUMBER)
    IS
        -- Simplifiquem cerca de l'edat del ciutad noms tractant els anys
        -- (sense mirar mes i dia).
        SELECT C.ciuId, C.Nom, C.Cognoms, U.NumAfiliacioSanitaria
          FROM TFC_BD.CIUTADA C, TFC_BD.USUARI U
         WHERE C.PaisId = parPaisId
           AND C.ciuId = U.usuId
           AND EXTRACT(YEAR FROM SYSDATE) - EXTRACT(YEAR FROM DataNaixement) = parEdat
      ORDER BY Cognoms, Nom;
      
BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'LLISTAT_USUARIS';
    l_ParEntrada     := 'pAny: '||pAny||
                        ', pCodiPais : '||pCodiPais||
                        ', pEdat : '||pEdat||
                        ', pLlindarCalories : '||pLlindarCalories;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA LLISTAT USUARIS [R3]a EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    IF (pCodiPais IS NULL OR pCodiPais = '') THEN 
      RAISE eSensePais; 
    END IF;

    IF (pEdat IS NULL OR pEdat = 0) THEN 
      RAISE eSenseEdat; 
    END IF;

    IF (pLlindarCalories IS NULL OR pLlindarCalories = 0) THEN 
      RAISE eSenseLlindar; 
    END IF;

    -- Revisem que el Pas existeixi a la base, i a ms saber-ne el seu identificador
    vPaisId := PQ_UTILITATS.DONA_PAIS_ID(UPPER(pCodiPais));
    IF (vPaisId <= 0) THEN
      RAISE ePaisNoExist;
    END IF;
  
    SELECT Nom
      INTO vNomPais
      FROM TFC_BD.PAIS
     WHERE paiId = vPaisId; 

    -- Capalera estandard de les consultes
    PQ_UTILITATS.CAPCALERA_CONSULTA('LLISTAT HISTORIAL CIUTADANS D''UN PAS',
                                    'Del Pas : '|| pCodiPais || ' - ' || vNomPais|| 
                                    ' Any : ' || pAny ||
                                    ' Edat : ' || pEdat || ' anys ' ||
                                    ' Llindar Calories : ' || pLlindarCalories);  
    
    -- Ttols de les columnes
    DBMS_OUTPUT.PUT_LINE(RPAD('Cognoms i Nom',30) ||
                         LPAD('Visites',10) || ' ' ||
                         LPAD('Mitj.Cal.Gas/Dia',16) || ' ' ||
                         LPAD('Any CAR',7) || ' ' ||
                         LPAD('Any CAN',7) || ' ' ||
                         LPAD('Any DEG',7) || ' ' ||
                         LPAD('N Mal.Lleus',12));
    DBMS_OUTPUT.PUT_LINE(RPAD('-',29, '-') || ' ' ||
                         '---------- ' ||
                         '---------------- ' ||
                         '------- ' ||
                         '------- ' ||
                         '------- ' ||
                         '------------');
    
    vNumCiutadans := 0;                     
    vTotalVisites := 0;
    vTotalMitjCal := 0;
    vTotalMLleus  := 0;
 
    OPEN C_USUARIS_PAIS(vPaisId, pEdat);
    LOOP
      FETCH C_USUARIS_PAIS INTO vUsuariId, vNomUsuari, vCognomsUsuari, vNumAfilUsuari;
      EXIT WHEN C_USUARIS_PAIS%NotFound;

      -- Comptar les visites del ciutad en aquest any
      BEGIN
        SELECT COUNT(*)
          INTO vVisites
          FROM TFC_BD.VISITA
         WHERE EXTRACT(YEAR FROM DataHora) = pAny
           AND UsuariId = vUsuariId;

      -- No hi ha cap visita del ciutad en aquest any
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           vVisites := 0;
      END;
      
      -- Calcular el nombre mitj de calories gastades cada dia en l'any
      BEGIN
          SELECT AVG(NombreCalories)
            INTO vMitjCalDia
            FROM TFC_BD.ACTIVITAT
           WHERE UsuariId = vUsuariId
             AND EXTRACT(YEAR FROM DataAct) = pAny
          HAVING AVG(NombreCalories) <= pLlindarCalories
        GROUP BY UsuariId, EXTRACT(YEAR FROM DataAct);

      -- No hi ha cap activitat del ciutad en aquest any
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           vMitjCalDia := 0;
      END;
      
      -- Cercar any de la primera malaltia cardaca
      BEGIN
        SELECT TO_CHAR(EXTRACT(YEAR FROM V.DataHora))
          INTO vAnyCAR
          FROM TFC_BD.VISITA V, TFC_BD.DIAGNOSTIC D, TFC_BD.MALALTIA M 
         WHERE V.UsuariId         = vUsuariId
           AND V.visId            = D.VisitaId
           AND D.MalaltiaId       = M.malId
           AND M.EspecialitatCodi = 'CARDI'
           AND ROWNUM             = 1; -- Per agafar noms el primer Any de diagnosi

      -- No hi ha cap activitat del ciutad en aquest any
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           vAnyCAR := ' ';
      END;
    
      -- Cercar any de la primera malaltia cancergena
      BEGIN
        SELECT TO_CHAR(EXTRACT(YEAR FROM V.DataHora))
          INTO vAnyCAN
          FROM TFC_BD.VISITA V, TFC_BD.DIAGNOSTIC D, TFC_BD.MALALTIA M 
         WHERE V.UsuariId         = vUsuariId
           AND V.visId            = D.VisitaId
           AND D.MalaltiaId       = M.malId
           AND M.EspecialitatCodi = 'ONCOL'
           AND ROWNUM             = 1; -- Per agafar noms el primer Any de diagnosi

      -- No hi ha cap activitat del ciutad en aquest any
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           vAnyCAN := ' ';
      END;
    
      -- Cercar any de la primera malaltia neurolgica degenerativa (de nivell Greu)
      BEGIN
        SELECT TO_CHAR(EXTRACT(YEAR FROM V.DataHora))
          INTO vAnyDEG
          FROM TFC_BD.VISITA V, TFC_BD.DIAGNOSTIC D, TFC_BD.MALALTIA M 
         WHERE V.UsuariId         = vUsuariId
           AND V.visId            = D.VisitaId
           AND D.MalaltiaId       = M.malId
           AND M.EspecialitatCodi = 'NEURO'
           AND M.NivellGravetat   = 'Greu'
           AND ROWNUM             = 1; -- Per agafar noms el primer Any de diagnosi

        -- No hi ha cap activitat del ciutad en aquest any
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           vAnyDEG := ' ';
      END;
    
      -- Comptar nombre de malalties lleus del ciutad en l'any
      BEGIN
        SELECT COUNT(*)
          INTO vSumaMalLleus
          FROM TFC_BD.VISITA V, TFC_BD.DIAGNOSTIC D, TFC_BD.MALALTIA M 
         WHERE V.UsuariId   = vUsuariId
           AND EXTRACT(YEAR FROM V.DataHora) = pAny
           AND V.visId                       = D.VisitaId
           AND D.MalaltiaId                  = M.malId
           AND M.NivellGravetat              = 'Lleu'
      GROUP BY V.UsuariId, EXTRACT(YEAR FROM V.DataHora);

      -- No hi ha cap activitat del ciutad en aquest any
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           vSumaMalLleus := 0;
      END;
      
    
      DBMS_OUTPUT.PUT_LINE(
              RPAD(vCognomsUsuari || ', ' || vNomUsuari,30) ||
              LPAD(TO_CHAR(vVisites,'9G999G990'),10) || ' ' ||
              LPAD(TO_CHAR(vMitjCalDia,'999G999G990'),16) || ' ' ||
              LPAD(vAnyCAR,7) || ' ' ||
              LPAD(vAnyCAN,7) || ' ' ||
              LPAD(vAnyDEG,7) || ' ' ||
              LPAD(TO_CHAR(vSumaMalLleus,'9G999G990'),12));
              
      vTotalVisites := vTotalVisites + vVisites;
      vTotalMitjCal := vTotalMitjCal + vMitjCalDia;
      IF vMitjCalDia > 0 THEN
        vNumCiutadans := vNumCiutadans + 1;
      END IF;
      
      vTotalMLleus := vTotalMLleus + vSumaMalLleus; 
      
    END LOOP;
    
    CLOSE C_USUARIS_PAIS;
    
    -- Sumatoris	
    DBMS_OUTPUT.PUT_LINE(RPAD(' ',30, ' ') ||
                         '---------- ' ||
                         '---------------- ' ||
                         '        ' ||
                         '        ' ||
                         '        ' ||
                         '------------');
    
    -- Evitar divisi per zero
    IF vNumCiutadans = 0 THEN
      vNumCiutadans := 1;
    END IF;
                         
    DBMS_OUTPUT.PUT_LINE(RPAD(' ',30, ' ') ||
            LPAD(TO_CHAR(vTotalVisites,'9G999G990'),10) || ' ' ||
            LPAD(TO_CHAR(vTotalMitjCal/vNumCiutadans,'999G999G990'),16) ||
            RPAD(' ',25, ' ') ||
            LPAD(TO_CHAR(vTotalMLleus,'9G999G990'),12));
      
    DBMS_OUTPUT.PUT_LINE('');                             
    DBMS_OUTPUT.PUT_LINE('');                             
    

    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

  EXCEPTION
  
    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R3] a'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSensePais THEN
      l_Resultat := 'ERROR: SENSE CODI DE PAIS A CONSULTAR [R3] a'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseEdat THEN
      l_Resultat := 'ERROR: SENSE EDAT A CONSULTAR [R3] a'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseLlindar THEN
      l_Resultat := 'ERROR: SENSE LLINDAR DE CALORIES A CONSULTAR [R3] a'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN ePaisNoExist THEN
      l_Resultat := 'ERROR : No s''ha trobat el Pas per a fer la consulta [R3] a'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END LLISTAT_USUARIS;
	
  /******************************************************************************
   NOM         : LLISTAT_MESOS_USUARI
   
   PROPSIT    : Mostrar llistat de tots els mesos d'un any d'un usuari, on per 
                 cada mes es vegi :
                 - El nombre de passes realitzades
                 - El nombre mitj de calories gastades per dia
                 - El nombre d'hores de poca activitat
                                
   PARMETRES  :
      ENTRADA  : pNumAfiliacioUsuari : Nmero d'afiliaci sanitria que 
                                       identifica l'usuari a consultar
                 pAny                : Any a consultar

      SORTIDA  : oRSP                : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO D'AFILIACI DE L'USUARI A CONSULTAR [R3] b' 
                 'ERROR: SENSE NMERO D'ANY A CONSULTAR [R3] b' 
                 'ERROR: ERROR: No s''ha trobat l'Usuari a Consultar [R3] b'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/
   
  PROCEDURE LLISTAT_MESOS_USUARI (
    pNumAfiliacioUsuari IN USUARI.NumAfiliacioSanitaria%TYPE,
    pAny                IN NUMBER,
    oRSP               OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseUsuari    EXCEPTION;
    eSenseAny       EXCEPTION;
    eUsuariNoExist  EXCEPTION;

    vUsuariId                NUMBER;
    vNomUsuari               VARCHAR2(50);
    vCognomsUsuari           VARCHAR2(100);
    vNumMes                  NUMBER;
    vSumaPasses              NUMBER;
    vMitjanaCalories         NUMBER;
    vSumaHoresPocaActivitat  NUMBER;
    vTotalPasses             NUMBER;
    vTotalMitjanaCalories    NUMBER;
    vTotalMesos              NUMBER;
    vTotalHoresPocaActivitat NUMBER;
     
    -- Cursor per a retornar les dades d'activitat d'un usuari en un mes.
    CURSOR C_ACTIVITAT_MES(parUsuariId NUMBER, parANY NUMBER, parMES NUMBER)
    IS
        SELECT SUM(NombrePasses), SUM(NombreCalories)/COUNT(*),
               SUM(NombreHoresInactives) 
          FROM TFC_BD.ACTIVITAT  
         WHERE UsuariId = parUsuariId
           AND EXTRACT(YEAR FROM DataAct)  = parANY
           AND EXTRACT(MONTH FROM DataAct) = parMES
      GROUP BY UsuariId;
      
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'LLISTAT_MESOS_USUARI';
    l_ParEntrada     := 'pNumAfiliacioUsuari : '||pNumAfiliacioUsuari||
                        ', pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA LLISTAT MESOS USUARI [R3]b EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pNumAfiliacioUsuari IS NULL OR pNumAfiliacioUsuari = '') THEN 
      RAISE eSenseUsuari; 
    END IF;    

    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

  	-- Revisem que l'Usuari existeixi a la base, i a ms saber-ne el seu identificador
    vUsuariId := PQ_UTILITATS.DONA_USUARI_ID(UPPER(pNumAfiliacioUsuari));
    IF (vUsuariId <= 0) THEN
      RAISE eUsuariNoExist;
    END IF;

    SELECT Nom, Cognoms
      INTO vNomUsuari, vCognomsUsuari
      FROM TFC_BD.CIUTADA
     WHERE ciuId = vUsuariId; 

    vNumMes := 1;
    vTotalPasses             := 0;
    vTotalMitjanaCalories    := 0;
    vTotalMesos              := 0;
    vTotalHoresPocaActivitat := 0;
    
    -- Capalera estandard de les consultes
    PQ_UTILITATS.CAPCALERA_CONSULTA('LLISTAT ACTIVITAT PER MESOS',
                                    'De l''Usuari : '|| pNumAfiliacioUsuari || ' - '||
                                    vNomUsuari || ' ' || vCognomsUsuari || 
                                    '   l''Any : ' || pAny );  
    
    -- Ttols de les columnes
    DBMS_OUTPUT.PUT_LINE(LPAD('Mes',4) || ' ' ||
                         LPAD('Suma Passes',18) || ' ' ||
                         LPAD('Mitj.Cal.Gastades',18) || ' ' ||
                         LPAD('Hores poca Activ.',18));
    DBMS_OUTPUT.PUT_LINE('---- ' ||
                         '------------------ ' ||
                         '------------------ ' ||
                         '------------------');
 
    WHILE (vNumMes <= 12) LOOP
      OPEN C_ACTIVITAT_MES(vUsuariId, pAny, vNumMes);
      FETCH C_ACTIVITAT_MES INTO vSumaPasses, vMitjanaCalories, vSumaHoresPocaActivitat;
      
      -- Pot ser que no hi hagi res d'activitat d'un mes 
      IF C_ACTIVITAT_MES%Found THEN
      
        DBMS_OUTPUT.PUT_LINE(
                LPAD(TO_CHAR(vNumMes,'90'),4) || ' ' ||
                LPAD(TO_CHAR(vSumaPasses,'9G999G990'),18) || ' ' ||
                LPAD(TO_CHAR(vMitjanaCalories,'9G999G990'),18) || ' ' ||
                LPAD(TO_CHAR(vSumaHoresPocaActivitat,'9G999G990'),18));
      
        vTotalPasses             := vTotalPasses             + vSumaPasses;
        vTotalMitjanaCalories    := vTotalMitjanaCalories    + vMitjanaCalories;
        vTotalMesos              := vTotalMesos              + 1;
        vTotalHoresPocaActivitat := vTotalHoresPocaActivitat + vSumaHoresPocaActivitat;
        
      END IF;
      
      CLOSE C_ACTIVITAT_MES; 
      
      vNumMes := vNumMes + 1;    
    END LOOP;

    -- Sumatoris	
    DBMS_OUTPUT.PUT_LINE('     ' ||
                         '------------------ ' ||
                         '------------------ ' ||
                         '------------------');
                         
    -- Evitar divisi per zero
    IF vTotalMesos = 0 THEN
      vTotalMesos := 1;
    END IF;
    
    DBMS_OUTPUT.PUT_LINE('     ' ||
            LPAD(TO_CHAR(vTotalPasses,'9G999G990'),18) || ' ' ||
            LPAD(TO_CHAR(vTotalMitjanaCalories/vTotalMesos,'9G999G990'),18) || ' ' ||
            LPAD(TO_CHAR(vTotalHoresPocaActivitat,'9G999G990'),18));
      
    DBMS_OUTPUT.PUT_LINE('');                             
    DBMS_OUTPUT.PUT_LINE('');                             
    
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

  EXCEPTION

    WHEN eSenseUsuari THEN
      l_Resultat := 'ERROR: SENSE NMERO D''AFILIACI DE L''USUARI A CONSULTAR [R3] b'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R3] b'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eUsuariNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat l''Usuari a Consultar [R3] b'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
        
  END LLISTAT_MESOS_USUARI;
  
  /******************************************************************************
   NOM         : LLISTAT_PAISOS
   
   PROPSIT    : Mostrar un llistat dels pasos que conformen la UE, on per a 
                 cada pas en un any determinat es doni la segent informaci :
                 - El nombre mitj de passes diries que han realitzat els seus
                   ciutadans, sempre que la seva edat sigui menor o igual a la 
                   passada per parmetre
                 - El nombre mitj de calories gastades per dia pels seus ciutadans
                 - El nombre mitj d'hores de poca activitat dels seus ciutadans
                   (Suma de totes les hores de baixa activitat dividides per suma 
                    de dies-persona que s'hagin registrat)
                                
                    
         - Quan es parla de 'nombre mitj' sempre es refereix al conjunt de dades d'un
           any determinat, o d'un mes per la consulta B. Fins aqui, tot b. La pregunta
           s: La mitja s'ha de fer sobre els 365 dies de l'any (o els dies del mes), o
           noms sobre el nombre de dies que tinguin dades? Per exemple, si un ciutad 
           noms t dades 15 dies a l'any, la mitja s dividint pels 15 dies o
           pels 365 dies de l'any? Ho dic per qu a la resposta del consultor es parla
           dels "dies-persona" i voldria aclarir aquest terme.
           Resposta : Noms sobre els dies que hi hagi dades.
         
           
- "(cal filtrar per l'edat i l'any)".
                                
   PARMETRES  :
      ENTRADA  : pAny  : Any a consultar
                 pEdat : Edat lmit dels ciutadans a consultar
                 
      SORTIDA  :  oRSP : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO D'ANY A CONSULTAR [R3] c' 
                 'ERROR: SENSE EDAT A CONSULTAR [R3] c' 
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE LLISTAT_PAISOS (
    pAny   IN NUMBER,
    pEdat  IN NUMBER,
    oRSP  OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseAny  EXCEPTION;
    eSenseEdat EXCEPTION;

    vNomPais     VARCHAR2(100);
    vCodiPais    VARCHAR2(3);
    vNumPasses   NUMBER;
    vNumCalories NUMBER;
    vNumHPocaAct NUMBER;
    
    -- Cursor per a retornar tots els pasos que formen la uni.
    CURSOR C_PAISOS_UNIO(parAny NUMBER, parEdat NUMBER)
    IS
        SELECT P.Nom, P.Codi, AVG(A.NombrePasses), 
               AVG(A.NombreCalories), AVG(A.NombreHoresInactives)
          FROM TFC_BD.PAIS P, TFC_BD.CIUTADA C, TFC_BD.ACTIVITAT A
         WHERE P.paiId                            =  C.PaisId  
           AND EXTRACT(YEAR FROM SYSDATE) - 
               EXTRACT(YEAR FROM C.DataNaixement) <= parEdat
           AND A.UsuariId                         =  C.ciuId
           AND EXTRACT(YEAR FROM A.DataAct)       =  parAny
      GROUP BY P.Nom, P.Codi
      ORDER BY P.Nom;
      
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'LLISTAT_PAISOS';
    l_ParEntrada     := 'pAny: '||pAny||
                        ', pEdat : '||pEdat;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA LLISTAT PASOS [R3]c EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    IF (pEdat IS NULL OR pEdat = 0) THEN 
      RAISE eSenseEdat; 
    END IF;
    
    
    -- Capalera estandard de les consultes
    PQ_UTILITATS.CAPCALERA_CONSULTA('LLISTAT ACTIVITAT MITJANA CIUTADANS PER PASOS',
                                    'De l''Any : ' || pAny ||
                                    '    Edat lmit : ' || pEdat);  
    
    -- Ttols de les columnes
    DBMS_OUTPUT.PUT_LINE(RPAD('Nom Pas',24) ||
                         'Codi ' ||
                         LPAD('Mitj.Passes/Dia',16) || ' ' ||
                         LPAD('Mitj.Cal.Gas/Dia',16) || ' ' ||
                         LPAD('Mitj.H.P.Ac./Dia',16));
    DBMS_OUTPUT.PUT_LINE(RPAD('-',23, '-') || ' ' ||
                         '---- ' ||
                         '---------------- ' ||
                         '---------------- ' ||
                         '---------------- ');
                         
    OPEN C_PAISOS_UNIO(pAny, pEdat);
    LOOP
      FETCH C_PAISOS_UNIO INTO vNomPais, vCodiPais, vNumPasses, vNumCalories, vNumHPocaAct;
      EXIT WHEN C_PAISOS_UNIO%NotFound;
      
      DBMS_OUTPUT.PUT_LINE(
              RPAD(vNomPais, 24) || ' ' ||
              RPAD(vCodiPais, 4) ||
              LPAD(TO_CHAR(vNumPasses  ,'999G999G990'),16) || ' ' ||
              LPAD(TO_CHAR(vNumCalories,'999G999G990'),16) || ' ' ||
              LPAD(TO_CHAR(vNumHPocaAct,'999G999G990'),16));
              
    END LOOP;
    
    CLOSE C_PAISOS_UNIO;
    
    DBMS_OUTPUT.PUT_LINE('');                             
    DBMS_OUTPUT.PUT_LINE('');                             
    
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

  EXCEPTION
  
    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R3] c'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseEdat THEN
      l_Resultat := 'ERROR: SENSE EDAT A CONSULTAR [R3] c'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END LLISTAT_PAISOS;
  
  
END PQ_CONSULTES;
/

COMMIT;  
