/******************************************************************************

 ARXIU    : 2_10_Estadistiques.sql
 AUTOR    : Jess Hernndez Garcia - TFC - Aula 1
 
 PROPSIT : Script de creaci del paquet PQ_ACT_ESTADISTIQUES on estan definits 
            els procediments d'actualitzaci d'estadstiques,i del paquet 
            PQ_CON_ESTADISTIQUES on estan definits els procediments de consulta.
                              
 ******************************************************************************/

/*******************************************************************************
    PAQUET PER A ACTUALITZACI D'ESTADSTIQUES
*******************************************************************************/

CREATE OR REPLACE PACKAGE TFC_BD.PQ_ACT_ESTADISTIQUES AS


  /* Inserci dades diagnstic a taules estadstiques */
  PROCEDURE INS_EST_DIAGNOSTIC (
    pVisitaId    IN DIAGNOSTIC.VisitaId%TYPE,
    pMalaltiaId  IN DIAGNOSTIC.MalaltiaId%TYPE,
    oRSP        OUT VARCHAR2
  );

  /* Descomptat dades diagnstic a taules estadstiques */
  PROCEDURE DEL_EST_DIAGNOSTIC (
    pVisitaId    IN DIAGNOSTIC.VisitaId%TYPE,
    pMalaltiaId  IN DIAGNOSTIC.MalaltiaId%TYPE,
    oRSP        OUT VARCHAR2
  );

  /* Inserci dades activitat diria a taules estadstiques */
  PROCEDURE INS_EST_ACTIVITAT (
    pUsuariId              IN ACTIVITAT.UsuariId%TYPE,
    pDataAct               IN ACTIVITAT.DataAct%TYPE,
    pNombrePasses          IN ACTIVITAT.NombrePasses%TYPE,
    pNombreCalories        IN ACTIVITAT.NombreCalories%TYPE,
    pNombreHoresInactives  IN ACTIVITAT.NombreHoresInactives%TYPE,
    oRSP                  OUT VARCHAR2
  );

  /* Modificaci dades activitat d'un dia a taules estadstiques */
  PROCEDURE UPD_EST_ACTIVITAT (
    pOLDUsuariId              IN ACTIVITAT.UsuariId%TYPE,
    pOLDDataAct               IN ACTIVITAT.DataAct%TYPE,
    pOLDNombrePasses          IN ACTIVITAT.NombrePasses%TYPE,
    pOLDNombreCalories        IN ACTIVITAT.NombreCalories%TYPE,
    pOLDNombreHoresInactives  IN ACTIVITAT.NombreHoresInactives%TYPE,
    pUsuariId                 IN ACTIVITAT.UsuariId%TYPE,
    pDataAct                  IN ACTIVITAT.DataAct%TYPE,
    pNombrePasses             IN ACTIVITAT.NombrePasses%TYPE,
    pNombreCalories           IN ACTIVITAT.NombreCalories%TYPE,
    pNombreHoresInactives     IN ACTIVITAT.NombreHoresInactives%TYPE,
    oRSP                     OUT VARCHAR2
  );

  /* Descomptar dades activitat diria borrada a taules estadstiques */
  PROCEDURE DEL_EST_ACTIVITAT (
    pUsuariId              IN ACTIVITAT.UsuariId%TYPE,
    pDataAct               IN ACTIVITAT.DataAct%TYPE,
    pNombrePasses          IN ACTIVITAT.NombrePasses%TYPE,
    pNombreCalories        IN ACTIVITAT.NombreCalories%TYPE,
    pNombreHoresInactives  IN ACTIVITAT.NombreHoresInactives%TYPE,
    oRSP                  OUT VARCHAR2
  );
  
END PQ_ACT_ESTADISTIQUES;
/

CREATE OR REPLACE PACKAGE BODY TFC_BD.PQ_ACT_ESTADISTIQUES IS

    -- Variables de treball per a guardar moviments al registre d'activitat LOG_S
    l_NomUsuari        VARCHAR2(12);
    l_NomProcediment   VARCHAR2(60);
    l_ParEntrada       VARCHAR2(400);
    l_ParSortida       VARCHAR2(400);
    l_Resultat         VARCHAR2(300);
    l_DataHora         DATE;
    
    
  /******************************************************************************
   NOM         : INS_EST_DIAGNOSTIC
   
   PROPSIT    : Inserci dades diagnstic a taules estadstiques
   
   PARMETRES  :
      ENTRADA  :  pVisitaId   : Identificador nic de la visita on s'ha fet diagnosi
                  pMalaltiaId : Identificador nic malaltia diagnosticada
    
      SORTIDA  :  oRSP        : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc l'actualitzaci d'estadstiques
                 En cas contrari : 'ERROR: <NumError><DescError>'
   
   TASQUES QUE PORTA A TERME : 
   o Cada cop que a un usuari del sistema se li diagnostiqui per primer cop dins 
     l'any una malaltia :
     - Si aquesta malaltia s de la especialitat de cardiologia ('CARDI'), 
       incrementar en 1 la quantitat de diagnstics a taules 
       E5_USUCARDIACA i E5C_USUCARDIACA
     - Si aquesta malaltia s de la especialitat d'oncologia ('ONCOL'), 
       incrementar en 1 la quantitat de diagnstics a taules 
       E6_USUCANCERIGENA i E6C_USUCANCERIGENA
     - Si aquesta malaltia s de la especialitat de neurologia ('NEURO') i el seu 
       nivell de gravetat s considerat 'Greu' o 'Mortal', incrementar en 1 la quantitat 
       de diagnstics a taules E7_USUDEGENERATIVA i E7C_USUDEGENERATIVA

   o Sigui quina sigui la malaltia diagnosticada, si el seu nivell de gravetat s 
     considerat 'Lleu', incrementar en 1 la quantitat de diagnstics a 
     taules E8_MALALTIESLLEUS i E8C_MALALTIESLLEUS
   
   ******************************************************************************/
   
  PROCEDURE INS_EST_DIAGNOSTIC (
    pVisitaId    IN DIAGNOSTIC.VisitaId%TYPE,
    pMalaltiaId  IN DIAGNOSTIC.MalaltiaId%TYPE,
    oRSP        OUT VARCHAR2
  ) IS
  
    vComptador      NUMBER;
    vNivellGravetat VARCHAR2(20);
    vEspecialitat   VARCHAR2(5);
    vUsuariId       NUMBER;
    vPaisId         NUMBER;
    vAnyEst         NUMBER;
  
  BEGIN
   
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'INS_EST_DIAGNOSTIC';
    l_ParEntrada     := 'pVisitaId: '||pVisitaId||
                        ', pMalaltiaId: '||pMalaltiaId;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. INSERCI ESTADISTIQUES [R4] 5, 6, 7, i 8 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Revisem de quina especialitat i gravetat s la malaltia diagnosticada
    SELECT EspecialitatCodi, NivellGravetat 
      INTO vEspecialitat, vNivellGravetat 
      FROM TFC_BD.MALALTIA
     WHERE malId = pMalaltiaId;
  
    -- Cercar l'Usuari, el pas al que pertany i l'any de la visita en que s'ha ems. 
    SELECT V.UsuariId, C.PaisId, EXTRACT(YEAR FROM V.DataHora)
      INTO vUsuariId, vPaisId, vAnyEst
      FROM TFC_BD.VISITA V, TFC_BD.CIUTADA C
     WHERE V.UsuariId = C.ciuId
       AND V.visId    = pVisitaId;
    
    -- *************************************************************************** 
    -- Estadstica 5 - Ciutadans en que s'ha diagnosticat una malaltia cardaca
    IF (vEspecialitat = 'CARDI') THEN
        /* Comptar quants cops se li ha diagnosticat a l'usuari aquesta malaltia.
           Noms s'actualitzar en cas que sigui el primer cop.                  */ 
        SELECT COUNT(*)
          INTO vComptador
          FROM TFC_BD.VISITA V, TFC_BD.MALALTIA M, TFC_BD.DIAGNOSTIC D
         WHERE V.UsuariId                    = vUsuariId
           AND V.visId                       = D.VisitaId
           AND EXTRACT(YEAR FROM V.DataHora) = vAnyEst
           AND M.malId                       = D.MalaltiaId
           AND M.EspecialitatCodi            = vEspecialitat
      GROUP BY V.UsuariId, EXTRACT(YEAR FROM V.DataHora);
         
      IF (vComptador = 1) THEN
        -- Estadstica 5 ---------------------------------------------
        SELECT COUNT(*)
         INTO vComptador
         FROM TFC_BD.E5_USUCARDIACA
        WHERE PaisId = vPaisId
          AND NumAny = vAnyEst;
       
        IF (vComptador <= 0) THEN
          INSERT INTO TFC_BD.E5_USUCARDIACA (PaisId, NumAny, Quantitat)
            VALUES (vPaisId, vAnyEst, 1);
        ELSE
          -- Sumatori d'un usuari ms 
          UPDATE TFC_BD.E5_USUCARDIACA
             SET Quantitat = Quantitat + 1
           WHERE PaisId = vPaisId
             AND NumAny = vAnyEst;
        END IF;   
        
        -- Estadstica 5 Comunitria ---------------------------------------------
        SELECT COUNT(*)
          INTO vComptador
          FROM TFC_BD.E5C_USUCARDIACA
         WHERE NumAny = vAnyEst;
           
        IF (vComptador <= 0) THEN
          INSERT INTO TFC_BD.E5C_USUCARDIACA (NumAny, Quantitat)
            VALUES (vAnyEst, 1);
        ELSE
          -- Sumatori d'un usuari ms 
          UPDATE TFC_BD.E5C_USUCARDIACA
             SET Quantitat = Quantitat + 1
           WHERE NumAny = vAnyEst;
        END IF;   
      
      END IF;
       
    END IF;
    
    -- *************************************************************************** 
    -- Estadstica 6 - Ciutadans en que s'ha diagnosticat una malaltia cancergena
    IF (vEspecialitat = 'ONCOL') THEN
        /* Comptar quants cops se li ha diagnosticat a l'usuari aquesta malaltia.
           Noms s'actualitzar en cas que sigui el primer cop.                  */ 
        SELECT COUNT(*)
          INTO vComptador
          FROM TFC_BD.VISITA V, TFC_BD.MALALTIA M, TFC_BD.DIAGNOSTIC D
         WHERE V.UsuariId                    = vUsuariId
           AND V.visId                       = D.VisitaId
           AND EXTRACT(YEAR FROM V.DataHora) = vAnyEst
           AND M.malId                       = D.MalaltiaId
           AND M.EspecialitatCodi            = vEspecialitat
      GROUP BY V.UsuariId, EXTRACT(YEAR FROM V.DataHora);
         
      IF (vComptador = 1) THEN
        -- Estadstica 6 ---------------------------------------------
        SELECT COUNT(*)
         INTO vComptador
         FROM TFC_BD.E6_USUCANCERIGENA
        WHERE PaisId = vPaisId
          AND NumAny = vAnyEst;
       
        IF (vComptador <= 0) THEN
          INSERT INTO TFC_BD.E6_USUCANCERIGENA (PaisId, NumAny, Quantitat)
            VALUES (vPaisId, vAnyEst, 1);
        ELSE
          -- Sumatori d'un usuari ms 
          UPDATE TFC_BD.E6_USUCANCERIGENA
             SET Quantitat = Quantitat + 1
           WHERE PaisId = vPaisId
             AND NumAny = vAnyEst;
        END IF;   
        
        -- Estadstica 6 Comunitria ---------------------------------------------
        SELECT COUNT(*)
          INTO vComptador
          FROM TFC_BD.E6C_USUCANCERIGENA
         WHERE NumAny = vAnyEst;
           
        IF (vComptador <= 0) THEN
          INSERT INTO TFC_BD.E6C_USUCANCERIGENA (NumAny, Quantitat)
            VALUES (vAnyEst, 1);
        ELSE
          -- Sumatori d'un usuari ms 
          UPDATE TFC_BD.E6C_USUCANCERIGENA
             SET Quantitat = Quantitat + 1
           WHERE NumAny = vAnyEst;
        END IF;   
      
      END IF;
       
    END IF;
    
    -- *************************************************************************** 
    IF (vEspecialitat = 'NEURO') AND 
	   (vNivellGravetat = 'Greu' OR vNivellGravetat = 'Mortal') THEN
        /* Tractament de l'estadstica [R4] 7 Malaltia degenerativa del sistema 
           nervis (es considera que s una malaltia greu o mortal).
           Comptar quants cops se li ha diagnosticat a l'usuari aquesta malaltia.
           Noms s'actualitzar en cas que sigui el primer cop.                  */ 
        SELECT COUNT(*)
          INTO vComptador
          FROM TFC_BD.VISITA V, TFC_BD.MALALTIA M, TFC_BD.DIAGNOSTIC D
         WHERE V.UsuariId                    = vUsuariId
           AND V.visId                       = D.VisitaId
           AND EXTRACT(YEAR FROM V.DataHora) = vAnyEst
           AND M.malId                       = D.MalaltiaId
           AND M.EspecialitatCodi            = vEspecialitat
           AND M.NivellGravetat              = vNivellGravetat
      GROUP BY V.UsuariId, EXTRACT(YEAR FROM V.DataHora);
         
      IF (vComptador = 1) THEN
        -- Estadstica 7 ---------------------------------------------
        SELECT COUNT(*)
         INTO vComptador
         FROM TFC_BD.E7_USUDEGENERATIVA
        WHERE PaisId = vPaisId
          AND NumAny = vAnyEst;
       
        IF (vComptador <= 0) THEN
          INSERT INTO TFC_BD.E7_USUDEGENERATIVA (PaisId, NumAny, Quantitat)
            VALUES (vPaisId, vAnyEst, 1);
        ELSE
          -- Sumatori d'un usuari ms 
          UPDATE TFC_BD.E7_USUDEGENERATIVA
             SET Quantitat = Quantitat + 1
           WHERE PaisId = vPaisId
             AND NumAny = vAnyEst;
        END IF;   
        
        -- Estadstica 7 Comunitria ---------------------------------------------
        SELECT COUNT(*)
          INTO vComptador
          FROM TFC_BD.E7C_USUDEGENERATIVA
         WHERE NumAny = vAnyEst;
           
        IF (vComptador <= 0) THEN
          INSERT INTO TFC_BD.E7C_USUDEGENERATIVA (NumAny, Quantitat)
            VALUES (vAnyEst, 1);
        ELSE
          -- Sumatori d'un usuari ms 
          UPDATE TFC_BD.E7C_USUDEGENERATIVA
             SET Quantitat = Quantitat + 1
           WHERE NumAny = vAnyEst;
        END IF;   
      
      END IF;
       
    END IF;
    
    -- Estadstica 8 - Suma total de les malalties lleus -------------------------
    IF (vNivellGravetat = 'Lleu') then
      -- Estadstica 8 -----------------------------------------------
      SELECT COUNT(*)
       INTO vComptador
       FROM TFC_BD.E8_MALALTIESLLEUS
      WHERE PaisId = vPaisId
        AND NumAny = vAnyEst;
     
      IF (vComptador <= 0) THEN
        INSERT INTO TFC_BD.E8_MALALTIESLLEUS (PaisId, NumAny, Quantitat)
          VALUES (vPaisId, vAnyEst, 1);
      ELSE
        -- Sumatori d'un usuari ms 
        UPDATE TFC_BD.E8_MALALTIESLLEUS
           SET Quantitat = Quantitat + 1
         WHERE PaisId = vPaisId
           AND NumAny = vAnyEst;
      END IF;   
      
      -- Estadstica 8 Comunitria ---------------------------------------------
      SELECT COUNT(*)
        INTO vComptador
        FROM TFC_BD.E8C_MALALTIESLLEUS
       WHERE NumAny = vAnyEst;
         
      IF (vComptador <= 0) THEN
        INSERT INTO TFC_BD.E8C_MALALTIESLLEUS (NumAny, Quantitat)
          VALUES (vAnyEst, 1);
      ELSE
        -- Sumatori d'un usuari ms 
        UPDATE TFC_BD.E8C_MALALTIESLLEUS
           SET Quantitat = Quantitat + 1
         WHERE NumAny = vAnyEst;
      END IF;   
      
    END IF;
  
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

  EXCEPTION  
    WHEN OTHERS THEN
      ROLLBACK;
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END INS_EST_DIAGNOSTIC;
  
    
  /******************************************************************************
   NOM         : DEL_EST_DIAGNOSTIC
   
   PROPSIT    : Descomptar dades diagnstic borrat a taules estadstiques
   
   PARMETRES  :
      ENTRADA  :  pVisitaId   : Identificador nic de la visita on s'ha fet diagnosi
                  pMalaltiaId : Identificador nic malaltia diagnosticada
    
      SORTIDA  :  oRSP        : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc l'actualitzaci d'estadstiques
                 En cas contrari : 'ERROR: <NumError><DescError>'
   
   TASQUES QUE PORTA A TERME : 
   o Quan a un usuari del sistema no li quedi cap diagnstic d'un tipus de malaltia 
     dins l'any:
     - Si aquesta malaltia s de la especialitat de cardiologia ('CARDI'), 
       decrementar en 1 la quantitat de diagnstics a taules 
       E5_USUCARDIACA i E5C_USUCARDIACA
     - Si aquesta malaltia s de la especialitat d'oncologia ('ONCOL'), 
       decrementar en 1 la quantitat de diagnstics a taules 
       E6_USUCANCERIGENA i E6C_USUCANCERIGENA
     - Si aquesta malaltia s de la especialitat de neurologia ('NEURO') i el seu 
       nivell de gravetat s considerat 'Greu' o 'Mortal', decrementar en 1 la 
	   quantitat de diagnstics a taules E7_USUDEGENERATIVA i E7C_USUDEGENERATIVA
  
   o Sigui quin sigui el diagnstic esborrat d'una malaltia, si el seu nivell de 
     gravetat s considerat 'Lleu', decrementar en 1 la quantitat de diagnstics 
     a taules E8_MALALTIESLLEUS i E8C_MALALTIESLLEUS
   
   ******************************************************************************/
   
  PROCEDURE DEL_EST_DIAGNOSTIC (
    pVisitaId    IN DIAGNOSTIC.VisitaId%TYPE,
    pMalaltiaId  IN DIAGNOSTIC.MalaltiaId%TYPE,
    oRSP        OUT VARCHAR2
  ) IS
  
    vComptador      NUMBER;
    vNivellGravetat VARCHAR2(20);
    vEspecialitat   VARCHAR2(5);
    vQuantitat      NUMBER;
    vUsuariId       NUMBER;
    vPaisId         NUMBER;
    vAnyEst         NUMBER;
  
  BEGIN
   
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'DEL_EST_DIAGNOSTIC';
    l_ParEntrada     := 'pVisitaId: '||pVisitaId||
                        ', pMalaltiaId: '||pMalaltiaId;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. DESCOMPTE ESTADISTIQUES [R4] 5, 6, 7, i 8 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
    
    -- Revisem de quina especialitat i gravetat s la malaltia diagnosticada que
    -- s'ha borrat
    SELECT EspecialitatCodi, NivellGravetat 
      INTO vEspecialitat, vNivellGravetat 
      FROM TFC_BD.MALALTIA
     WHERE malId = pMalaltiaId;
  
    -- Cercar l'Usuari, el pas al que pertany i l'any de la visita en que s'havia ems. 
    SELECT V.UsuariId, C.PaisId, EXTRACT(YEAR FROM V.DataHora)
      INTO vUsuariId, vPaisId, vAnyEst
      FROM TFC_BD.VISITA V, TFC_BD.CIUTADA C
     WHERE V.UsuariId = C.ciuId
       AND V.visId    = pVisitaId;
    

    -- *************************************************************************** 
    -- Estadstica 5 - Ciutadans en que s'ha diagnosticat una malaltia cardaca
    IF (vEspecialitat = 'CARDI') THEN
        -- Comptar quants cops se li ha diagnosticat a l'usuari aquesta malaltia.
        SELECT COUNT(*)
          INTO vComptador
          FROM TFC_BD.VISITA V, TFC_BD.MALALTIA M, TFC_BD.DIAGNOSTIC D
         WHERE V.UsuariId                    = vUsuariId
           AND V.visId                       = D.VisitaId
           AND EXTRACT(YEAR FROM V.DataHora) = vAnyEst
           AND M.malId                       = D.MalaltiaId
           AND M.EspecialitatCodi            = vEspecialitat
      GROUP BY V.UsuariId, EXTRACT(YEAR FROM V.DataHora);
      
      -- Si no se n'ha trobat cap, restem estadstica   
      IF (vComptador <= 0) THEN
        -- Estadstica 5 ---------------------------------------------
        SELECT Quantitat
          INTO vQuantitat
          FROM TFC_BD.E5_USUCARDIACA
         WHERE PaisId = vPaisId
           AND NumAny = vAnyEst;
       
        IF (vQuantitat = 1) THEN
          -- Era l'nic usuari comptat, borrem registre
          DELETE TFC_BD.E5_USUCARDIACA
           WHERE PaisId = vPaisId
             AND NumAny = vAnyEst;
        ELSE 
          -- Restar un usuari 
          UPDATE TFC_BD.E5_USUCARDIACA
             SET Quantitat = Quantitat - 1
           WHERE PaisId = vPaisId
             AND NumAny = vAnyEst;
        END IF;
        
        -- Estadstica 5 Comunitria ---------------------------------------------
        SELECT Quantitat
          INTO vQuantitat
          FROM TFC_BD.E5C_USUCARDIACA
         WHERE NumAny = vAnyEst;
           
        IF (vQuantitat = 1) THEN
          -- Era l'nic usuari comptat, borrem registre
          DELETE TFC_BD.E5C_USUCARDIACA
           WHERE NumAny = vAnyEst;
        ELSE
          -- Sumatori d'un usuari ms 
          UPDATE TFC_BD.E5C_USUCARDIACA
             SET Quantitat = Quantitat - 1
           WHERE NumAny = vAnyEst;
        END IF;   
      
      END IF;
       
    END IF;
    
    -- *************************************************************************** 
    -- Estadstica 6 - Ciutadans en que s'ha diagnosticat una malaltia cancergena
    IF (vEspecialitat = 'ONCOL') THEN
        /* Comptar quants cops se li ha diagnosticat a l'usuari aquesta malaltia.
           Noms s'actualitzar en cas que sigui el primer cop.                  */ 
        SELECT COUNT(*)
          INTO vComptador
          FROM TFC_BD.VISITA V, TFC_BD.MALALTIA M, TFC_BD.DIAGNOSTIC D
         WHERE V.UsuariId                    = vUsuariId
           AND V.visId                       = D.VisitaId
           AND EXTRACT(YEAR FROM V.DataHora) = vAnyEst
           AND M.malId                       = D.MalaltiaId
           AND M.EspecialitatCodi            = vEspecialitat
      GROUP BY V.UsuariId, EXTRACT(YEAR FROM V.DataHora);
         
      -- Si no se n'ha trobat cap, restem estadstica   
      IF (vComptador <= 0) THEN
        -- Estadstica 6 ---------------------------------------------
        SELECT Quantitat
          INTO vQuantitat
          FROM TFC_BD.E6_USUCANCERIGENA
         WHERE PaisId = vPaisId
           AND NumAny = vAnyEst;
       
        IF (vQuantitat = 1) THEN
          -- Era l'nic usuari comptat, borrem registre
          DELETE TFC_BD.E6_USUCANCERIGENA
           WHERE PaisId = vPaisId
             AND NumAny = vAnyEst;
        ELSE 
          -- Restar un usuari 
          UPDATE TFC_BD.E6_USUCANCERIGENA
             SET Quantitat = Quantitat - 1
           WHERE PaisId = vPaisId
             AND NumAny = vAnyEst;
        END IF;
        
        -- Estadstica 6 Comunitria ---------------------------------------------
        SELECT Quantitat
          INTO vQuantitat
          FROM TFC_BD.E6C_USUCANCERIGENA
         WHERE NumAny = vAnyEst;
       
        IF (vQuantitat = 1) THEN
          -- Era l'nic usuari comptat, borrem registre
          DELETE TFC_BD.E6C_USUCANCERIGENA
           WHERE NumAny = vAnyEst;
        ELSE 
          -- Restar un usuari 
          UPDATE TFC_BD.E6C_USUCANCERIGENA
             SET Quantitat = Quantitat - 1
           WHERE NumAny = vAnyEst;
        END IF;
      
      END IF;
       
    END IF;
    
    -- *************************************************************************** 
    IF (vEspecialitat = 'NEURO') AND 
	   (vNivellGravetat = 'Greu' OR vNivellGravetat = 'Mortal') THEN
        /* Tractament de l'estadstica [R4] 7 Malaltia degenerativa del sistema 
           nervis (es considera que s una malaltia greu o mortal).
           Comptar quants cops se li ha diagnosticat a l'usuari aquesta malaltia.
           Noms s'actualitzar en cas que sigui el primer cop.                  */ 
        SELECT COUNT(*)
          INTO vComptador
          FROM TFC_BD.VISITA V, TFC_BD.MALALTIA M, TFC_BD.DIAGNOSTIC D
         WHERE V.UsuariId                    = vUsuariId
           AND V.visId                       = D.VisitaId
           AND EXTRACT(YEAR FROM V.DataHora) = vAnyEst
           AND M.malId                       = D.MalaltiaId
           AND M.EspecialitatCodi            = vEspecialitat
           AND M.NivellGravetat              = vNivellGravetat
      GROUP BY V.UsuariId, EXTRACT(YEAR FROM V.DataHora);
         
      -- Si no se n'ha trobat cap, restem estadstica   
      IF (vComptador <= 0) THEN
        -- Estadstica 7 ---------------------------------------------
        SELECT Quantitat
          INTO vQuantitat
          FROM TFC_BD.E7_USUDEGENERATIVA
         WHERE PaisId = vPaisId
           AND NumAny = vAnyEst;
       
        IF (vQuantitat = 1) THEN
          -- Era l'nic usuari comptat, borrem registre
          DELETE TFC_BD.E7_USUDEGENERATIVA
           WHERE PaisId = vPaisId
             AND NumAny = vAnyEst;
        ELSE 
          -- Restar un usuari 
          UPDATE TFC_BD.E7_USUDEGENERATIVA
             SET Quantitat = Quantitat - 1
           WHERE PaisId = vPaisId
             AND NumAny = vAnyEst;
        END IF;
        
        -- Estadstica 7 Comunitria ---------------------------------------------
        SELECT Quantitat
          INTO vQuantitat
          FROM TFC_BD.E7C_USUDEGENERATIVA
         WHERE NumAny = vAnyEst;
       
        IF (vQuantitat = 1) THEN
          -- Era l'nic usuari comptat, borrem registre
          DELETE TFC_BD.E7C_USUDEGENERATIVA
           WHERE NumAny = vAnyEst;
        ELSE 
          -- Restar un usuari 
          UPDATE TFC_BD.E7C_USUDEGENERATIVA
             SET Quantitat = Quantitat - 1
           WHERE NumAny = vAnyEst;
        END IF;
      
      END IF;
       
    END IF;
    
    -- Estadstica 8 - Suma total de les malalties lleus -------------------------
    IF (vNivellGravetat = 'Lleu') then
      -- Estadstica 8 ---------------------------------------------
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E8_MALALTIESLLEUS
       WHERE PaisId = vPaisId
         AND NumAny = vAnyEst;
     
      IF (vQuantitat = 1) THEN
        -- Era l'nic usuari comptat, borrem registre
        DELETE TFC_BD.E8_MALALTIESLLEUS
         WHERE PaisId = vPaisId
           AND NumAny = vAnyEst;
      ELSE 
        -- Restar un usuari 
        UPDATE TFC_BD.E8_MALALTIESLLEUS
           SET Quantitat = Quantitat - 1
         WHERE PaisId = vPaisId
           AND NumAny = vAnyEst;
      END IF;
      
      -- Estadstica 8 Comunitria ---------------------------------------------
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E8C_MALALTIESLLEUS
       WHERE NumAny = vAnyEst;
     
      IF (vQuantitat = 1) THEN
        -- Era l'nic usuari comptat, borrem registre
        DELETE TFC_BD.E8C_MALALTIESLLEUS
         WHERE NumAny = vAnyEst;
      ELSE 
        -- Restar un usuari 
        UPDATE TFC_BD.E8C_MALALTIESLLEUS
           SET Quantitat = Quantitat - 1
         WHERE NumAny = vAnyEst;
      END IF;
      
    END IF;
  
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

  EXCEPTION  
    WHEN OTHERS THEN
      ROLLBACK;
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END DEL_EST_DIAGNOSTIC;

  
  /******************************************************************************
   NOM      : INS_EST_ACTIVITAT
   
   PROPSIT : Inserci dades activitat diria a taules estadstiques
   
   PARMETRES  :
      ENTRADA  :  pUsuariId             : Identificador usuari que ha fet l'activitat
                  pDataAct              : Data de l'activitat
                  pNombrePasses         : Nombre de passes de l'activitat
                  pNombreCalories       : Nombre de calories de l'activitat
                  pNombreHoresInactives : Nombre d'hores inactives de l'activitat
    
      SORTIDA  :  oRSP                  : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc l'actualitzaci d'estadstiques
                 En cas contrari : 'ERROR: <NumError><DescError>'
   
   TASQUES QUE PORTA A TERME : 
   - Suma 1 a taules E1_USUSISTEMA i E1C_USUSISTEMA quan s la primera activitat 
     registrada de l'usuari dins l'any
   - Suma el nombre de calories del dia d'activitat a taules 
     E2_CALORIESUSU i E2C_CALORIESUSU
   - Suma el nombre de passes del dia d'activitat a taules 
     E3_PASSESUSU i E3C_PASSESUSU
   - Suma les hores de poca activitat del dia d'activitat a taules 
     E4_HORESPOCAACTUSU i E4C_HORESPOCAACTUSU
   - Actualitzaci a taules E9_USUMESCALORIES i/o E9C_USUMESCALORIES :
     Si no hi ha dades del pas/any, insereix l'usuari com el que ms calories
     gastat. Si ja n'hi ha, fa la suma total de calories gastades que porta 
     l'usuari dins l'any, i si s major a la de la estadstica actual, registra 
     aquest com a l'usuari amb ms calories gastades 
   
   ******************************************************************************/
   
  PROCEDURE INS_EST_ACTIVITAT (
    pUsuariId              IN ACTIVITAT.UsuariId%TYPE,
    pDataAct               IN ACTIVITAT.DataAct%TYPE,
    pNombrePasses          IN ACTIVITAT.NombrePasses%TYPE,
    pNombreCalories        IN ACTIVITAT.NombreCalories%TYPE,
    pNombreHoresInactives  IN ACTIVITAT.NombreHoresInactives%TYPE,
    oRSP                  OUT VARCHAR2
  ) IS
    vComptador    NUMBER;
    vCiutadaId    NUMBER;
    vPaisId       NUMBER;
  BEGIN
   
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'INS_EST_ACTIVITAT';
    l_ParEntrada     := 'pUsuariId: '||pUsuariId||
                        ', pDataAct: '||pDataAct||
                        ', pNombrePasses: '||pNombrePasses||
                        ', pNombreCalories: '||pNombreCalories||
                        ', pNombreHoresInactives: '||pNombreHoresInactives;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. INSERCI ESTADISTIQUES [R4] 1, 2, 3, 4 i 9 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Comptar quantes activitats hi ha de l'usuari t en l'any
        SELECT COUNT(*), C.PaisId
          INTO vComptador, vPaisId
          FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
         WHERE A.UsuariId = pUsuariId
           AND A.UsuariId = C.ciuId
           AND EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pDataAct)
      GROUP BY C.PaisId;
         
    -- Si n'hi ha noms una (la que s'ha creat), afegim estadstiques 
    IF (vComptador = 1) THEN     
    
      -- Estadstica 1 ---------------------------------------------------
      SELECT COUNT(*)
        INTO vComptador
        FROM TFC_BD.E1_USUSISTEMA
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
         
      IF (vComptador <= 0) THEN
        INSERT INTO TFC_BD.E1_USUSISTEMA (PaisId, NumAny, Quantitat)
          VALUES (vPaisId, EXTRACT(YEAR FROM pDataAct), 1);
      ELSE
        -- Sumatori d'un usuari ms 
        UPDATE TFC_BD.E1_USUSISTEMA
           SET Quantitat = Quantitat + 1
         WHERE PaisId = vPaisId
           AND NumAny = EXTRACT(YEAR FROM pDataAct);
      END IF;   
      
      -- Estadstica 1 Comunitria ---------------------------------------------
      SELECT COUNT(*)
        INTO vComptador
        FROM TFC_BD.E1C_USUSISTEMA
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
         
      IF (vComptador <= 0) THEN
        INSERT INTO TFC_BD.E1C_USUSISTEMA (NumAny, Quantitat)
          VALUES (EXTRACT(YEAR FROM pDataAct), 1);
      ELSE
        -- Sumatori d'un usuari ms 
        UPDATE TFC_BD.E1C_USUSISTEMA
           SET Quantitat = Quantitat + 1
         WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
      END IF;   
      
    END IF;
    
    -- Estadstica 2 ---------------------------------------------------
    SELECT COUNT(*)
      INTO vComptador
      FROM TFC_BD.E2_CALORIESUSU
     WHERE PaisId = vPaisId
       AND NumAny = EXTRACT(YEAR FROM pDataAct);
       
    IF (vComptador <= 0) THEN
      INSERT INTO TFC_BD.E2_CALORIESUSU (PaisId, NumAny, Quantitat)
        VALUES (vPaisId, EXTRACT(YEAR FROM pDataAct), pNombreCalories);
    ELSE
      -- Sumatori de calories 
      UPDATE TFC_BD.E2_CALORIESUSU
         SET Quantitat = Quantitat + pNombreCalories
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;   
    
    -- Estadstica 2 Comunitria ---------------------------------------------
    SELECT COUNT(*)
      INTO vComptador
      FROM TFC_BD.E2C_CALORIESUSU
     WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
       
    IF (vComptador <= 0) THEN
      INSERT INTO TFC_BD.E2C_CALORIESUSU (NumAny, Quantitat)
        VALUES (EXTRACT(YEAR FROM pDataAct), pNombreCalories);
    ELSE
      -- Sumatori de calories 
      UPDATE TFC_BD.E2C_CALORIESUSU
         SET Quantitat = Quantitat + pNombreCalories
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;   
    
    -- Estadstica 3 ---------------------------------------------------
    SELECT COUNT(*)
      INTO vComptador
      FROM TFC_BD.E3_PASSESUSU
     WHERE PaisId = vPaisId
       AND NumAny = EXTRACT(YEAR FROM pDataAct);
       
    IF (vComptador <= 0) THEN
      INSERT INTO TFC_BD.E3_PASSESUSU (PaisId, NumAny, Quantitat)
        VALUES (vPaisId, EXTRACT(YEAR FROM pDataAct), pNombrePasses);
    ELSE
      -- Sumatori de passes 
      UPDATE TFC_BD.E3_PASSESUSU
         SET Quantitat = Quantitat + pNombrePasses
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;   
    
    -- Estadstica 3 Comunitaria ---------------------------------------------
    SELECT COUNT(*)
      INTO vComptador
      FROM TFC_BD.E3C_PASSESUSU
     WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
       
    IF (vComptador <= 0) THEN
      INSERT INTO TFC_BD.E3C_PASSESUSU (NumAny, Quantitat)
        VALUES (EXTRACT(YEAR FROM pDataAct), pNombrePasses);
    ELSE
      -- Sumatori de passes 
      UPDATE TFC_BD.E3C_PASSESUSU
         SET Quantitat = Quantitat + pNombrePasses
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;   
    
    -- Estadstica 4 ---------------------------------------------------
    SELECT COUNT(*)
      INTO vComptador
      FROM TFC_BD.E4_HORESPOCAACTUSU
     WHERE PaisId = vPaisId
       AND NumAny = EXTRACT(YEAR FROM pDataAct);
       
    IF (vComptador <= 0) THEN
      INSERT INTO TFC_BD.E4_HORESPOCAACTUSU (PaisId, NumAny, Quantitat)
        VALUES (vPaisId, EXTRACT(YEAR FROM pDataAct), pNombreHoresInactives);
    ELSE
      -- Sumatori d'Hores de Poca Activitat 
      UPDATE TFC_BD.E4_HORESPOCAACTUSU
         SET Quantitat = Quantitat + pNombreHoresInactives
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;   
    
    -- Estadstica 4 Comuntria ----------------------------------------------
    SELECT COUNT(*)
      INTO vComptador
      FROM TFC_BD.E4C_HORESPOCAACTUSU
     WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
       
    IF (vComptador <= 0) THEN
      INSERT INTO TFC_BD.E4C_HORESPOCAACTUSU (NumAny, Quantitat)
        VALUES (EXTRACT(YEAR FROM pDataAct), pNombreHoresInactives);
    ELSE
      -- Sumatori d'Hores de Poca Activitat 
      UPDATE TFC_BD.E4C_HORESPOCAACTUSU
         SET Quantitat = Quantitat + pNombreHoresInactives
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;   
        
    -- Estadistica 9 -----------------------------------------------------------
    SELECT COUNT(*)
      INTO vComptador
      FROM TFC_BD.E9_USUMESCALORIES
     WHERE PaisId = vPaisId
       AND NumAny = EXTRACT(YEAR FROM pDataAct);
       
    IF (vComptador <= 0) THEN
      INSERT INTO TFC_BD.E9_USUMESCALORIES (PaisId, UsuariId, NumAny)
        VALUES (vPaisId, pUsuariId ,EXTRACT(YEAR FROM pDataAct));
    ELSE 
        SELECT A.USUARIID
          INTO vCiutadaId
          FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
         WHERE EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pDataAct)
           AND C.PAISID = vPaisId     
           AND A.USUARIID = C.CIUID
      GROUP BY A.USUARIID, EXTRACT(YEAR FROM A.DataAct)
        HAVING SUM(A.NombreCalories) = 
               (SELECT MAX(SUM(A.NombreCalories))
                  FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
                 WHERE EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pDataAct)
                   AND C.PAISID = vPaisId     
                   AND A.USUARIID = C.CIUID
              GROUP BY A.USUARIID, EXTRACT(YEAR FROM A.DataAct));     
   
      -- Si l'usuari s el que ostenta el consum mxim de calories, actualitzar. 
      IF (vCiutadaId = pUsuariId) THEN
        UPDATE TFC_BD.E9_USUMESCALORIES
          SET UsuariId = pUsuariId
         WHERE PaisId = vPaisId
           AND NumAny = EXTRACT(YEAR FROM pDataAct);
      END IF;   
    END IF;
      
    -- Estadistica 9 Comunitari ------------------------------------------------
    SELECT COUNT(*)
      INTO vComptador
      FROM TFC_BD.E9C_USUMESCALORIES
     WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
     
    IF (vComptador <= 0) THEN
      INSERT INTO TFC_BD.E9C_USUMESCALORIES (UsuariId, NumAny)
        VALUES (pUsuariId, EXTRACT(YEAR FROM pDataAct));
    ELSE 
        SELECT A.USUARIID
          INTO vCiutadaId
          FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
         WHERE EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pDataAct)
           AND A.USUARIID = C.CIUID
      GROUP BY A.USUARIID, EXTRACT(YEAR FROM A.DataAct)
        HAVING SUM(A.NombreCalories) = 
               (SELECT MAX(SUM(A.NombreCalories))
                  FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
                 WHERE EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pDataAct)
                   AND A.USUARIID = C.CIUID
              GROUP BY A.USUARIID, EXTRACT(YEAR FROM A.DataAct));     
              
      -- Si l'usuari s el que ostenta el consum mxim de calories, actualitzar. 
      IF (vCiutadaId = pUsuariId) THEN
        UPDATE TFC_BD.E9C_USUMESCALORIES
          SET UsuariId = pUsuariId
         WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
      END IF;   
    END IF;
    
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

  EXCEPTION  
    WHEN OTHERS THEN
      ROLLBACK;
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);


  END INS_EST_ACTIVITAT;
  

  /******************************************************************************
   NOM      : UPD_EST_ACTIVITAT
   
   PROPSIT : Modificaci dades activitat d'un dia a taules estadstiques
   
   PARMETRES  :
      ENTRADA  :  pOLDUsuariId             : Identificador anterior usuari 
                                             que havia fet l'activitat
                  pOLDDataAct              : Data anterior de l'activitat
                  pOLDNombrePasses         : Nombre de passes anteriors de 
                                             l'activitat
                  pOLDNombreCalories       : Nombre de calories anteriors de 
                                             l'activitat
                  pOLDNombreHoresInactives : Nombre d'hores inactives anteriors 
                                             de l'activitat
                  pUsuariId                : Identificador nou usuari que ha fet 
                                             l'activitat
                  pDataAct                 : Data nova de l'activitat
                  pNombrePasses            : Nombre de passes noves de l'activitat
                  pNombreCalories          : Nombre de calories noves de l'activitat
                  pNombreHoresInactives    : Nombre d'hores inactives noves de 
                                             l'activitat
    
      SORTIDA  :  oRSP                     : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc l'actualitzaci d'estadstiques
                 En cas contrari : 'ERROR: <NumError><DescError>'
   
   TASQUES QUE PORTA A TERME : 
   - Si s'ha canviat l'usuari, resta 1 a taules E1_USUSISTEMA i E1C_USUSISTEMA si 
     l'activitat que es modifica era la nica que aquest tenia registrada dins 
     l'any vell.
   - Si s'ha canviat l'any, resta 1 a taules E1_USUSISTEMA i E1C_USUSISTEMA si 
     l'activitat que es modifica era la nica que hi tenia registrada l'usuari 
     nou dins l'any vell.
   - Suma 1 a taules E1_USUSISTEMA i E1C_USUSISTEMA quan s la primera activitat 
     registrada de l'usuari nou dins l'any nou (noms en cas de canvi d'usuari 
     o canvi d'any)
   - Si s'ha canviat d'any, resta el nombre de calories de l'any vell a taules 
     E2_CALORIESUSU i E2C_CALORIESUSU
   - Actualitza diferncia entre l'antic i el nou nombre de calories a les taules 
     E2_CALORIESUSU i E2C_CALORIESUSU
   - Si s'ha canviat d'any, resta el nombre de passes de l'any vell a taules 
     E3_PASSESUSU i E3C_PASSESUSU
   - Actualitza diferncia entre l'antic i el nou nombre de passes a les taules 
     E3_PASSESUSU i E3C_PASSESUSU
   - Si s'ha canviat d'any, resta el nombre d'hores de poca activitat de l'any 
     vell a taules E4_HORESPOCAACTUSU i E4C_HORESPOCAACTUSU
   - Actualitza diferncia entre l'antic i el nou nombre d'hores de poca activitat 
     a les taules E4_HORESPOCAACTUSU i E4C_HORESPOCAACTUSU
   - Refresca desprs de la modificaci, si cal, els nous usuaris amb ms calories 
     gastades, tant a l'any antic com a l'any nou, a les taules
     E9_USUMESCALORIES i/o E9C_USUMESCALORIES
   
   ******************************************************************************/
   
  PROCEDURE UPD_EST_ACTIVITAT (
    pOLDUsuariId              IN ACTIVITAT.UsuariId%TYPE,
    pOLDDataAct               IN ACTIVITAT.DataAct%TYPE,
    pOLDNombrePasses          IN ACTIVITAT.NombrePasses%TYPE,
    pOLDNombreCalories        IN ACTIVITAT.NombreCalories%TYPE,
    pOLDNombreHoresInactives  IN ACTIVITAT.NombreHoresInactives%TYPE,
    pUsuariId                 IN ACTIVITAT.UsuariId%TYPE,
    pDataAct                  IN ACTIVITAT.DataAct%TYPE,
    pNombrePasses             IN ACTIVITAT.NombrePasses%TYPE,
    pNombreCalories           IN ACTIVITAT.NombreCalories%TYPE,
    pNombreHoresInactives     IN ACTIVITAT.NombreHoresInactives%TYPE,
    oRSP                     OUT VARCHAR2
  ) IS
  
    vComptador               NUMBER;
    vSumaCalories            NUMBER;
    vSumaCaloriesOLD         NUMBER;
    vNombreCaloriesOLD       NUMBER;
    vNombrePassesOLD         NUMBER;
    vNombreHoresInactivesOLD NUMBER;
    vCiutadaId               NUMBER;
    vPaisId                  NUMBER;
    vPaisIdOLD               NUMBER;
    vQuantitat               NUMBER;
  
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'UPD_EST_ACTIVITAT';
    l_ParEntrada     := 'pUsuariId: '||pUsuariId||
                        ', pDataAct: '||pDataAct||
                        ', pNombrePasses: '||pNombrePasses||
                        ', pNombreCalories: '||pNombreCalories||
                        ', pNombreHoresInactives: '||pNombreHoresInactives;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. ACTUALITZACI ESTADISTIQUES [R4] 1, 2, 3, 4 i 9 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Si es canvia l'usuari :
    IF (pOLDUsuariId <> pUsuariId) THEN
      
      -- Cal veure quantes activitats tenia el vell a l'any vell
        SELECT COUNT(*), SUM(A.NombreCalories), C.PaisId
          INTO vComptador, vSumaCaloriesOLD, vPaisIdOLD
          FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
         WHERE A.UsuariId = pOLDUsuariId
           AND A.UsuariId = C.ciuId
           AND EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pOLDDataAct)
      GROUP BY C.PaisId;
      
      -- Si ja no queda cap activitat de l'usuari vell a l'any vell, restar de l'estadstica 1
      IF (vComptador <= 0) THEN
      
        -- Estadstica 1 ---------------------------------------------------
        SELECT Quantitat
          INTO vQuantitat
          FROM TFC_BD.E1_USUSISTEMA
         WHERE PaisId = vPaisIdOLD
           AND NumAny = EXTRACT(YEAR FROM pOLDDataAct);
           
        IF (vQuantitat > 1) THEN
          -- Resta de l'usuari 
          UPDATE TFC_BD.E1_USUSISTEMA
             SET Quantitat = Quantitat - 1
           WHERE PaisId = vPaisIdOLD
             AND NumAny = EXTRACT(YEAR FROM pOLDDataAct);
        ELSE
          -- No queda cap ms usuari a l'any/pas. Borrar.
          DELETE TFC_BD.E1_USUSISTEMA
           WHERE PaisId = vPaisIdOLD
             AND NumAny = EXTRACT(YEAR FROM pOLDDataAct);
        END IF;   
        
        -- Estadstica 1 Comunitria ---------------------------------------------
        SELECT Quantitat
          INTO vQuantitat
          FROM TFC_BD.E1C_USUSISTEMA
         WHERE NumAny = EXTRACT(YEAR FROM pOLDDataAct);
           
        IF (vQuantitat > 1) THEN
          -- Resta de l'usuari 
          UPDATE TFC_BD.E1C_USUSISTEMA
             SET Quantitat = Quantitat - 1
           WHERE NumAny = EXTRACT(YEAR FROM pOLDDataAct);
        ELSE
          -- No queda cap ms usuari a l'any. Borrar.
          DELETE TFC_BD.E1C_USUSISTEMA
           WHERE NumAny = EXTRACT(YEAR FROM pOLDDataAct);
        END IF;   
        
      END IF;
           
    END IF;
    
    -- Cal veure quantes activitats t l'usuari nou a l'any vell
      SELECT COUNT(*), SUM(A.NombreCalories), C.PaisId
        INTO vComptador, vSumaCalories, vPaisId
        FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
       WHERE A.UsuariId = pUsuariId
         AND A.UsuariId = C.ciuId
         AND EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pOLDDataAct)
    GROUP BY C.PaisId;
    
    -- Si ja no queda cap activitat, restar de l'estadstica 1
    IF (vComptador <= 0) THEN
    
      -- Estadstica 1 ---------------------------------------------------
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E1_USUSISTEMA
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pOLDDataAct);
         
      IF (vQuantitat > 1) THEN
        -- Resta de l'usuari 
        UPDATE TFC_BD.E1_USUSISTEMA
           SET Quantitat = Quantitat - 1
         WHERE PaisId = vPaisId
           AND NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      ELSE
        -- No queda cap ms usuari a l'any/pas. Borrar.
        DELETE TFC_BD.E1_USUSISTEMA
         WHERE PaisId = vPaisId
           AND NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      END IF;   
      
      
      -- Estadstica 1 Comunitria ---------------------------------------------
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E1C_USUSISTEMA
       WHERE NumAny = EXTRACT(YEAR FROM pOLDDataAct);
         
      IF (vQuantitat > 1) THEN
        -- Resta de l'usuari 
        UPDATE TFC_BD.E1C_USUSISTEMA
           SET Quantitat = Quantitat - 1
         WHERE NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      ELSE
        -- No queda cap ms usuari a l'any. Borrar.
        DELETE TFC_BD.E1C_USUSISTEMA
         WHERE NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      END IF;   
      
    END IF;
        
    -- Comptar quantes activitats hi ha de l'usuari nou en l'any nou
      SELECT COUNT(*), SUM(A.NombreCalories), C.PaisId
        INTO vComptador, vSumaCalories, vPaisId
        FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
       WHERE A.UsuariId = pUsuariId
         AND A.UsuariId = C.ciuId
         AND EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pDataAct)
    GROUP BY C.PaisId;
         
    -- Si no ha canviat d'usuari, el pas antic que tractarem ser el mateix que el nou
    IF (pOLDUsuariId = pUsuariId) THEN
      vPaisIdOLD := vPaisId;
    END IF;
    
    -- En cas de canvi d'usuari o canvi d'any, i si n'hi ha noms una de l'usuari nou, afegim estadstica 1 
    IF (pOLDUsuariId <> pUsuariId OR 
        EXTRACT(YEAR FROM pOLDDataAct) <> EXTRACT(YEAR FROM pDataAct)) AND 
       (vComptador = 1) THEN     
    
      -- Estadstica 1 ---------------------------------------------------
      SELECT COUNT(*)
        INTO vComptador
        FROM TFC_BD.E1_USUSISTEMA
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
         
      IF (vComptador <= 0) THEN
        INSERT INTO TFC_BD.E1_USUSISTEMA (PaisId, NumAny, Quantitat)
          VALUES (vPaisId, EXTRACT(YEAR FROM pDataAct), 1);
      ELSE
        -- Sumatori d'un usuari ms 
        UPDATE TFC_BD.E1_USUSISTEMA
           SET Quantitat = Quantitat + 1
         WHERE PaisId = vPaisId
           AND NumAny = EXTRACT(YEAR FROM pDataAct);
      END IF;   
      
      -- Estadstica 1 Comunitria ---------------------------------------------
      SELECT COUNT(*)
        INTO vComptador
        FROM TFC_BD.E1C_USUSISTEMA
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
         
      IF (vComptador <= 0) THEN
        INSERT INTO TFC_BD.E1C_USUSISTEMA (NumAny, Quantitat)
          VALUES (EXTRACT(YEAR FROM pDataAct), 1);
      ELSE
        -- Sumatori d'un usuari ms 
        UPDATE TFC_BD.E1C_USUSISTEMA
           SET Quantitat = Quantitat + 1
         WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
      END IF;   
      
    END IF;
    
    -- Si hi ha canvi d'any, no hem restar en l'any actual   
    IF (EXTRACT(YEAR FROM pOLDDataAct) <> EXTRACT(YEAR FROM pDataAct)) THEN
      vNombreCaloriesOLD       := 0;
      vNombrePassesOLD         := 0;
      vNombreHoresInactivesOLD := 0;
    ELSE
      vNombreCaloriesOLD       := pOLDNombreCalories;
      vNombrePassesOLD         := pOLDNombrePasses;
      vNombreHoresInactivesOLD := pOLDNombreHoresInactives;
    END IF;
    
    -- Estadstica 2 -----------------------------------------------------------
    -- Hem de vegilar canvi d'any o de pas. Si s aix, hem de restar de l'any vell

    IF (vPaisIdOLD <> vPaisId OR
        EXTRACT(YEAR FROM pOLDDataAct) <> EXTRACT(YEAR FROM pDataAct)) THEN
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E2_CALORIESUSU
       WHERE PaisId = vPaisIdOLD
         AND NumAny = EXTRACT(YEAR FROM pOLDDataAct);
       
      IF (vQuantitat <> pOLDNombreCalories) THEN
        -- Resta de l'usuari 
        UPDATE TFC_BD.E2_CALORIESUSU
           SET Quantitat = Quantitat - pOLDNombreCalories
         WHERE PaisId = vPaisIdOLD
           AND NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      ELSE
        -- No queda cap ms usuari a l'any. Borrar.
        DELETE TFC_BD.E2_CALORIESUSU
         WHERE PaisId = vPaisIdOLD
           AND NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      END IF;
    END IF;

    -- Actualitzem el nou nombre de calories.
    BEGIN
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E2_CALORIESUSU
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
      vComptador := 1;  
    EXCEPTION
      WHEN OTHERS THEN
        vComptador := 0;
    END;
       
    IF (vComptador <= 0) THEN
      -- Si no hi ha dades de l'any, creem el nou registre
      INSERT INTO TFC_BD.E2_CALORIESUSU (PaisId, NumAny, Quantitat)
        VALUES (vPaisId, EXTRACT(YEAR FROM pDataAct), pNombreCalories);
    ELSIF (vQuantitat - vNombreCaloriesOLD + pNombreCalories = 0) THEN
      -- Si la nova quantitat s zero, borrem el registre
      DELETE TFC_BD.E2_CALORIESUSU
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    ELSE
      -- Sumatori/Resta de calories 
      UPDATE TFC_BD.E2_CALORIESUSU
         SET Quantitat = Quantitat - vNombreCaloriesOLD + pNombreCalories
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;   
    
    -- Estadstica 2 Comunitria ---------------------------------------------
    -- Hem de vegilar canvi d'any. Si s aix, hem de restar de l'any vell
    IF (EXTRACT(YEAR FROM pOLDDataAct) <> EXTRACT(YEAR FROM pDataAct)) THEN
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E2C_CALORIESUSU
       WHERE NumAny = EXTRACT(YEAR FROM pOLDDataAct);
       
      IF (vQuantitat <> pOLDNombreCalories) THEN
        -- Resta de l'usuari 
        UPDATE TFC_BD.E2C_CALORIESUSU
           SET Quantitat = Quantitat - pOLDNombreCalories
         WHERE NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      ELSE
        -- No queda cap ms usuari a l'any. Borrar.
        DELETE TFC_BD.E2C_CALORIESUSU
         WHERE NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      END IF;
    END IF;
    
    -- Actualitzem el nou nombre de calories
    BEGIN
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E2C_CALORIESUSU
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
      vComptador := 1;  
    EXCEPTION
      WHEN OTHERS THEN
        vComptador := 0;
    END;
       
    IF (vComptador <= 0) THEN
      -- Si no hi ha dades de l'any, creem el nou registre
      INSERT INTO TFC_BD.E2C_CALORIESUSU (NumAny, Quantitat)
        VALUES (EXTRACT(YEAR FROM pDataAct), pNombreCalories);
    ELSIF (vQuantitat - vNombreCaloriesOLD + pNombreCalories = 0) THEN
      -- Si la nova quantitat s zero, borrem el registre
      DELETE TFC_BD.E2C_CALORIESUSU
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    ELSE
      -- Sumatori/Resta de calories 
      UPDATE TFC_BD.E2C_CALORIESUSU
         SET Quantitat = Quantitat - vNombreCaloriesOLD + pNombreCalories
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;  
    
    -- Estadstica 3 ---------------------------------------------------
    -- Hem de vegilar canvi d'any o de pas. Si s aix, hem de restar de l'any vell
    IF (vPaisIdOLD <> vPaisId OR
        EXTRACT(YEAR FROM pOLDDataAct) <> EXTRACT(YEAR FROM pDataAct)) THEN
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E3_PASSESUSU
       WHERE PaisId = vPaisIdOLD
         AND NumAny = EXTRACT(YEAR FROM pOLDDataAct);
       
      IF (vQuantitat <> pOLDNombrePasses) THEN
        -- Resta de l'usuari 
        UPDATE TFC_BD.E3_PASSESUSU
           SET Quantitat = Quantitat - pOLDNombrePasses
         WHERE PaisId = vPaisIdOLD
           AND NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      ELSE
        -- No queda cap ms usuari a l'any. Borrar.
        DELETE TFC_BD.E3_PASSESUSU
         WHERE PaisId = vPaisIdOLD
           AND NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      END IF;
    END IF;
    
    -- Actualitzem el nou nombre de passes
    BEGIN
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E3_PASSESUSU
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
      vComptador := 1;  
    EXCEPTION
      WHEN OTHERS THEN
        vComptador := 0;
    END;
       
    IF (vComptador <= 0) THEN
      INSERT INTO TFC_BD.E3_PASSESUSU (PaisId, NumAny, Quantitat)
        VALUES (vPaisId, EXTRACT(YEAR FROM pDataAct), pNombrePasses);
    ELSIF (vQuantitat - vNombrePassesOLD + pNombrePasses = 0) THEN
      -- Si la nova quantitat s zero, borrem el registre
      DELETE TFC_BD.E3_PASSESUSU
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    ELSE
      -- Sumatori/Resta de passes 
      UPDATE TFC_BD.E3_PASSESUSU
         SET Quantitat = Quantitat - vNombrePassesOLD + pNombrePasses
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;   
    
    -- Estadstica 3 Comunitaria ---------------------------------------------
    -- Hem de vegilar canvi d'any. Si s aix, hem de restar de l'any vell
    IF (EXTRACT(YEAR FROM pOLDDataAct) <> EXTRACT(YEAR FROM pDataAct)) THEN
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E3C_PASSESUSU
       WHERE NumAny = EXTRACT(YEAR FROM pOLDDataAct);
       
      IF (vQuantitat <> pOLDNombrePasses) THEN
        -- Resta de l'usuari 
        UPDATE TFC_BD.E3C_PASSESUSU
           SET Quantitat = Quantitat - pOLDNombrePasses
         WHERE NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      ELSE
        -- No queda cap ms usuari a l'any. Borrar.
        DELETE TFC_BD.E3C_PASSESUSU
         WHERE NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      END IF;
    END IF;
    
    -- Actualitzem el nou nombre de passes
    BEGIN
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E3C_PASSESUSU
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
      vComptador := 1;  
    EXCEPTION
      WHEN OTHERS THEN
        vComptador := 0;
    END;
       
    IF (vComptador <= 0) THEN
      INSERT INTO TFC_BD.E3C_PASSESUSU (NumAny, Quantitat)
        VALUES (EXTRACT(YEAR FROM pDataAct), pNombrePasses);
    ELSIF (vQuantitat - vNombrePassesOLD + pNombrePasses = 0) THEN
      -- Si la nova quantitat s zero, borrem el registre
      DELETE TFC_BD.E3C_PASSESUSU
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    ELSE
      -- Sumatori de passes 
      UPDATE TFC_BD.E3C_PASSESUSU
         SET Quantitat = Quantitat - vNombrePassesOLD + pNombrePasses
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;   

    -- Estadstica 4 ---------------------------------------------------
    -- Hem de vegilar canvi d'any o de pas. Si s aix, hem de restar de l'any vell
    IF (vPaisIdOLD <> vPaisId OR
        EXTRACT(YEAR FROM pOLDDataAct) <> EXTRACT(YEAR FROM pDataAct)) THEN
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E4_HORESPOCAACTUSU
       WHERE PaisId = vPaisIdOLD
         AND NumAny = EXTRACT(YEAR FROM pOLDDataAct);
       
      IF (vQuantitat <> pOLDNombreHoresInactives) THEN
        -- Resta de l'usuari 
        UPDATE TFC_BD.E4_HORESPOCAACTUSU
           SET Quantitat = Quantitat - pOLDNombreHoresInactives
         WHERE PaisId = vPaisIdOLD
           AND NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      ELSE
        -- No queda cap ms usuari a l'any. Borrar.
        DELETE TFC_BD.E4_HORESPOCAACTUSU
         WHERE PaisId = vPaisIdOLD
           AND NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      END IF;
    END IF;
    
    -- Actualitzem el nou nombre d'hores inactives
    BEGIN
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E4_HORESPOCAACTUSU
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
      vComptador := 1;  
    EXCEPTION
      WHEN OTHERS THEN
        vComptador := 0;
    END;
           
    IF (vComptador <= 0) THEN
      INSERT INTO TFC_BD.E4_HORESPOCAACTUSU (PaisId, NumAny, Quantitat)
        VALUES (vPaisId, EXTRACT(YEAR FROM pDataAct), pNombreHoresInactives);
    ELSIF (vQuantitat - vNombreHoresInactivesOLD + pNombreHoresInactives = 0) THEN
      -- Si la nova quantitat s zero, borrem el registre
      DELETE TFC_BD.E4_HORESPOCAACTUSU
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    ELSE
      -- Sumatori/Resta d'Hores inactives 
      UPDATE TFC_BD.E4_HORESPOCAACTUSU
         SET Quantitat = Quantitat - vNombreHoresInactivesOLD + pNombreHoresInactives
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;   
    
    -- Estadstica 4 Comuntria ----------------------------------------------
    -- Hem de vegilar canvi d'any. Si s aix, hem de restar de l'any vell
    IF (EXTRACT(YEAR FROM pOLDDataAct) <> EXTRACT(YEAR FROM pDataAct)) THEN
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E4C_HORESPOCAACTUSU
       WHERE NumAny = EXTRACT(YEAR FROM pOLDDataAct);
       
      IF (vQuantitat <> pOLDNombreHoresInactives) THEN
        -- Resta de l'usuari 
        UPDATE TFC_BD.E4C_HORESPOCAACTUSU
           SET Quantitat = Quantitat - pOLDNombreHoresInactives
         WHERE NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      ELSE
        -- No queda cap ms usuari a l'any. Borrar.
        DELETE TFC_BD.E4C_HORESPOCAACTUSU
         WHERE NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      END IF;
    END IF;
    
    -- Actualitzem el nou nombre d'hores inactives
    BEGIN
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E4C_HORESPOCAACTUSU
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
      vComptador := 1;  
    EXCEPTION
      WHEN OTHERS THEN
        vComptador := 0;
    END;
           
    IF (vComptador <= 0) THEN
      INSERT INTO TFC_BD.E4C_HORESPOCAACTUSU (NumAny, Quantitat)
        VALUES (EXTRACT(YEAR FROM pDataAct), pNombreHoresInactives);
    ELSIF (vQuantitat - vNombreHoresInactivesOLD + pNombreHoresInactives = 0) THEN
      -- Si la nova quantitat s zero, borrem el registre
      DELETE TFC_BD.E4C_HORESPOCAACTUSU
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    ELSE
      -- Sumatori de passes 
      UPDATE TFC_BD.E4C_HORESPOCAACTUSU
         SET Quantitat = Quantitat - vNombreHoresInactivesOLD + pNombreHoresInactives
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;   
        
    -- Estadistica 9 (any vell) ------------------------------------------------
    -- Actualitzarem l'any antic si hem canviat d'any l'activitat o hem canviat
    -- de pas per un canvi d'usuari
    IF (vPaisIdOLD <> vPaisId OR
        EXTRACT(YEAR FROM pOLDDataAct) <> EXTRACT(YEAR FROM pDataAct)) THEN
        
      BEGIN  
          SELECT A.USUARIID
            INTO vCiutadaId
            FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
           WHERE EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pOLDDataAct)
             AND C.PAISID   = vPaisIdOLD     
             AND A.USUARIID = C.CIUID
        GROUP BY A.USUARIID, EXTRACT(YEAR FROM A.DataAct)
          HAVING SUM(A.NombreCalories) = 
                 (SELECT MAX(SUM(A.NombreCalories))
                    FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
                   WHERE EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pOLDDataAct)
                     AND C.PAISID = vPaisIdOLD
                     AND A.USUARIID = C.CIUID
                GROUP BY A.USUARIID, EXTRACT(YEAR FROM A.DataAct));     
        vComptador := 1;  
      EXCEPTION
        WHEN OTHERS THEN
          vComptador := 0;
      END;

      IF vComptador <= 0 THEN
        DELETE TFC_BD.E9_USUMESCALORIES
         WHERE PaisId = vPaisIdOLD
           AND NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      ELSE
        UPDATE TFC_BD.E9_USUMESCALORIES
           SET UsuariId = vCiutadaId
         WHERE PaisId = vPaisIdOLD
           AND NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      END IF;      
        
      -- Estadistica 9 Comunitari ------------------------------------------------
      BEGIN  
          SELECT A.USUARIID
            INTO vCiutadaId
            FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
           WHERE EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pOLDDataAct)
             AND A.USUARIID = C.CIUID
        GROUP BY A.USUARIID, EXTRACT(YEAR FROM A.DataAct)
          HAVING SUM(A.NombreCalories) = 
                 (SELECT MAX(SUM(A.NombreCalories))
                    FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
                   WHERE EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pOLDDataAct)
                     AND A.USUARIID = C.CIUID
                GROUP BY A.USUARIID, EXTRACT(YEAR FROM A.DataAct));     
        vComptador := 1;  
      EXCEPTION
        WHEN OTHERS THEN
          vComptador := 0;
      END;
              
      IF vComptador <= 0 THEN
        DELETE TFC_BD.E9C_USUMESCALORIES
         WHERE NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      ELSE
        UPDATE TFC_BD.E9C_USUMESCALORIES
           SET UsuariId = vCiutadaId
         WHERE NumAny = EXTRACT(YEAR FROM pOLDDataAct);
      END IF;
      
    END IF;

    -- Estadistica 9 (any nou) -------------------------------------------------
    BEGIN
        SELECT A.USUARIID
          INTO vCiutadaId
          FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
         WHERE EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pDataAct)
           AND C.PAISID = vPaisId     
           AND A.USUARIID = C.CIUID
      GROUP BY A.USUARIID, EXTRACT(YEAR FROM A.DataAct)
        HAVING SUM(A.NombreCalories) = 
               (SELECT MAX(SUM(A.NombreCalories))
                  FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
                 WHERE EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pDataAct)
                   AND C.PAISID = vPaisId     
                   AND A.USUARIID = C.CIUID
              GROUP BY A.USUARIID, EXTRACT(YEAR FROM A.DataAct));     
      vComptador := 1;  
    EXCEPTION
      WHEN OTHERS THEN
        vComptador := 0;
    END;

    IF vComptador <= 0 THEN
      DELETE TFC_BD.E9_USUMESCALORIES
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    ELSE
      UPDATE TFC_BD.E9_USUMESCALORIES
         SET UsuariId = vCiutadaId
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;      

    -- Estadistica 9 Comunitari ------------------------------------------------
    BEGIN
        SELECT A.USUARIID
          INTO vCiutadaId
          FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
         WHERE EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pDataAct)
           AND A.USUARIID = C.CIUID
      GROUP BY A.USUARIID, EXTRACT(YEAR FROM A.DataAct)
        HAVING SUM(A.NombreCalories) = 
               (SELECT MAX(SUM(A.NombreCalories))
                  FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
                 WHERE EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pDataAct)
                   AND A.USUARIID = C.CIUID
              GROUP BY A.USUARIID, EXTRACT(YEAR FROM A.DataAct));     
      vComptador := 1;  
    EXCEPTION
      WHEN OTHERS THEN
        vComptador := 0;
    END;
   
    IF vComptador <= 0 THEN
      DELETE TFC_BD.E9C_USUMESCALORIES
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    ELSE
      UPDATE TFC_BD.E9C_USUMESCALORIES
         SET UsuariId = vCiutadaId
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;

    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

  EXCEPTION  
    WHEN OTHERS THEN
      ROLLBACK;
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END UPD_EST_ACTIVITAT;

  /******************************************************************************
   NOM      : DEL_EST_ACTIVITAT
   
   PROPSIT : Descomptar dades activitat diria borrada a taules estadstiques
   
   PARMETRES  :
      ENTRADA  :  pUsuariId             : Identificador usuari que ha fet l'activitat
                  pDataAct              : Data de l'activitat
                  pNombrePasses         : Nombre de passes de l'activitat
                  pNombreCalories       : Nombre de calories de l'activitat
                  pNombreHoresInactives : Nombre d'hores inactives de l'activitat
    
      SORTIDA  :  oRSP                  : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc l'actualitzaci d'estadstiques
                 En cas contrari : 'ERROR: <NumError><DescError>'
   
   TASQUES QUE PORTA A TERME : 
   - Resta 1 a taules E1_USUSISTEMA i E1C_USUSISTEMA si ja no queden activitats 
     registrades de l'usuari dins l'any
   - Resta el nombre de calories del dia d'activitat esborrat a taules 
     E2_CALORIESUSU i E2C_CALORIESUSU
   - Resta el nombre de passes del dia d'activitat esborrat a taules 
     E3_PASSESUSU i E3C_PASSESUSU
   - Resta les hores de poca activitat del dia d'activitat esborrat a taules 
     E4_HORESPOCAACTUSU i E4C_HORESPOCAACTUSU
   - Actualitza usuari amb ms calories gastades de l'any desprs d'haver borrat 
     l'activitat a taules E9_USUMESCALORIES i E9C_USUMESCALORIES 
   
   ******************************************************************************/

   
  PROCEDURE DEL_EST_ACTIVITAT (
    pUsuariId              IN ACTIVITAT.UsuariId%TYPE,
    pDataAct               IN ACTIVITAT.DataAct%TYPE,
    pNombrePasses          IN ACTIVITAT.NombrePasses%TYPE,
    pNombreCalories        IN ACTIVITAT.NombreCalories%TYPE,
    pNombreHoresInactives  IN ACTIVITAT.NombreHoresInactives%TYPE,
    oRSP                  OUT VARCHAR2
  ) IS
    vComptador    NUMBER;
    vQuantitat    NUMBER;
    vCiutadaId    NUMBER;
    vPaisId       NUMBER;
  BEGIN
   
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'DEL_EST_ACTIVITAT';
    l_ParEntrada     := 'pUsuariId: '||pUsuariId||
                        ', pDataAct: '||pDataAct||
                        ', pNombrePasses: '||pNombrePasses||
                        ', pNombreCalories: '||pNombreCalories||
                        ', pNombreHoresInactives: '||pNombreHoresInactives;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. DESCOMPTE ESTADISTIQUES [R4] 1, 2, 3, 4 i 9 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;

      -- Comptar quantes activitats hi ha de l'usuari t en l'any
      SELECT COUNT(*), C.PaisId
        INTO vComptador, vPaisId
        FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
       WHERE A.UsuariId = pUsuariId
         AND A.UsuariId = C.ciuId
         AND EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pDataAct)
    GROUP BY C.PaisId;

    -- Si no n'hi ha cap, restem estadstiques 
    IF (vComptador <= 0) THEN     
    
      -- Estadstica 1 ---------------------------------------------------
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E1_USUSISTEMA
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
         
      IF (vQuantitat = 1) THEN
        -- Era l'nic usuari comptat, borrem registre
        DELETE TFC_BD.E1_USUSISTEMA
         WHERE PaisId = vPaisId
           AND NumAny = EXTRACT(YEAR FROM pDataAct);
      ELSE
        -- Restar un usuari 
        UPDATE TFC_BD.E1_USUSISTEMA
           SET Quantitat = Quantitat - 1
         WHERE PaisId = vPaisId
           AND NumAny = EXTRACT(YEAR FROM pDataAct);
      END IF;   
      
      -- Estadstica 1 Comunitria ---------------------------------------------
      SELECT Quantitat
        INTO vQuantitat
        FROM TFC_BD.E1C_USUSISTEMA
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
         
      IF (vQuantitat = 1) THEN
        -- Era l'nic usuari comptat, borrem registre
        DELETE TFC_BD.E1C_USUSISTEMA
         WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
      ELSE
        -- Restar un usuari 
        UPDATE TFC_BD.E1C_USUSISTEMA
           SET Quantitat = Quantitat - 1
         WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
      END IF;   
      
    END IF;

    -- Estadstica 2 ---------------------------------------------------
    SELECT Quantitat
      INTO vQuantitat
      FROM TFC_BD.E2_CALORIESUSU
     WHERE PaisId = vPaisId
       AND NumAny = EXTRACT(YEAR FROM pDataAct);
       
    IF (vQuantitat = pNombreCalories) THEN
      -- Era l'nic usuari comptat, borrem registre
      DELETE TFC_BD.E2_CALORIESUSU
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    ELSE
      -- Resta de calories 
      UPDATE TFC_BD.E2_CALORIESUSU
         SET Quantitat = Quantitat - pNombreCalories
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;   
    
    -- Estadstica 2 Comunitria ---------------------------------------------
    SELECT Quantitat
      INTO vQuantitat
      FROM TFC_BD.E2C_CALORIESUSU
     WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
       
    IF (vQuantitat = pNombreCalories) THEN
      -- Era l'nic usuari comptat, borrem registre
      DELETE TFC_BD.E2C_CALORIESUSU
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    ELSE
      -- Resta de calories 
      UPDATE TFC_BD.E2C_CALORIESUSU
         SET Quantitat = Quantitat - pNombreCalories
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;   

    -- Estadstica 3 ---------------------------------------------------
    SELECT Quantitat
      INTO vQuantitat
      FROM TFC_BD.E3_PASSESUSU
     WHERE PaisId = vPaisId
       AND NumAny = EXTRACT(YEAR FROM pDataAct);
       
    IF (vQuantitat = pNombrePasses) THEN
      -- Era l'nic usuari comptat, borrem registre
      DELETE TFC_BD.E3_PASSESUSU
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    ELSE
      -- Resta de passes 
      UPDATE TFC_BD.E3_PASSESUSU
         SET Quantitat = Quantitat - pNombrePasses
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;   
    
    -- Estadstica 3 Comunitaria ---------------------------------------------
    SELECT Quantitat
      INTO vQuantitat
      FROM TFC_BD.E3C_PASSESUSU
     WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
       
    IF (vQuantitat = pNombrePasses) THEN
      -- Era l'nic usuari comptat, borrem registre
      DELETE TFC_BD.E3C_PASSESUSU
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    ELSE
      -- Resta de passes 
      UPDATE TFC_BD.E3C_PASSESUSU
         SET Quantitat = Quantitat - pNombrePasses
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;   

    -- Estadstica 4 ---------------------------------------------------
    SELECT Quantitat
      INTO vQuantitat
      FROM TFC_BD.E4_HORESPOCAACTUSU
     WHERE PaisId = vPaisId
       AND NumAny = EXTRACT(YEAR FROM pDataAct);
       
    IF (vQuantitat = pNombreHoresInactives) THEN
      -- Era l'nic usuari comptat, borrem registre
      DELETE TFC_BD.E4_HORESPOCAACTUSU
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    ELSE
      -- Resta d'Hores Inactives 
      UPDATE TFC_BD.E4_HORESPOCAACTUSU
         SET Quantitat = Quantitat - pNombreHoresInactives
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;   
    
    -- Estadstica 4 COmuntria ----------------------------------------------
    SELECT Quantitat
      INTO vQuantitat
      FROM TFC_BD.E4C_HORESPOCAACTUSU
     WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
       
    IF (vQuantitat = pNombreHoresInactives) THEN
      -- Era l'nic usuari comptat, borrem registre
      DELETE TFC_BD.E4C_HORESPOCAACTUSU
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    ELSE
      -- Resta d'Hores Inactives 
      UPDATE TFC_BD.E4C_HORESPOCAACTUSU
         SET Quantitat = Quantitat - pNombreHoresInactives
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;   

    -- Estadistica 9 -----------------------------------------------------------
      SELECT COUNT(*), A.USUARIID
        INTO vComptador, vCiutadaId
        FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
       WHERE EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pDataAct)
         AND C.PAISID = vPaisId     
         AND A.USUARIID = C.CIUID
    GROUP BY A.USUARIID, EXTRACT(YEAR FROM A.DataAct)
      HAVING SUM(A.NombreCalories) = 
             (SELECT MAX(SUM(A.NombreCalories))
                FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
               WHERE EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pDataAct)
                 AND C.PAISID = vPaisId     
                 AND A.USUARIID = C.CIUID
            GROUP BY A.USUARIID, EXTRACT(YEAR FROM A.DataAct));     
            
    IF vComptador <= 0 THEN
      DELETE TFC_BD.E9_USUMESCALORIES
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    ELSE
      UPDATE TFC_BD.E9_USUMESCALORIES
         SET UsuariId = vCiutadaId
       WHERE PaisId = vPaisId
         AND NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;      
      
    -- Estadistica 9 Comunitari ------------------------------------------------
      SELECT COUNT(*), A.USUARIID
        INTO vComptador, vCiutadaId
        FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
       WHERE EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pDataAct)
         AND A.USUARIID = C.CIUID
    GROUP BY A.USUARIID, EXTRACT(YEAR FROM A.DataAct)
      HAVING SUM(A.NombreCalories) = 
             (SELECT MAX(SUM(A.NombreCalories))
                FROM TFC_BD.ACTIVITAT A, TFC_BD.CIUTADA C
               WHERE EXTRACT(YEAR FROM A.DataAct) = EXTRACT(YEAR FROM pDataAct)
                 AND A.USUARIID = C.CIUID
            GROUP BY A.USUARIID, EXTRACT(YEAR FROM A.DataAct));     
   
    IF vComptador <= 0 THEN
      DELETE TFC_BD.E9C_USUMESCALORIES
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    ELSE
      UPDATE TFC_BD.E9C_USUMESCALORIES
         SET UsuariId = vCiutadaId
       WHERE NumAny = EXTRACT(YEAR FROM pDataAct);
    END IF;
     
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    COMMIT;

  EXCEPTION  
    WHEN OTHERS THEN
      ROLLBACK;
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);


  END DEL_EST_ACTIVITAT;
  
END PQ_ACT_ESTADISTIQUES;
/

/*******************************************************************************
    PAQUET PER A CONSULTES D'ESTADSTIQUES
*******************************************************************************/

CREATE OR REPLACE PACKAGE TFC_BD.PQ_CON_ESTADISTIQUES AS

  /* Mostrar per consola la dada d'una estadstica */
  PROCEDURE MOSTRA_ESTADISTICA(
    pTitol      IN VARCHAR2,
    pAny        IN NUMBER,
    pCodiPais   IN VARCHAR2, 
    pNomPais    IN VARCHAR2, 
    pTitolQuant IN VARCHAR2, 
	  pQuantitat  IN NUMBER
  );
  
  /*  Consulta Estadstica Ciutadans que han fet servir el sistema */
  PROCEDURE CONSULTA_EST1 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

  /* Consulta Estadstica Calories gastades */
  PROCEDURE CONSULTA_EST2 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

  /*  Consulta Estadstica passes donades */
  PROCEDURE CONSULTA_EST3 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

  /*  Consulta Estadstica hores poca activitat */
  PROCEDURE CONSULTA_EST4 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

  /*  Consulta Estadstica Ciutadans amb malaltia cardaca diagnosticada */
  PROCEDURE CONSULTA_EST5 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

  /*  Consulta Estadstica Ciutadans amb malaltia cancergena diagnosticada */
  PROCEDURE CONSULTA_EST6 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

  /*  Consulta Estadstica Ciutadans amb malaltia degenerativa s. nervis diagnosticada */
  PROCEDURE CONSULTA_EST7 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

  /*  Consulta Estadstica suma de malalties lleus */
  PROCEDURE CONSULTA_EST8 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

  /*  Consulta Estadstica persona amb ms calories gastades */
  PROCEDURE CONSULTA_EST9 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

  /*  Consulta Estadstica Ciutadans que han fet servir el sistema */
  PROCEDURE CONSULTA_ESTC1 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

  /*  Consulta Estadstica Calories gastades */
  PROCEDURE CONSULTA_ESTC2 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

  /*  Consulta Estadstica passes donades */
  PROCEDURE CONSULTA_ESTC3 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

  /*  Consulta Estadstica hores poca activitat */
  PROCEDURE CONSULTA_ESTC4 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

  /*  Consulta Estadstica Ciutadans amb malaltia cardaca diagnosticada */
  PROCEDURE CONSULTA_ESTC5 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

  /*  Consulta Estadstica Ciutadans amb malaltia cancergena diagnosticada */
  PROCEDURE CONSULTA_ESTC6 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

  /*  Consulta Estadstica Ciutadans amb malaltia degenerativa s. nervis diagnosticada */
  PROCEDURE CONSULTA_ESTC7 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

  /*  Consulta Estadstica suma de malalties lleus */
  PROCEDURE CONSULTA_ESTC8 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

  /*  Consulta Estadstica persona amb ms calories gastades */
  PROCEDURE CONSULTA_ESTC9 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	);

END PQ_CON_ESTADISTIQUES;
/

CREATE OR REPLACE PACKAGE BODY TFC_BD.PQ_CON_ESTADISTIQUES IS

    -- Variables de treball per a guardar moviments al registre d'activitat LOG_S
    l_NomUsuari        VARCHAR2(12);
    l_NomProcediment   VARCHAR2(60);
    l_ParEntrada       VARCHAR2(400);
    l_ParSortida       VARCHAR2(400);
    l_Resultat         VARCHAR2(300);
    l_DataHora         DATE;
    
  /* Mostrar per consola la dada d'una estadstica */
  PROCEDURE MOSTRA_ESTADISTICA(
    pTitol      IN VARCHAR2,
    pAny        IN NUMBER,
    pCodiPais   IN VARCHAR2, 
    pNomPais    IN VARCHAR2, 
    pTitolQuant IN VARCHAR2, 
	  pQuantitat  IN NUMBER
  ) IS

  BEGIN
    -- Capalera estandard de les consultes
  	IF (pCodiPais IS NULL OR pCodiPais = '') THEN
      PQ_UTILITATS.CAPCALERA_CONSULTA(pTitol, 'De l''Any : ' || pAny);  
  	ELSE
      PQ_UTILITATS.CAPCALERA_CONSULTA(pTitol, 
                                      'Del Pas : ' || pCodiPais || ' - ' ||
                                      pNomPais ||
                                      '  l''Any : ' || pAny);  
	  END IF;
            
    DBMS_OUTPUT.PUT_LINE(LPAD(pTitolQuant, 18));
    DBMS_OUTPUT.PUT_LINE('------------------');
    DBMS_OUTPUT.PUT_LINE(LPAD(TO_CHAR(pQuantitat,'9G999G999G990'),18));
    DBMS_OUTPUT.PUT_LINE('');
    DBMS_OUTPUT.PUT_LINE('');
  
  END MOSTRA_ESTADISTICA;
 
 
  /******************************************************************************
   NOM         : CONSULTA_EST1
   
   PROPSIT    : Consulta Estadstica Ciutadans que han fet servir el sistema.
                                
   PARMETRES  :
      ENTRADA  :  pCodiPais  : Codi del pas sobre el que es vol consultar
                  pAny       : Any que es vol consultar  

      SORTIDA  :  oRSP       : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 1' 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 1' 
                 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 1'
                 'ERROR: No s''han trobat dades estadstiques [R4] 1'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_EST1 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSensePais   EXCEPTION;
    eSenseAny    EXCEPTION;
    ePaisNoExist EXCEPTION;
    
    vPaisId      NUMBER;   -- Identificador del Pas del que es fa la consulta
    vCodiPais    VARCHAR2(3);
    vNomPais     VARCHAR2(100);
    vQuantitat   NUMBER;
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_EST1';
    l_ParEntrada     := 'pCodiPais: '||pCodiPais||
                        ', pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] 1 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pCodiPais IS NULL OR pCodiPais = '') THEN 
      RAISE eSensePais; 
    END IF;

    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    -- Revisem que el Pas existeixi a la base, i a ms saber-ne el seu identificador
    vPaisId := PQ_UTILITATS.DONA_PAIS_ID(UPPER(pCodiPais));
    IF (vPaisId <= 0) THEN
      RAISE ePaisNoExist;
    END IF;

    -- Llista per pantalla de les dades de l'estadstica recollides
    SELECT P.Codi, P.Nom, E.Quantitat
      INTO vCodiPais, vNomPais, vQuantitat
      FROM TFC_BD.PAIS P, TFC_BD.E1_USUSISTEMA E
     WHERE P.paiId  = vPaisId
       AND P.paiId  = E.PaisId
       AND E.NumAny = pAny;

    -- Llista estandard de les estadstiques
    PQ_CON_ESTADISTIQUES.MOSTRA_ESTADISTICA
      ('ESTADISTICA CIUTADANS QUE HAN USAT EL SISTEMA ([R4] 1)', pAny,
	     vCodiPais, vNomPais, 'N Ciutadans', vQuantitat);  
									
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION
  
    WHEN eSensePais THEN
      l_Resultat := 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 1'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 1'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN ePaisNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 1'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] 1';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_EST1;

  /******************************************************************************
   NOM         : CONSULTA_EST2
   
   PROPSIT    : Consulta Estadstica Calories gastades.
                                
   PARMETRES  :
      ENTRADA  :  pCodiPais  : Codi del pas sobre el que es vol consultar
                  pAny       : Any que es vol consultar  

      SORTIDA  :  oRSP       : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 2' 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 2' 
                 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 2'
                 'ERROR: No s''han trobat dades estadstiques [R4] 2'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_EST2 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSensePais   EXCEPTION;
    eSenseAny    EXCEPTION;
    ePaisNoExist EXCEPTION;
    
    vPaisId      NUMBER;   -- Identificador del Pas del que es fa la consulta
    vCodiPais    VARCHAR2(3);
    vNomPais     VARCHAR2(100);
    vQuantitat   NUMBER;
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_EST2';
    l_ParEntrada     := 'pCodiPais: '||pCodiPais||
                        ', pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] 2 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pCodiPais IS NULL OR pCodiPais = '') THEN 
      RAISE eSensePais; 
    END IF;

    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    -- Revisem que el Pas existeixi a la base, i a ms saber-ne el seu identificador
    vPaisId := PQ_UTILITATS.DONA_PAIS_ID(UPPER(pCodiPais));
    IF (vPaisId <= 0) THEN
      RAISE ePaisNoExist;
    END IF;
  
    -- Llista per pantalla de les dades de l'estadstica recollides
    SELECT P.Codi, P.Nom, E.Quantitat
      INTO vCodiPais, vNomPais, vQuantitat
      FROM TFC_BD.PAIS P, TFC_BD.E2_CALORIESUSU E
     WHERE P.paiId  = vPaisId
       AND P.paiId  = E.PaisId
       AND E.NumAny = pAny;

    -- Llista estandard de les estadstiques
    PQ_CON_ESTADISTIQUES.MOSTRA_ESTADISTICA
      ('ESTADISTICA CALORIES GASTADES PELS CIUTADANS ([R4] 2)', pAny,
	     vCodiPais, vNomPais, 'Suma Calories', vQuantitat);  
									
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION
  
    WHEN eSensePais THEN
      l_Resultat := 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 2'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 2'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN ePaisNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 2'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] 2';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_EST2;

  /******************************************************************************
   NOM         : CONSULTA_EST3
   
   PROPSIT    : Consulta Estadstica passes donades.
                                
   PARMETRES  :
      ENTRADA  :  pCodiPais  : Codi del pas sobre el que es vol consultar
                  pAny       : Any que es vol consultar  

      SORTIDA  :  oRSP       : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 3' 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 3' 
                 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 3'
                 'ERROR: No s''han trobat dades estadstiques [R4] 3'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_EST3 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSensePais   EXCEPTION;
    eSenseAny    EXCEPTION;
    ePaisNoExist EXCEPTION;
    
    vPaisId      NUMBER;   -- Identificador del Pas del que es fa la consulta
    vCodiPais    VARCHAR2(3);
    vNomPais     VARCHAR2(100);
    vQuantitat   NUMBER;
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_EST3';
    l_ParEntrada     := 'pCodiPais: '||pCodiPais||
                        ', pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] 3 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pCodiPais IS NULL OR pCodiPais = '') THEN 
      RAISE eSensePais; 
    END IF;

    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    -- Revisem que el Pas existeixi a la base, i a ms saber-ne el seu identificador
    vPaisId := PQ_UTILITATS.DONA_PAIS_ID(UPPER(pCodiPais));
    IF (vPaisId <= 0) THEN
      RAISE ePaisNoExist;
    END IF;
  
    -- Llista per pantalla de les dades de l'estadstica recollides
    SELECT P.Codi, P.Nom, E.Quantitat
      INTO vCodiPais, vNomPais, vQuantitat
      FROM TFC_BD.PAIS P, TFC_BD.E3_PASSESUSU E
     WHERE P.paiId  = vPaisId
       AND P.paiId  = E.PaisId
       AND E.NumAny = pAny;

    -- Llista estandard de les estadstiques
    PQ_CON_ESTADISTIQUES.MOSTRA_ESTADISTICA
      ('ESTADISTICA PASSES FETES PELS CIUTADANS ([R4] 3)', pAny,
	     vCodiPais, vNomPais, 'Suma Passes', vQuantitat);  
									
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION
  
    WHEN eSensePais THEN
      l_Resultat := 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 3'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 3'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN ePaisNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 3'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] 3';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_EST3;

  /******************************************************************************
   NOM         : CONSULTA_EST4
   
   PROPSIT    : Consulta Estadstica hores poca activitat.
                                
   PARMETRES  :
      ENTRADA  :  pCodiPais  : Codi del pas sobre el que es vol consultar
                  pAny       : Any que es vol consultar  

      SORTIDA  :  oRSP       : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 4' 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 4' 
                 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 4'
                 'ERROR: No s''han trobat dades estadstiques [R4] 4'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_EST4 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSensePais   EXCEPTION;
    eSenseAny    EXCEPTION;
    ePaisNoExist EXCEPTION;
    
    vPaisId      NUMBER;   -- Identificador del Pas del que es fa la consulta
    vCodiPais    VARCHAR2(3);
    vNomPais     VARCHAR2(100);
    vQuantitat   NUMBER;
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_EST4';
    l_ParEntrada     := 'pCodiPais: '||pCodiPais||
                        ', pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] 4 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pCodiPais IS NULL OR pCodiPais = '') THEN 
      RAISE eSensePais; 
    END IF;

    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    -- Revisem que el Pas existeixi a la base, i a ms saber-ne el seu identificador
    vPaisId := PQ_UTILITATS.DONA_PAIS_ID(UPPER(pCodiPais));
    IF (vPaisId <= 0) THEN
      RAISE ePaisNoExist;
    END IF;
  
    -- Llista per pantalla de les dades de l'estadstica recollides
    SELECT P.Codi, P.Nom, E.Quantitat
      INTO vCodiPais, vNomPais, vQuantitat
      FROM TFC_BD.PAIS P, TFC_BD.E4_HORESPOCAACTUSU E
     WHERE P.paiId  = vPaisId
       AND P.paiId  = E.PaisId
       AND E.NumAny = pAny;

    -- Llista estandard de les estadstiques
    PQ_CON_ESTADISTIQUES.MOSTRA_ESTADISTICA
      ('ESTADISTICA HORES POCA ACTIVITAT DELS CIUTADANS ([R4] 4)', pAny,
	     vCodiPais, vNomPais, 'Suma Hores', vQuantitat);  
									
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION
  
    WHEN eSensePais THEN
      l_Resultat := 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 4'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 4'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN ePaisNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 4'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] 4';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_EST4;

  /******************************************************************************
   NOM         : CONSULTA_EST5
   
   PROPSIT    : Consulta Estadstica Ciutadans amb malaltia cardaca diagnosticada.
                                
   PARMETRES  :
      ENTRADA  :  pCodiPais  : Codi del pas sobre el que es vol consultar
                  pAny       : Any que es vol consultar  

      SORTIDA  :  oRSP       : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 5' 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 5' 
                 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 5'
                 'ERROR: No s''han trobat dades estadstiques [R4] 5'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_EST5 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSensePais   EXCEPTION;
    eSenseAny    EXCEPTION;
    ePaisNoExist EXCEPTION;
    
    vPaisId      NUMBER;   -- Identificador del Pas del que es fa la consulta
    vCodiPais    VARCHAR2(3);
    vNomPais     VARCHAR2(100);
    vQuantitat   NUMBER;
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_EST5';
    l_ParEntrada     := 'pCodiPais: '||pCodiPais||
                        ', pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] 5 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pCodiPais IS NULL OR pCodiPais = '') THEN 
      RAISE eSensePais; 
    END IF;

    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    -- Revisem que el Pas existeixi a la base, i a ms saber-ne el seu identificador
    vPaisId := PQ_UTILITATS.DONA_PAIS_ID(UPPER(pCodiPais));
    IF (vPaisId <= 0) THEN
      RAISE ePaisNoExist;
    END IF;
  
    -- Llista per pantalla de les dades de l'estadstica recollides
    SELECT P.Codi, P.Nom, E.Quantitat
      INTO vCodiPais, vNomPais, vQuantitat
      FROM TFC_BD.PAIS P, TFC_BD.E5_USUCARDIACA E
     WHERE P.paiId  = vPaisId
       AND P.paiId  = E.PaisId
       AND E.NumAny = pAny;

    -- Llista estandard de les estadstiques
    PQ_CON_ESTADISTIQUES.MOSTRA_ESTADISTICA
      ('ESTADISTICA CIUTADANS AMB DIAGNSTIC DE MALALTIA CARDACA ([R4] 5)', pAny,
	     vCodiPais, vNomPais, 'N Ciutadans', vQuantitat);  
									
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION
  
    WHEN eSensePais THEN
      l_Resultat := 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 5'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 5'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN ePaisNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 5'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] 5';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_EST5;

  /******************************************************************************
   NOM         : CONSULTA_EST6
   
   PROPSIT    : Consulta Estadstica Ciutadans amb malaltia cancergena diagnosticada.
                                
   PARMETRES  :
      ENTRADA  :  pCodiPais  : Codi del pas sobre el que es vol consultar
                  pAny       : Any que es vol consultar  

      SORTIDA  :  oRSP       : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 6' 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 6' 
                 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 6'
                 'ERROR: No s''han trobat dades estadstiques [R4] 6'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_EST6 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSensePais   EXCEPTION;
    eSenseAny    EXCEPTION;
    ePaisNoExist EXCEPTION;
    
    vPaisId      NUMBER;   -- Identificador del Pas del que es fa la consulta
    vCodiPais    VARCHAR2(3);
    vNomPais     VARCHAR2(100);
    vQuantitat   NUMBER;
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_EST6';
    l_ParEntrada     := 'pCodiPais: '||pCodiPais||
                        ', pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] 6 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pCodiPais IS NULL OR pCodiPais = '') THEN 
      RAISE eSensePais; 
    END IF;

    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    -- Revisem que el Pas existeixi a la base, i a ms saber-ne el seu identificador
    vPaisId := PQ_UTILITATS.DONA_PAIS_ID(UPPER(pCodiPais));
    IF (vPaisId <= 0) THEN
      RAISE ePaisNoExist;
    END IF;
  
    -- Llista per pantalla de les dades de l'estadstica recollides
    SELECT P.Codi, P.Nom, E.Quantitat
      INTO vCodiPais, vNomPais, vQuantitat
      FROM TFC_BD.PAIS P, TFC_BD.E6_USUCANCERIGENA E
     WHERE P.paiId  = vPaisId
       AND P.paiId  = E.PaisId
       AND E.NumAny = pAny;

    -- Llista estandard de les estadstiques
    PQ_CON_ESTADISTIQUES.MOSTRA_ESTADISTICA
      ('ESTADISTICA CIUTADANS AMB DIAGNSTIC DE MALALTIA CANCERGENA ([R4] 6)', pAny,
	     vCodiPais, vNomPais, 'N Ciutadans', vQuantitat);  
									
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION
  
    WHEN eSensePais THEN
      l_Resultat := 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 6'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 6'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN ePaisNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 6'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] 6';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_EST6;

  /******************************************************************************
   NOM         : CONSULTA_EST7
   
   PROPSIT    : Consulta Estadstica Ciutadans amb malaltia degenerativa s. nervis diagnosticada.
                                
   PARMETRES  :
      ENTRADA  :  pCodiPais  : Codi del pas sobre el que es vol consultar
                  pAny       : Any que es vol consultar  

      SORTIDA  :  oRSP       : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 7' 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 7' 
                 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 7'
                 'ERROR: No s''han trobat dades estadstiques [R4] 7'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_EST7 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSensePais   EXCEPTION;
    eSenseAny    EXCEPTION;
    ePaisNoExist EXCEPTION;
    
    vPaisId      NUMBER;   -- Identificador del Pas del que es fa la consulta
    vCodiPais    VARCHAR2(3);
    vNomPais     VARCHAR2(100);
    vQuantitat   NUMBER;
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_EST7';
    l_ParEntrada     := 'pCodiPais: '||pCodiPais||
                        ', pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] 7 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pCodiPais IS NULL OR pCodiPais = '') THEN 
      RAISE eSensePais; 
    END IF;

    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    -- Revisem que el Pas existeixi a la base, i a ms saber-ne el seu identificador
    vPaisId := PQ_UTILITATS.DONA_PAIS_ID(UPPER(pCodiPais));
    IF (vPaisId <= 0) THEN
      RAISE ePaisNoExist;
    END IF;
  
    -- Llista per pantalla de les dades de l'estadstica recollides
    SELECT P.Codi, P.Nom, E.Quantitat
      INTO vCodiPais, vNomPais, vQuantitat
      FROM TFC_BD.PAIS P, TFC_BD.E7_USUDEGENERATIVA E
     WHERE P.paiId  = vPaisId
       AND P.paiId  = E.PaisId
       AND E.NumAny = pAny;

    -- Llista estandard de les estadstiques
    PQ_CON_ESTADISTIQUES.MOSTRA_ESTADISTICA
      ('ESTADISTICA CIUTADANS AMB DIAGNSTIC DE MALALTIA NEUROL. DEGENERATIVA ([R4] 7)', pAny,
	     vCodiPais, vNomPais, 'N Ciutadans', vQuantitat);  
									
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION
  
    WHEN eSensePais THEN
      l_Resultat := 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 7'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 7'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN ePaisNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 7'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] 7';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_EST7;

  /******************************************************************************
   NOM         : CONSULTA_EST8
   
   PROPSIT    : Consulta Estadstica suma de malalties lleus.
                                
   PARMETRES  :
      ENTRADA  :  pCodiPais  : Codi del pas sobre el que es vol consultar
                  pAny       : Any que es vol consultar  

      SORTIDA  :  oRSP       : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 8' 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 8' 
                 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 8'
                 'ERROR: No s''han trobat dades estadstiques [R4] 8'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_EST8 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSensePais   EXCEPTION;
    eSenseAny    EXCEPTION;
    ePaisNoExist EXCEPTION;
    
    vPaisId      NUMBER;   -- Identificador del Pas del que es fa la consulta
    vCodiPais    VARCHAR2(3);
    vNomPais     VARCHAR2(100);
    vQuantitat   NUMBER;
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_EST8';
    l_ParEntrada     := 'pCodiPais: '||pCodiPais||
                        ', pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] 8 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pCodiPais IS NULL OR pCodiPais = '') THEN 
      RAISE eSensePais; 
    END IF;

    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    -- Revisem que el Pas existeixi a la base, i a ms saber-ne el seu identificador
    vPaisId := PQ_UTILITATS.DONA_PAIS_ID(UPPER(pCodiPais));
    IF (vPaisId <= 0) THEN
      RAISE ePaisNoExist;
    END IF;
  
    -- Llista per pantalla de les dades de l'estadstica recollides
    SELECT P.Codi, P.Nom, E.Quantitat
      INTO vCodiPais, vNomPais, vQuantitat
      FROM TFC_BD.PAIS P, TFC_BD.E8_MALALTIESLLEUS E
     WHERE P.paiId  = vPaisId
       AND P.paiId  = E.PaisId
       AND E.NumAny = pAny;

    -- Llista estandard de les estadstiques
    PQ_CON_ESTADISTIQUES.MOSTRA_ESTADISTICA
      ('ESTADISTICA DE DIAGNSTICS DE MALALTIES LLEUS ([R4] 8)', pAny,
	     vCodiPais, vNomPais, 'N Diagnstics', vQuantitat);  
									
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION
  
    WHEN eSensePais THEN
      l_Resultat := 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 8'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 8'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN ePaisNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 8'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] 8';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_EST8;

  /******************************************************************************
   NOM         : CONSULTA_EST9
   
   PROPSIT    : Consulta Estadstica persona amb ms calories gastades.
                                
   PARMETRES  :
      ENTRADA  :  pCodiPais  : Codi del pas sobre el que es vol consultar
                  pAny       : Any que es vol consultar  

      SORTIDA  :  oRSP       : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 9' 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 9' 
                 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 9'
                 'ERROR: No s''han trobat dades estadstiques [R4] 9'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_EST9 (
    pCodiPais  IN PAIS.Codi%TYPE,
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSensePais   EXCEPTION;
    eSenseAny    EXCEPTION;
    ePaisNoExist EXCEPTION;
    
    vPaisId         NUMBER;   -- Identificador del Pas del que es fa la consulta
    vCodiPais       VARCHAR2(3);
    vNomPais        VARCHAR2(100);
    vCognomsCiutada VARCHAR2(100);
    vNomCiutada     VARCHAR2(100);
    vSumaCalories   NUMBER;
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_EST9';
    l_ParEntrada     := 'pCodiPais: '||pCodiPais||
                        ', pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] 9 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pCodiPais IS NULL OR pCodiPais = '') THEN 
      RAISE eSensePais; 
    END IF;

    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    -- Revisem que el Pas existeixi a la base, i a ms saber-ne el seu identificador
    vPaisId := PQ_UTILITATS.DONA_PAIS_ID(UPPER(pCodiPais));
    IF (vPaisId <= 0) THEN
      RAISE ePaisNoExist;
    END IF;
  
    -- Llista per pantalla de les dades de l'estadstica recollides
      SELECT P.Codi, P.Nom, C.Nom, C.Cognoms, sum(A.NombreCalories)
        INTO vCodiPais, vNomPais, vNomCiutada, vCognomsCiutada, vSumaCalories
        FROM TFC_BD.PAIS P, TFC_BD.CIUTADA C, TFC_BD.ACTIVITAT A, 
             TFC_BD.E9_USUMESCALORIES E
       WHERE P.paiId                      = vPaisId
         AND P.paiId                      = E.PaisId
         AND E.NumAny                     = pAny
         AND E.UsuariId                   = C.ciuId
         AND A.UsuariId                   = C.CiuId
         AND EXTRACT(YEAR FROM A.DataAct) = pAny
    GROUP BY P.Codi, P.Nom, C.Nom, C.Cognoms;
       
       
    -- Capalera estandard de les consultes
    PQ_UTILITATS.CAPCALERA_CONSULTA('CIUTAD AMB MS CONSUM DE CALORIES ([R4] 9)', 
                                    'Del Pas : ' || vCodiPais || ' - ' ||
                                    vNomPais ||
                                    '  l''Any : ' || pAny);  
            
    DBMS_OUTPUT.PUT_LINE(RPAD('Cognoms i Nom',30) ||
                         LPAD('Suma Calories', 18));
    DBMS_OUTPUT.PUT_LINE(RPAD('-',29, '-') || ' ' ||
                         '------------------');
    DBMS_OUTPUT.PUT_LINE(RPAD(vCognomsCiutada || ', ' || vNomCiutada,30) ||
                         LPAD(TO_CHAR(vSumaCalories,'9G999G999G990'),18));
    DBMS_OUTPUT.PUT_LINE('');
    DBMS_OUTPUT.PUT_LINE('');


    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION
  
    WHEN eSensePais THEN
      l_Resultat := 'ERROR: SENSE CODI DE PAS A CONSULTAR [R4] 9'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] 9'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN ePaisNoExist THEN
      l_Resultat := 'ERROR: No s''ha trobat el Pas que es vol consultar [R4] 9'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] 9';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_EST9;

  /******************************************************************************
   NOM         : CONSULTA_ESTC1
   
   PROPSIT    : Consulta Estadstica Ciutadans que han fet servir el sistema.
                                
   PARMETRES  :
      ENTRADA  :  pAny : Any que es vol consultar  

      SORTIDA  :  oRSP : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C1' 
                 'ERROR: No s''han trobat dades estadstiques [R4] C1'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_ESTC1 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseAny EXCEPTION;
    
    vQuantitat NUMBER;  
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_ESTC1';
    l_ParEntrada     := 'pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] C1 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    -- Llista per pantalla de les dades de l'estadstica recollides
    SELECT Quantitat
      INTO vQuantitat
      FROM TFC_BD.E1C_USUSISTEMA
     WHERE NumAny = pAny;

    -- Llista estandard de les estadstiques
    PQ_CON_ESTADISTIQUES.MOSTRA_ESTADISTICA
      ('ESTADISTICA CIUTADANS U.E. QUE HAN USAT EL SISTEMA ([R4] C1)', pAny,
	     NULL, NULL, 'N Ciutadans', vQuantitat);  
									
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION
    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C1'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] C1';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_ESTC1;

  /******************************************************************************
   NOM         : CONSULTA_ESTC2
   
   PROPSIT    : Consulta Estadstica Calories gastades.
                                
   PARMETRES  :
      ENTRADA  :  pAny : Any que es vol consultar  

      SORTIDA  :  oRSP : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C2' 
                 'ERROR: No s''han trobat dades estadstiques [R4] C2'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_ESTC2 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseAny EXCEPTION;
    
    vQuantitat NUMBER;  
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_ESTC2';
    l_ParEntrada     := 'pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] C2 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;
  
    -- Llista per pantalla de les dades de l'estadstica recollides
    SELECT Quantitat
      INTO vQuantitat
      FROM TFC_BD.E2C_CALORIESUSU
     WHERE NumAny = pAny;

    -- Llista estandard de les estadstiques
    PQ_CON_ESTADISTIQUES.MOSTRA_ESTADISTICA
      ('ESTADISTICA CALORIES GASTADES PELS CIUTADANS U.E. ([R4] C2)', pAny,
	     NULL, NULL, 'Suma Calories', vQuantitat);  
									
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION

    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C2'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] C2';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_ESTC2;

  /******************************************************************************
   NOM         : CONSULTA_ESTC3
   
   PROPSIT    : Consulta Estadstica passes donades.
                                
   PARMETRES  :
      ENTRADA  :  pAny : Any que es vol consultar  

      SORTIDA  :  oRSP : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C3' 
                 'ERROR: No s''han trobat dades estadstiques [R4] C3'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_ESTC3 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseAny EXCEPTION;
    
    vQuantitat NUMBER;  
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_ESTC3';
    l_ParEntrada     := 'pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] C3 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;
  
    -- Llista per pantalla de les dades de l'estadstica recollides
    SELECT Quantitat
      INTO vQuantitat
      FROM TFC_BD.E3C_PASSESUSU
     WHERE NumAny = pAny;

    -- Llista estandard de les estadstiques
    PQ_CON_ESTADISTIQUES.MOSTRA_ESTADISTICA
      ('ESTADISTICA PASSES FETES PELS CIUTADANS U.E. ([R4] C3)', pAny,
	     NULL, NULL, 'Suma Passes', vQuantitat);  
									
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION
  
    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C3'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] C3';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_ESTC3;

  /******************************************************************************
   NOM         : CONSULTA_ESTC4
   
   PROPSIT    : Consulta Estadstica hores poca activitat.
                                
   PARMETRES  :
      ENTRADA  :  pAny : Any que es vol consultar  

      SORTIDA  :  oRSP : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C4' 
                 'ERROR: No s''han trobat dades estadstiques [R4] C4'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_ESTC4 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseAny EXCEPTION;
    
    vQuantitat NUMBER;  
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_ESTC4';
    l_ParEntrada     := 'pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] C4 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    -- Llista per pantalla de les dades de l'estadstica recollides
    SELECT Quantitat
      INTO vQuantitat
      FROM TFC_BD.E4C_HORESPOCAACTUSU
     WHERE NumAny = pAny;

    -- Llista estandard de les estadstiques
    PQ_CON_ESTADISTIQUES.MOSTRA_ESTADISTICA
      ('ESTADISTICA HORES POCA ACTIVITAT DELS CIUTADANS U.E. ([R4] C4)', pAny,
	     NULL, NULL, 'Suma Hores', vQuantitat);  
									
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION
  
    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C4'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] C4';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_ESTC4;

  /******************************************************************************
   NOM         : CONSULTA_ESTC5
   
   PROPSIT    : Consulta Estadstica Ciutadans amb malaltia cardaca diagnosticada.
                                
   PARMETRES  :
      ENTRADA  :  pAny : Any que es vol consultar  

      SORTIDA  :  oRSP : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C5' 
                 'ERROR: No s''han trobat dades estadstiques [R4] C5'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_ESTC5 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseAny EXCEPTION;
    
    vQuantitat NUMBER;  
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_ESTC5';
    l_ParEntrada     := 'pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] C5 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    -- Llista per pantalla de les dades de l'estadstica recollides
    SELECT Quantitat
      INTO vQuantitat
      FROM TFC_BD.E5C_USUCARDIACA
     WHERE NumAny = pAny;

    -- Llista estandard de les estadstiques
    PQ_CON_ESTADISTIQUES.MOSTRA_ESTADISTICA
      ('ESTADISTICA CIUTADANS AMB DIAGNSTIC DE MALALTIA CARDACA U.E. ([R4] C5)', pAny,
	     NULL, NULL, 'N Ciutadans', vQuantitat);  
									
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION

    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C5'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] C5';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_ESTC5;

  /******************************************************************************
   NOM         : CONSULTA_ESTC6
   
   PROPSIT    : Consulta Estadstica Ciutadans amb malaltia cancergena diagnosticada.
                                
   PARMETRES  :
      ENTRADA  :  pAny : Any que es vol consultar  

      SORTIDA  :  oRSP : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C6' 
                 'ERROR: No s''han trobat dades estadstiques [R4] C6'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_ESTC6 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseAny EXCEPTION;
    
    vQuantitat NUMBER;  
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_ESTC6';
    l_ParEntrada     := 'pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] C6 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    -- Llista per pantalla de les dades de l'estadstica recollides
    SELECT Quantitat
      INTO vQuantitat
      FROM TFC_BD.E6C_USUCANCERIGENA
     WHERE NumAny = pAny;

    -- Llista estandard de les estadstiques
    PQ_CON_ESTADISTIQUES.MOSTRA_ESTADISTICA
      ('ESTADISTICA CIUTADANS AMB DIAGNSTIC DE MALALTIA CANCERGENA DE LA U.E. ([R4] C6)', pAny,
	     NULL, NULL, 'N Ciutadans', vQuantitat);  
									
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION
  
    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C6'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] C6';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_ESTC6;

  /******************************************************************************
   NOM         : CONSULTA_ESTC7
   
   PROPSIT    : Consulta Estadstica Ciutadans amb malaltia degenerativa s. nervis diagnosticada.
                                
   PARMETRES  :
      ENTRADA  :  pAny : Any que es vol consultar  

      SORTIDA  :  oRSP : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C7' 
                 'ERROR: No s''han trobat dades estadstiques [R4] C7'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_ESTC7 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseAny EXCEPTION;
    
    vQuantitat NUMBER;  
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_ESTC7';
    l_ParEntrada     := 'pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] C7 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    -- Llista per pantalla de les dades de l'estadstica recollides
    SELECT Quantitat
      INTO vQuantitat
      FROM TFC_BD.E7C_USUDEGENERATIVA
     WHERE NumAny = pAny;

    -- Llista estandard de les estadstiques
    PQ_CON_ESTADISTIQUES.MOSTRA_ESTADISTICA
      ('ESTADISTICA CIUTADANS AMB DIAGNSTIC DE MALALTIA NEUROL. DEGENERATIVA DE LA U.E. ([R4] C7)', pAny,
	     NULL, NULL, 'N Ciutadans', vQuantitat);  
									
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION
  
    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C7'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] C7';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_ESTC7;

  /******************************************************************************
   NOM         : CONSULTA_ESTC8
   
   PROPSIT    : Consulta Estadstica suma de malalties lleus.
                                
   PARMETRES  :
      ENTRADA  :  pAny : Any que es vol consultar  

      SORTIDA  :  oRSP : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C8' 
                 'ERROR: No s''han trobat dades estadstiques [R4] C8'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_ESTC8 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseAny EXCEPTION;
    
    vQuantitat NUMBER;  
    
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_ESTC8';
    l_ParEntrada     := 'pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] C8 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    -- Llista per pantalla de les dades de l'estadstica recollides
    SELECT Quantitat
      INTO vQuantitat
      FROM TFC_BD.E8C_MALALTIESLLEUS
     WHERE NumAny = pAny;

    -- Llista estandard de les estadstiques
    PQ_CON_ESTADISTIQUES.MOSTRA_ESTADISTICA
      ('ESTADISTICA DE DIAGNSTICS DE MALALTIES LLEUS DE LA U.E. ([R4] C8)', pAny,
	     NULL, NULL, 'N Diagnstics', vQuantitat);  
									
    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION
  
    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C8'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] C8';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_ESTC8;

  /******************************************************************************
   NOM         : CONSULTA_ESTC9
   
   PROPSIT    : Consulta Estadstica persona amb ms calories gastades.
                                
   PARMETRES  :
      ENTRADA  :  pAny : Any que es vol consultar  

      SORTIDA  :  oRSP : Resposta del procediment            
		
   RETORNA     : 'OK' si ha tingut lloc la consulta amb dades a mostrar
                 Si no, algun dels missatges segents segons la causa : 
                 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C9' 
                 'ERROR: No s''han trobat dades estadstiques [R4] C9'
                 'ERROR: <NumError><DescError>' per les altres causes
   
   ******************************************************************************/

  PROCEDURE CONSULTA_ESTC9 (
    pAny       IN NUMBER,
    oRSP      OUT VARCHAR2
	) IS
  
    -- variables pel tractament d'excepcions
    eSenseAny    EXCEPTION;
    
    vCodiPais       VARCHAR2(3);
    vNomPais        VARCHAR2(100);
    vCognomsCiutada VARCHAR2(100);
    vNomCiutada     VARCHAR2(100);
    vSumaCalories   NUMBER;
  BEGIN
  
    -- Inicialitzacions
    l_NomUsuari      := USER;
    l_NomProcediment := 'CONSULTA_ESTC9';
    l_ParEntrada     := 'pAny: '||pAny;
    l_ParSortida     := ' ';               
    l_Resultat       := 'OK. CONSULTA ESTADISTICA [R4] C9 EFECTUADA';
    l_DataHora       := SYSDATE;
    
    oRSP := l_Resultat;
	
    -- Mirar validesa dels parmetres d'entrada
    IF (pAny IS NULL OR pAny = 0) THEN 
      RAISE eSenseAny; 
    END IF;

    -- Llista per pantalla de les dades de l'estadstica recollides
      SELECT P.Codi, P.Nom, C.Nom, C.Cognoms, sum(A.NombreCalories)
        INTO vCodiPais, vNomPais, vNomCiutada, vCognomsCiutada, vSumaCalories
        FROM TFC_BD.PAIS P, TFC_BD.CIUTADA C, TFC_BD.ACTIVITAT A, 
             TFC_BD.E9C_USUMESCALORIES E
       WHERE E.NumAny                     = pAny
         AND E.UsuariId                   = C.ciuId
         AND A.UsuariId                   = C.CiuId
         AND P.paiId                      = C.PaisId
         AND EXTRACT(YEAR FROM A.DataAct) = pAny
    GROUP BY P.Codi, P.Nom, C.Nom, C.Cognoms;
       
       
    -- Capalera estandard de les consultes
    PQ_UTILITATS.CAPCALERA_CONSULTA('CIUTAD AMB MS CONSUM DE CALORIES DE LA U.E. ([R4] C9)', 
                                    'De l''Any : ' || pAny);  
            
    DBMS_OUTPUT.PUT_LINE(RPAD('Pas',30) ||
                         RPAD('Cognoms i Nom',30) ||
                         LPAD('Suma Calories', 18));
    DBMS_OUTPUT.PUT_LINE(RPAD('-',29, '-') || ' ' ||
                         RPAD('-',29, '-') || ' ' ||
                         '------------------');
    DBMS_OUTPUT.PUT_LINE(RPAD(vCodiPais || ' - ' || vNomPais,30) ||
                         RPAD(vCognomsCiutada || ', ' || vNomCiutada,30) ||
                         LPAD(TO_CHAR(vSumaCalories,'9G999G999G990'),18));
    DBMS_OUTPUT.PUT_LINE('');
    DBMS_OUTPUT.PUT_LINE('');


    -- Execuci correcta del procediment. Gravar registre d'activitat
    PQ_UTILITATS.PRO_ALTALOG
      (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
    
  EXCEPTION
    WHEN eSenseAny THEN
      l_Resultat := 'ERROR: SENSE NMERO D''ANY A CONSULTAR [R4] C9'; 
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);
        
    WHEN NO_DATA_FOUND THEN
      l_Resultat := 'ERROR: No s''han trobat dades estadstiques [R4] C9';
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

    WHEN OTHERS THEN
      l_resultat := 'ERROR: [' || TO_CHAR(SQLCODE) || '] ' || SQLERRM;
      oRSP := l_Resultat;
      PQ_UTILITATS.PRO_ALTALOG
        (l_NomUsuari,l_NomProcediment,l_ParEntrada,l_ParSortida,l_Resultat,l_DataHora);

  END CONSULTA_ESTC9;

END PQ_CON_ESTADISTIQUES;
/

COMMIT;  
